@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -32,10 +32,13 @@
 #include "widgets/icons.h"
 #include "support/actioncollection.h"
 #include "support/action.h"
+#include "support/configuration.h"
 #include "models/mpdlibrarymodel.h"
+#include "mpd-interface/cuefile.h"
 #include <QScrollBar>
 #include <QFile>
 #include <QUrl>
+#include <QUrlQuery>
 #include <QMenu>
 #include <QTimer>
 #include <QCoreApplication>
@@ -45,6 +48,8 @@
 const QLatin1String AlbumView::constCacheDir("albums/");
 const QLatin1String AlbumView::constInfoExt(".html.gz");
 
+static const QLatin1String constScheme("cantata");
+
 static QString cacheFileName(const QString &artist, const QString &album, const QString &lang, bool createDir)
 {
     return Utils::cacheDir(AlbumView::constCacheDir, createDir)+Covers::encodeName(artist)+QLatin1String(" - ")+Covers::encodeName(album)+"."+lang+AlbumView::constInfoExt;
@@ -62,7 +67,10 @@ AlbumView::AlbumView(QWidget *p)
 {
     engine=ContextEngine::create(this);
     refreshAction = ActionCollection::get()->createAction("refreshalbum", tr("Refresh Album Information"), Icons::self()->refreshIcon);
+    fullWidthCoverAction = new Action(tr("Full Width Cover"), this);
+    fullWidthCoverAction->setCheckable(true);
     connect(refreshAction, SIGNAL(triggered()), this, SLOT(refresh()));
+    connect(fullWidthCoverAction, SIGNAL(toggled(bool)), this, SLOT(setScaleImage(bool)));
     connect(engine, SIGNAL(searchResult(QString,QString)), this, SLOT(searchResponse(QString,QString)));
     connect(Covers::self(), SIGNAL(cover(Song,QImage,QString)), SLOT(coverRetrieved(Song,QImage,QString)));
     connect(Covers::self(), SIGNAL(coverUpdated(Song,QImage,QString)), SLOT(coverUpdated(Song,QImage,QString)));
@@ -79,6 +87,12 @@ AlbumView::AlbumView(QWidget *p)
         connect(timer, SIGNAL(timeout()), this, SLOT(clearCache()));
         timer->start((int)((ArtistView::constCacheAge/2.0)*1000*24*60*60));
     }
+    fullWidthCoverAction->setChecked(Configuration(metaObject()->className()).get("fullWidthCover", false));
+}
+
+AlbumView::~AlbumView()
+{
+    Configuration(metaObject()->className()).set("fullWidthCover", fullWidthCoverAction->isChecked());
 }
 
 void AlbumView::showContextMenu(const QPoint &pos)
@@ -90,6 +104,7 @@ void AlbumView::showContextMenu(const QPoint &pos)
     } else {
         menu->addAction(refreshAction);
     }
+    menu->addAction(fullWidthCoverAction);
     menu->exec(text->mapToGlobal(pos));
     delete menu;
 }
@@ -99,7 +114,7 @@ void AlbumView::refresh()
     if (currentSong.isEmpty()) {
         return;
     }
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QFile::remove(cacheFileName(Covers::fixArtist(currentSong.albumArtist()), currentSong.album, engine->getPrefix(lang), false));
     }
     update(currentSong, true);
@@ -159,8 +174,10 @@ void AlbumView::update(const Song &song, bool force)
 
 void AlbumView::playSong(const QUrl &url)
 {
-    if (QLatin1String("cantata")==url.scheme()) {
+    if (url.scheme() == constScheme) {
         emit playSong(url.path().mid(1)); // Remove leading /
+    } else if (CueFile::isCue(url.toString())) {
+        emit playSong(url.toString());
     } else {
         QDesktopServices::openUrl(url);
     }
@@ -168,16 +185,45 @@ void AlbumView::playSong(const QUrl &url)
 
 void AlbumView::getTrackListing()
 {
+    if (currentSong.isNonMPD()) {
+        if (!pic.isEmpty()) {
+            updateDetails();
+        }
+        return;
+    }
+
     if (songs.isEmpty()) {
         songs=MpdLibraryModel::self()->getAlbumTracks(currentSong);
     }
 
     if (!songs.isEmpty()) {
         trackList=View::subHeader(tr("Tracks"))+QLatin1String("<p><table>");
-        foreach (const Song &s, songs) {
-            trackList+=QLatin1String("<tr><td align='right'>")+QString::number(s.track)+
-                       QLatin1String("</td><td><a href=\"cantata:///")+s.file+"\">"+
-                       (s.file==currentSong.file ? "<b>"+s.displayTitle()+"</b>" : s.displayTitle())+QLatin1String("</a></td></tr>");
+        for (const Song &s: songs) {
+            if (CueFile::isCue(s.file)) {
+                QUrl u(s.file);
+                QUrlQuery q(u);
+
+                q.addQueryItem("artist", s.artist);
+                q.addQueryItem("albumartist", s.albumartist);
+                q.addQueryItem("album", s.album);
+                q.addQueryItem("title", s.title);
+                q.addQueryItem("disc", QString::number(s.disc));
+                q.addQueryItem("track", QString::number(s.track));
+                q.addQueryItem("time", QString::number(s.time));
+                q.addQueryItem("year", QString::number(s.year));
+                q.addQueryItem("origYear", QString::number(s.origYear));
+                u.setQuery(q);
+
+                trackList+=QLatin1String("<tr><td align='right'>")+QString::number(s.track)+
+                           QLatin1String("</td><td><a href=\"")+u.toString()+QLatin1String("\">") +
+                           (s.sameMetadata(currentSong) ? "<b>"+s.displayTitle()+"</b>" : s.displayTitle())+
+                           QLatin1String("</a></td></tr>");
+            } else {
+                trackList+=QLatin1String("<tr><td align='right'>")+QString::number(s.track)+
+                           QLatin1String("</td><td><a href=\"")+constScheme+QLatin1String(":///")+s.file+QLatin1String("\">") +
+                           (s.file==currentSong.file ? "<b>"+s.displayTitle()+"</b>" : s.displayTitle())+
+                           QLatin1String("</a></td></tr>");
+            }
         }
 
         trackList+=QLatin1String("</table></p>");
@@ -188,7 +234,7 @@ void AlbumView::getTrackListing()
 void AlbumView::getDetails()
 {
     engine->cancel();
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QString prefix=engine->getPrefix(lang);
         QString cachedFile=cacheFileName(Covers::fixArtist(currentSong.albumArtist()), currentSong.album, prefix, false);
         if (QFile::exists(cachedFile)) {
@@ -293,3 +339,5 @@ void AlbumView::clearDetails()
     engine->cancel();
     detailsReceived=0;
 }
+
+#include "moc_albumview.cpp"
