@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -42,8 +42,9 @@ public:
     static const QLatin1String constInfoExt;
 
     AlbumView(QWidget *p);
+    ~AlbumView() override;
 
-    void update(const Song &song, bool force=false);
+    void update(const Song &song, bool force=false) override;
 
 Q_SIGNALS:
     void playSong(const QString &file);
@@ -64,11 +65,12 @@ private:
     void getTrackListing();
     void getDetails();
     void updateDetails(bool preservePos=false);
-    void abort();
+    void abort() override;
 
 private:
     QString currentArtist;
     Action *refreshAction;
+    Action *fullWidthCoverAction;
     ContextEngine *engine;
     int detailsReceived;
     QString pic;
