@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,9 +28,10 @@
 #include "onlineview.h"
 #include "mpd-interface/song.h"
 #include "support/utils.h"
+#include "gui/apikeys.h"
 #include "gui/covers.h"
-#include "network/networkaccessmanager.h"
 #include "gui/settings.h"
+#include "network/networkaccessmanager.h"
 #include "wikipediaengine.h"
 #include "support/gtkstyle.h"
 #include "widgets/playqueueview.h"
@@ -70,8 +71,6 @@ void ContextWidget::enableDebug()
 }
 
 const QLatin1String ContextWidget::constBackdropFileName("backdrop");
-//const QLatin1String ContextWidget::constHtbApiKey(0); // API key required
-const QLatin1String ContextWidget::constFanArtApiKey("ee86404cb429fa27ac32a1a3c117b006");
 const QLatin1String ContextWidget::constCacheDir("backdrops/");
 
 static QString cacheFileName(const QString &artist, bool createDir)
@@ -83,7 +82,7 @@ class ViewSelectorButton : public QToolButton
 {
 public:
     ViewSelectorButton(QWidget *p) : QToolButton(p) { }
-    void paintEvent(QPaintEvent *ev)
+    void paintEvent(QPaintEvent *ev) override
     {
         Q_UNUSED(ev)
         QPainter painter(this);
@@ -231,7 +230,7 @@ QSplitterHandle * ThinSplitter::createHandle()
 void ThinSplitter::reset()
 {
     int totalSize=0;
-    foreach (int s, sizes()) {
+    for (int s: sizes()) {
         totalSize+=s;
     }
     QList<int> newSizes;
@@ -245,16 +244,17 @@ void ThinSplitter::reset()
 
 ContextWidget::ContextWidget(QWidget *parent)
     : QWidget(parent)
-    , job(0)
+    , shown(false)
+    , job(nullptr)
     , alwaysCollapsed(false)
     , backdropType(PlayQueueView::BI_Cover)
     , darkBackground(false)
     , fadeValue(1.0)
     , isWide(false)
-    , stack(0)
-    , onlineContext(0)
-    , splitter(0)
-    , viewSelector(0)
+    , stack(nullptr)
+    , onlineContext(nullptr)
+    , splitter(nullptr)
+    , viewSelector(nullptr)
 {
     QHBoxLayout *layout=new QHBoxLayout(this);
     mainStack=new QStackedWidget(this);
@@ -333,6 +333,9 @@ void ContextWidget::setWide(bool w)
         splitter->addWidget(album);
         splitter->setVisible(true);
         splitter->addWidget(song);
+        splitter->setCollapsible(0, false);
+        splitter->setCollapsible(1, false);
+        splitter->setCollapsible(2, false);
         if (resetSplitter) {
             splitter->reset();
         } else if (!state.isEmpty()) {
@@ -421,7 +424,7 @@ void ContextWidget::readConfig()
         break;
    case PlayQueueView::BI_Custom:
         if (origType!=backdropType || backdropOpacity!=origOpacity || backdropBlur!=origBlur || origCustomBackdropFile!=customBackdropFile) {            
-            updateImage(QImage(customBackdropFile));
+            updateImage(customBackdropFile.isEmpty() ? QImage() : QImage(customBackdropFile));
         }
         break;
     }
@@ -450,47 +453,60 @@ void ContextWidget::saveConfig()
 void ContextWidget::useDarkBackground(bool u)
 {
     if (u!=darkBackground) {
-        darkBackground=u;
-        QPalette pal=darkBackground ? palette() : parentWidget()->palette();
-        QColor prevLinkColor;
-        QColor linkCol;
-
-        if (darkBackground) {
-            QColor dark(32, 32, 32);
-            QColor light(240, 240, 240);
-            QColor linkVisited(164, 164, 164);
-            pal.setColor(QPalette::Window, dark);
-            pal.setColor(QPalette::Base, dark);
-            // Dont globally change window/button text - because this can mess up scrollbar buttons
-            // with some styles (e.g. plastique)
-//            pal.setColor(QPalette::WindowText, light);
-//            pal.setColor(QPalette::ButtonText, light);
-            pal.setColor(QPalette::Text, light);
-            pal.setColor(QPalette::Link, light);
-            pal.setColor(QPalette::LinkVisited, linkVisited);
-            prevLinkColor=appLinkColor;
-            linkCol=pal.color(QPalette::Link);
-        } else {
-            linkCol=appLinkColor;
-            prevLinkColor=QColor(240, 240, 240);
-        }
-        setPalette(pal);
-        artist->setPal(pal, linkCol, prevLinkColor);
-        album->setPal(pal, linkCol, prevLinkColor);
-        song->setPal(pal, linkCol, prevLinkColor);
-        if (viewSelector) {
-            viewSelector->setPalette(pal);
-        }
-        QWidget::update();
+        darkBackground = u;
+        updatePalette();
     }
 }
 
+void ContextWidget::updatePalette()
+{
+    QPalette pal=darkBackground ? palette() : parentWidget()->palette();
+    QColor prevLinkColor;
+    QColor linkCol;
+
+    if (darkBackground) {
+        QColor dark(32, 32, 32);
+        QColor light(240, 240, 240);
+        QColor linkVisited(164, 164, 164);
+        pal.setColor(QPalette::Window, dark);
+        pal.setColor(QPalette::Base, dark);
+        // Dont globally change window/button text - because this can mess up scrollbar buttons
+        // with some styles (e.g. plastique)
+        // pal.setColor(QPalette::WindowText, light);
+        // pal.setColor(QPalette::ButtonText, light);
+        pal.setColor(QPalette::Text, light);
+        pal.setColor(QPalette::Link, light);
+        pal.setColor(QPalette::LinkVisited, linkVisited);
+        prevLinkColor=appLinkColor;
+        linkCol=pal.color(QPalette::Link);
+    } else {
+        linkCol=appLinkColor;
+        prevLinkColor=QColor(240, 240, 240);
+        pal.setColor(QPalette::Base, pal.color(QPalette::Window));
+    }
+    setPalette(pal);
+    artist->setPal(pal, linkCol, prevLinkColor);
+    album->setPal(pal, linkCol, prevLinkColor);
+    song->setPal(pal, linkCol, prevLinkColor);
+    if (viewSelector) {
+        viewSelector->setPalette(pal);
+    }
+    QWidget::update();
+}
+
 void ContextWidget::showEvent(QShowEvent *e)
 {
     setWide(width()>minWidth && !alwaysCollapsed);
     if (backdropType) {
         updateBackdrop();
     }
+    if (!shown) {
+        // Some styles (e.g Adwaita-Qt) draw base colour for scrollbar background.
+        // We need to fix this to use the window background. Therefore, the first
+        // time we are shown set the palette.
+        updatePalette();
+        shown = true;
+    }
     QWidget::showEvent(e);
 }
 
@@ -654,7 +670,7 @@ void ContextWidget::cancel()
 {
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
 }
 
@@ -682,7 +698,7 @@ void ContextWidget::updateBackdrop(bool force)
             dirName+=Utils::getDir(currentSong.file);
 
             for (int level=0; level<2; ++level) {
-                foreach (const QString &fileName, names) {
+                for (const QString &fileName: names) {
                     DBUG << "Checking file(1)" << QString(dirName+fileName);
                     if (QFile::exists(dirName+fileName)) {
                         QImage img(dirName+fileName);
@@ -708,7 +724,7 @@ void ContextWidget::updateBackdrop(bool force)
         QString dirName=MPDConnection::self()->getDetails().dirReadable ? MPDConnection::self()->getDetails().dir : QString();
         if (!dirName.isEmpty() && !dirName.startsWith(QLatin1String("http:/"))) {
             dirName+=currentArtist+Utils::constDirSep;
-            foreach (const QString &fileName, names) {
+            for (const QString &fileName: names) {
                 DBUG << "Checking file(2)" << QString(dirName+fileName);
                 if (QFile::exists(dirName+fileName)) {
                     QImage img(dirName+fileName);
@@ -825,7 +841,7 @@ void ContextWidget::musicbrainzResponse()
         QUrl url("http://webservice.fanart.tv/v3/music/"+id);
         QUrlQuery query;
 
-        query.addQueryItem("api_key", constFanArtApiKey);
+        ApiKeys::self()->addKey(query, ApiKeys::FanArt);
         url.setQuery(query);
 
         job=NetworkAccessManager::self()->get(url);
@@ -897,41 +913,12 @@ void ContextWidget::downloadResponse()
     }
 
     if (!img.isNull()) {
-        bool saved=false;
-
-        if (Settings::self()->storeBackdropsInMpdDir() && !currentSong.isVariousArtists() &&
-            !currentSong.isNonMPD() && MPDConnection::self()->getDetails().dirReadable) {
-            QString mpdDir=MPDConnection::self()->getDetails().dir;
-            QString songDir=Utils::getDir(currentSong.file);
-            if (!mpdDir.isEmpty() && 2==songDir.split(Utils::constDirSep, QString::SkipEmptyParts).count()) {
-                QDir d(mpdDir+songDir);
-                d.cdUp();
-                QString fileName=Utils::fixPath(d.absolutePath())+constBackdropFileName+".jpg";
-                QFile f(fileName);
-                if (f.open(QIODevice::WriteOnly)) {
-                    f.write(data);
-                    f.close();
-                    DBUG << "Saved backdrop to" << fileName << "for artist" << currentArtist << ", current song" << currentSong.file;
-                    saved=true;
-                }
-            } else {
-                DBUG << "Not saving to mpd folder, mpd dir:" << mpdDir
-                     << "num parts:" << songDir.split(Utils::constDirSep, QString::SkipEmptyParts).count();
-            }
-        } else {
-            DBUG << "Not saving to mpd folder - set to save in mpd?" << Settings::self()->storeBackdropsInMpdDir()
-                 << "isVa:" << currentSong.isVariousArtists() << "isNonMPD:" << currentSong.isNonMPD()
-                 << "mpd readable:" << MPDConnection::self()->getDetails().dirReadable;
-        }
-
-        if (!saved) {
-            QString cacheName=cacheFileName(currentArtist, true);
-            QFile f(cacheName);
-            if (f.open(QIODevice::WriteOnly)) {
-                DBUG << "Saved backdrop to (cache)" << cacheName << "for artist" << currentArtist << ", current song" << currentSong.file;
-                f.write(data);
-                f.close();
-            }
+        QString cacheName=cacheFileName(currentArtist, true);
+        QFile f(cacheName);
+        if (f.open(QIODevice::WriteOnly)) {
+            DBUG << "Saved backdrop to (cache)" << cacheName << "for artist" << currentArtist << ", current song" << currentSong.file;
+            f.write(data);
+            f.close();
         }
     }
     updateImage(img);
@@ -949,13 +936,15 @@ NetworkJob * ContextWidget::getReply(QObject *obj)
 {
     NetworkJob *reply = qobject_cast<NetworkJob*>(obj);
     if (!reply) {
-        return 0;
+        return nullptr;
     }
 
     reply->deleteLater();
     if (reply!=job) {
-        return 0;
+        return nullptr;
     }
-    job=0;
+    job=nullptr;
     return reply;
 }
+
+#include "moc_contextwidget.cpp"
