@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -52,6 +52,7 @@
 #include <QTimer>
 #include <QScrollBar>
 #include <QDesktopServices>
+#include <QUrl>
 #include <QUrlQuery>
 
 const QLatin1String SongView::constLyricsDir("lyrics/");
@@ -61,51 +62,32 @@ const QLatin1String SongView::constInfoExt(".html.gz");
 
 static QString infoCacheFileName(const Song &song, const QString &lang, bool createDir)
 {
-    QString artist=song.artist;
-    QString title=song.title;
-    title.replace("/", "_");
-    artist.replace("/", "_");
-    QString dir=Utils::cacheDir(SongView::constCacheDir+Covers::encodeName(artist)+Utils::constDirSep, createDir);
-
+    QString dir=Utils::cacheDir(SongView::constCacheDir+Covers::encodeName(song.artist)+Utils::constDirSep, createDir);
     if (dir.isEmpty()) {
         return QString();
     }
-    return dir+Covers::encodeName(title)+"."+lang+SongView::constInfoExt;
+    return dir+Covers::encodeName(song.title)+"."+lang+SongView::constInfoExt;
 }
 
 static QString lyricsCacheFileName(const Song &song, bool createDir=false)
 {
-    QString artist=song.artist;
-    QString title=song.title;
-    title.replace("/", "_");
-    artist.replace("/", "_");
-    QString dir=Utils::cacheDir(SongView::constLyricsDir+Covers::encodeName(artist)+Utils::constDirSep, createDir);
-
+    QString dir=Utils::cacheDir(SongView::constLyricsDir+Covers::encodeName(song.artist)+Utils::constDirSep, createDir);
     if (dir.isEmpty()) {
         return QString();
     }
-    return dir+Covers::encodeName(title)+SongView::constExtension;
+    return dir+Covers::encodeName(song.title)+SongView::constExtension;
 }
 
 #if !defined Q_OS_WIN && !defined Q_OS_MAC
-static QString lyricsOtherFileName(const Song &song, bool createDir=false)
+static QString lyricsOtherFileName(const Song &song)
 {
-    QString artist=song.artist;
-    QString title=song.title;
-    title.replace("/", "_");
-    artist.replace("/", "_");
-    return QDir::homePath()+"/.lyrics/"+Covers::encodeName(artist)+" - "+Covers::encodeName(title)+".txt";
+    return QDir::homePath()+"/.lyrics/"+Covers::encodeName(song.artist)+" - "+Covers::encodeName(song.title)+".txt";
 }
 #endif
 
-static inline QString mpdLyricsFilePath(const QString &songFile)
-{
-    return Utils::changeExtension(MPDConnection::self()->getDetails().dir+songFile, SongView::constExtension);
-}
-
 static inline QString mpdLyricsFilePath(const Song &song)
 {
-    return mpdLyricsFilePath(song.filePath());
+    return Utils::changeExtension(song.filePath(MPDConnection::self()->getDetails().dir), SongView::constExtension);
 }
 
 static inline QString fixNewLines(const QString &o)
@@ -113,27 +95,15 @@ static inline QString fixNewLines(const QString &o)
     return QString(o).replace(QLatin1String("\n\n\n"), QLatin1String("\n\n")).replace("\n", "<br/>");
 }
 
-static QString actualFile(const Song &song)
-{
-    QString songFile=song.filePath();
-
-    if (song.isCantataStream()) {
-        QUrl qu(songFile);
-        QUrlQuery u(qu);
-        songFile=u.hasQueryItem("file") ? u.queryItemValue("file") : QString();
-    }
-    return songFile;
-}
-
 SongView::SongView(QWidget *p)
     : View(p, QStringList() << tr("Lyrics") << tr("Information") << tr("Metadata"))
-    , scrollTimer(0)
+    , scrollTimer(nullptr)
     , songPos(0)
     , currentProvider(-1)
     , currentRequest(0)
     , mode(Mode_Display)
-    , job(0)
-    , currentProv(0)
+    , job(nullptr)
+    , currentProv(nullptr)
     , lyricsNeedsUpdating(true)
     , infoNeedsUpdating(true)
     , metadataNeedsUpdating(true)
@@ -159,7 +129,7 @@ SongView::SongView(QWidget *p)
     connect(refreshInfoAction, SIGNAL(triggered()), SLOT(refreshInfo()));
     connect(cancelInfoJobAction, SIGNAL(triggered()), SLOT(abortInfoSearch()));
     connect(engine, SIGNAL(searchResult(QString,QString)), this, SLOT(infoSearchResponse(QString,QString)));
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         connect(t, SIGNAL(anchorClicked(QUrl)), SLOT(showMoreInfo(QUrl)));
     }
 
@@ -381,11 +351,11 @@ void SongView::loadLyrics()
     }
     lyricsNeedsUpdating=false;
 
-    if (!MPDConnection::self()->getDetails().dir.isEmpty() && !currentSong.file.isEmpty() && !currentSong.isNonMPD()) {
-        QString songFile=actualFile(currentSong);
-        QString mpdLyrics=mpdLyricsFilePath(songFile);
+    if (currentSong.isCantataStream() || (!MPDConnection::self()->getDetails().dir.isEmpty() && !currentSong.file.isEmpty() && !currentSong.isNonMPD())) {
+        QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
+        QString mpdLyrics=mpdLyricsFilePath(currentSong);
 
-        if (MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
+        if (!currentSong.isCantataStream() && MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
             QUrl url(mpdLyrics);
             job=NetworkAccessManager::self()->get(url);
             job->setProperty("file", currentSong.file);
@@ -393,7 +363,7 @@ void SongView::loadLyrics()
             return;
         } else {
             #ifdef TAGLIB_FOUND
-            QString tagLyrics=Tags::readLyrics(MPDConnection::self()->getDetails().dir+songFile);
+            QString tagLyrics=Tags::readLyrics(songFile);
 
             if (!tagLyrics.isEmpty()) {
                 text->setText(fixNewLines(tagLyrics));
@@ -458,7 +428,7 @@ void SongView::loadInfo()
         return;
     }
     infoNeedsUpdating=false;
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QString prefix=engine->getPrefix(lang);
         QString cachedFile=infoCacheFileName(currentSong, prefix, false);
         if (QFile::exists(cachedFile)) {
@@ -516,8 +486,8 @@ void SongView::loadMetadata()
     QMultiMap<int, QString> tags;
     QMultiMap<int, QString> audioProperties;
     #ifdef TAGLIB_FOUND
-    if (!currentSong.isStandardStream() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
-        QString songFile=actualFile(currentSong);
+    if (currentSong.isCantataStream() || (!currentSong.isStandardStream() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/")))) {
+        QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
         if (!songFile.isEmpty()) {
             static QMap<QString, MapEntry> tagMap;
             static QMap<QString, MapEntry> tagTimeMap;
@@ -560,7 +530,7 @@ void SongView::loadMetadata()
                 tagTimeMap.insert(QLatin1String("TAGGING TIME"), MapEntry(pos++, tr("Tagging time")));
             }
 
-            QMap<QString, QString> allTags=Tags::readAll(MPDConnection::self()->getDetails().dir+actualFile(currentSong));
+            QMap<QString, QString> allTags=Tags::readAll(songFile);
 
             if (!allTags.isEmpty()) {
                 QMap<QString, QString>::ConstIterator it=allTags.constBegin();
@@ -650,12 +620,12 @@ void SongView::loadMetadata()
     } else {
         tagInfo+=QLatin1String("<tr/>");
     }
-    if (MPDConnection::self()->getDetails().dirReadable) {
-        QString path=Utils::getDir(MPDConnection::self()->getDetails().dir+currentSong.filePath());
-        tagInfo+=createRow(tr("Filename"), QLatin1String("<a href=\"file://")+path+QLatin1String("\">")+
-                                             currentSong.filePath()+QLatin1String("</a>"));
+    QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
+    if (QFile::exists(songFile)) {
+        tagInfo+=createRow(tr("Filename"), QLatin1String("<a href=\"file://")+songFile+QLatin1String("\">")+
+                                           songFile+QLatin1String("</a>"));
     } else {
-        tagInfo+=createRow(tr("Filename"), currentSong.filePath());
+        tagInfo+=createRow(tr("Filename"), songFile);
     }
     tagInfo+=QLatin1String("</table>");
 
@@ -667,7 +637,7 @@ void SongView::refreshInfo()
     if (currentSong.isEmpty()) {
         return;
     }
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QFile::remove(infoCacheFileName(currentSong, engine->getPrefix(lang), false));
     }
     searchForInfo();
@@ -724,18 +694,16 @@ void SongView::abort()
 {
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
     currentProvider=-1;
     if (currentProv) {
         currentProv->abort();
-        currentProv=0;
+        currentProv=nullptr;
 
         text->setText(QString());
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
-                ? mpdLyricsFilePath(currentSong)
-                : lyricsCacheFileName(currentSong);
+        lyricsFile=lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
     cancelJobAction->setEnabled(false);
@@ -796,7 +764,7 @@ void SongView::downloadFinished()
     if (reply) {
         reply->deleteLater();
         if (job==reply) {
-            job=0;
+            job=nullptr;
             if (reply->ok()) {
                 QString file=reply->property("file").toString();
                 if (!file.isEmpty() && file==currentSong.file) {
@@ -838,10 +806,7 @@ void SongView::lyricsReady(int id, QString lyrics)
         } else {
             text->setText(fixNewLines(plain));
             lyricsFile=QString();
-            if (! ( Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD() &&
-                    saveFile(mpdLyricsFilePath(currentSong))) ) {
-                saveFile(lyricsCacheFileName(currentSong, true));
-            }
+            saveFile(lyricsCacheFileName(currentSong, true));
             setMode(Mode_Display);
         }
     }
@@ -883,9 +848,7 @@ void SongView::getLyrics()
         text->setText(QString());
         currentProvider=-1;
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
-                ? mpdLyricsFilePath(currentSong)
-                : lyricsCacheFileName(currentSong);
+        lyricsFile=lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
 }
@@ -930,3 +893,5 @@ bool SongView::setLyricsFromFile(const QString &filePath)
 
     return false;
 }
+
+#include "moc_songview.cpp"
