@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,7 +28,7 @@
 #include "config.h"
 #include "gui/currentcover.h"
 
-static inline qlonglong convertTime(int t)
+static inline qlonglong convertTime(qlonglong t)
 {
     return t*1000000;
 }
@@ -242,3 +242,5 @@ QString Mpris::currentTrackId() const
 {
     return QString("/org/mpris/MediaPlayer2/Track/%1").arg(QString::number(currentSong.id));
 }
+
+#include "moc_mpris.cpp"
