@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -77,8 +77,8 @@ ActionDialog::ActionDialog(QWidget *parent)
     : Dialog(parent)
     , spaceRequired(0)
     , mpdConfigured(false)
-    , currentDev(0)
-    , songDialog(0)
+    , currentDev(nullptr)
+    , songDialog(nullptr)
 {
     iCount++;
     setButtons(Ok|Cancel);
@@ -128,7 +128,7 @@ void ActionDialog::controlInfoLabel(Device *dev)
             int settingIndex=0;
             bool increase=encoder.values.at(0).value<encoder.values.at(1).value;
             int index=0;
-            foreach (const Encoders::Setting &s, encoder.values) {
+            for (const Encoders::Setting &s: encoder.values) {
                 if ((increase && s.value>opts.transcoderValue) || (!increase && s.value<opts.transcoderValue)) {
                     break;
                 } else {
@@ -137,10 +137,10 @@ void ActionDialog::controlInfoLabel(Device *dev)
                 index++;
             }
             codec->setText(QString("%1 (%2)").arg(encoder.name).arg(encoder.values.at(settingIndex).descr)+
-                           (sourceIsAudioCd && opts.transcoderWhenDifferent ? QLatin1String("")+tr("<i>(When different)</i>") : QString()));
+                           (!sourceIsAudioCd && DeviceOptions::TW_IfDifferent==opts.transcoderWhen ? tr("<i>(When different)</i>") : QString()));
         } else {
             codec->setText(encoder.name+
-                           (sourceIsAudioCd && opts.transcoderWhenDifferent ? QLatin1String("")+tr("<i>(When different)</i>") : QString()));
+                           (!sourceIsAudioCd && DeviceOptions::TW_IfDifferent==opts.transcoderWhen ? tr("<i>(When different)</i>") : QString()));
         }
     }
 }
@@ -164,7 +164,7 @@ void ActionDialog::updateSongCountLabel()
     QSet<QString> artists;
     QSet<QString> albums;
 
-    foreach (const Song &s, songsToAction) {
+    for (const Song &s: songsToAction) {
         artists.insert(s.albumArtist());
         albums.insert(s.albumArtist()+"--"+s.album);
     }
@@ -179,7 +179,7 @@ void ActionDialog::controlInfoLabel()
 
 void ActionDialog::calcFileSize()
 {
-    Device *dev=0;
+    Device *dev=nullptr;
     int toCalc=qMin(50, songsToCalcSize.size());
     for (int i=0; i<toCalc; ++i) {
         Song s=songsToCalcSize.takeAt(0);
@@ -369,7 +369,7 @@ void ActionDialog::remove(const QString &udi, const QList<Song> &songs)
     }
 
     setPage(PAGE_PROGRESS);
-    foreach (const Song &s, songsToAction) {
+    for (const Song &s: songsToAction) {
         dirsToClean.insert(baseDir+Utils::getDir(s.file));
     }
     show();
@@ -395,7 +395,7 @@ void ActionDialog::init(const QString &srcUdi, const QString &dstUdi, const QLis
     actionedSongs.clear();
     skippedSongs.clear();
     currentPercent=0;
-    currentDev=0;
+    currentDev=nullptr;
     count=0;
     #ifdef ENABLE_REPLAYGAIN_SUPPORT
     albumsWithoutRgTags.clear();
@@ -479,7 +479,7 @@ void ActionDialog::slotButtonClicked(int button)
         paused=true;
         if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Are you sure you wish to stop?"), tr("Stop"),
                                                        StdGuiItem::stop(), StdGuiItem::cont())) {
-            Device *dev=0;
+            Device *dev=nullptr;
             if(Copy==mode || Sync==mode) {
                 dev=getDevice(sourceUdi.isEmpty() ? destUdi : sourceUdi, false);
             } else if (!sourceUdi.isEmpty()) { // Must be a delete...
@@ -541,7 +541,7 @@ Device * ActionDialog::getDevice(const QString &udi, bool logErrors)
         MessageBox::error(parentWidget(), error);
     }
 
-    return 0;
+    return nullptr;
 }
 
 void ActionDialog::doNext()
@@ -570,7 +570,7 @@ void ActionDialog::doNext()
                 performingAction=true;
                 if (copyToDev) {
                     destFile=dev->path()+dev->options().createFilename(currentSong);
-                    currentSong.file=MPDConnection::self()->getDetails().dir+currentSong.filePath();
+                    currentSong.file=currentSong.filePath(MPDConnection::self()->getDetails().dir);
                     dev->addSong(currentSong, overwrite->isChecked(), !copiedCovers.contains(Utils::getDir(destFile)));
                 } else {
                     Song copy=currentSong;
@@ -601,7 +601,7 @@ void ActionDialog::doNext()
         }
         progressLabel->setText(formatSong(currentSong, false));
     } else if (Remove==mode && dirsToClean.count()) {
-        Device *dev=sourceUdi.isEmpty() ? 0 : DevicesModel::self()->device(sourceUdi);
+        Device *dev=sourceUdi.isEmpty() ? nullptr : DevicesModel::self()->device(sourceUdi);
         if (sourceUdi.isEmpty() || dev) {
             progressLabel->setText(tr("Clearing unused folders"));
             if (dev) {
@@ -626,7 +626,7 @@ void ActionDialog::doNext()
                     opts.load(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
                     Encoders::Encoder encoder=Encoders::getEncoder(opts.transcoderCodec);
 
-                    foreach (const Song &s, actionedSongs) {
+                    for (const Song &s: actionedSongs) {
                         if (albumsWithoutRgTags.contains(s.album)) {
                             Song song=s;
                             song.file=encoder.changeExtension(namingOptions.createFilename(s));
@@ -956,3 +956,5 @@ void ActionDialog::cacheSaved()
     emit completed();
     accept();
 }
+
+#include "moc_actiondialog.cpp"
