@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -54,6 +54,10 @@
 #include "solid-lite/genericinterface.h"
 #endif // ENABLE_DEVICES_SUPPORT
 
+#include <QDebug>
+#define DBUG_CLASS(CLASS) if (DevicesModel::debugEnabled()) qWarning() << CLASS << __FUNCTION__
+#define DBUG DBUG_CLASS(metaObject()->className())
+
 void Device::moveDir(const QString &from, const QString &to, const QString &base, const QString &coverFile)
 {
     QDir d(from);
@@ -70,7 +74,7 @@ void Device::moveDir(const QString &from, const QString &to, const QString &base
             }
         }
 
-        foreach (const QFileInfo &info, entries) {
+        for (const QFileInfo &info: entries) {
             if (info.isDir()) {
                 return;
             }
@@ -80,7 +84,7 @@ void Device::moveDir(const QString &from, const QString &to, const QString &base
             extraFiles.append(info.fileName());
         }
 
-        foreach (const QString &f, extraFiles) {
+        for (const QString &f: extraFiles) {
             if (!QFile::rename(from+'/'+f, to+'/'+f)) {
                 return;
             }
@@ -98,7 +102,7 @@ void Device::cleanDir(const QString &dir, const QString &base, const QString &co
         QSet<QString> others=Covers::standardNames().toSet();
         others << coverFile << "albumart.pamp";
 
-        foreach (const QFileInfo &info, entries) {
+        for (const QFileInfo &info: entries) {
             if (info.isDir()) {
                 return;
             }
@@ -108,7 +112,7 @@ void Device::cleanDir(const QString &dir, const QString &base, const QString &co
             extraFiles.append(info.absoluteFilePath());
         }
 
-        foreach (const QString &cf, extraFiles) {
+        for (const QString &cf: extraFiles) {
             if (!QFile::remove(cf)) {
                 return;
             }
@@ -158,8 +162,10 @@ Device * Device::create(MusicLibraryModel *m, const QString &udi)
 {
     Solid::Device device=Solid::Device(udi);
 
+    DBUG_CLASS("Device") << udi;
     if (device.is<Solid::OpticalDisc>()) {
         #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+        DBUG_CLASS("Device") << "AudioCD";
         return Encoders::getAvailable().isEmpty() ? 0 : new AudioCdDevice(m, device);
         #endif
     } else if (device.is<Solid::PortableMediaPlayer>()) {
@@ -175,30 +181,26 @@ Device * Device::create(MusicLibraryModel *m, const QString &udi)
                 QMap<QString, QVariant>::ConstIterator bus = properties.find(QLatin1String("BUSNUM"));
                 QMap<QString, QVariant>::ConstIterator dev = properties.find(QLatin1String("DEVNUM"));
 
+                DBUG_CLASS("Device") << "MTP";
                 return properties.constEnd()==bus || properties.end()==dev
-                        ? 0
+                        ? nullptr
                         : new MtpDevice(m, device, bus.value().toUInt(), dev.value().toUInt());
             }
         }
         #endif
     } else if (device.is<Solid::StorageAccess>()) {
+        DBUG_CLASS("Device") << "Storage";
 
-        const Solid::StorageAccess* ssa = device.as<Solid::StorageAccess>();
-
-        if( ssa && (!device.parent().as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.parent().as<Solid::StorageDrive>()->bus()) &&
-                   (!device.as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.as<Solid::StorageDrive>()->bus()) ) {
-            return 0;
+        if ( (!device.parent().as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.parent().as<Solid::StorageDrive>()->bus()) &&
+             (!device.as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.as<Solid::StorageDrive>()->bus()) ) {
+            DBUG_CLASS("Device") << "Not drive / usb ?";
+            return nullptr;
         }
 
-        //HACK: ignore apple stuff until we have common MediaDeviceFactory.
-        if (!device.vendor().contains("apple", Qt::CaseInsensitive)) {
-//             Solid::StorageAccess *sa = device.as<Solid::StorageAccess>();
-//             if (QLatin1String("usb")==sa->bus) {
-                return new UmsDevice(m, device);
-//             }
-        }
+        DBUG_CLASS("Device") << "UMS, Vendor:" << device.vendor();
+        return new UmsDevice(m, device);
     }
-    return 0;
+    return nullptr;
 }
 
 bool Device::fixVariousArtists(const QString &file, Song &song, bool applyFix)
@@ -280,7 +282,7 @@ QTemporaryFile * Device::copySongToTemp(Song &song)
         if (!QFile::copy(song.file, temp->fileName())) {
             temp->remove();
             delete temp;
-            temp=0;
+            temp=nullptr;
         }
     }
     return temp;
@@ -303,7 +305,7 @@ Device::Device(MusicLibraryModel *m, Solid::Device &dev, bool albumArtistSupport
     , configured(false)
     , solidDev(dev)
     , deviceId(dev.udi())
-    , update(0)
+    , update(nullptr)
     , needToFixVa(false)
     , jobAbortRequested(false)
     , transcoding(false)
@@ -317,7 +319,7 @@ Device::Device(MusicLibraryModel *m, const QString &name, const QString &id)
     : MusicLibraryItemRoot(name)
     , configured(false)
     , deviceId(id)
-    , update(0)
+    , update(nullptr)
     , needToFixVa(false)
     , jobAbortRequested(false)
     , transcoding(false)
@@ -342,10 +344,10 @@ void Device::applyUpdate()
         QSet<Song> removed=currentSongs-updateSongs;
         QSet<Song> added=updateSongs-currentSongs;
 
-        foreach (const Song &s, removed) {
+        for (const Song &s: removed) {
             removeSongFromList(s);
         }
-        foreach (const Song &s, added) {
+        for (const Song &s: added) {
             addSongToList(s);
         }
         delete update;
@@ -369,7 +371,7 @@ void Device::applyUpdate()
         }
     }
     delete update;
-    update=0;
+    update=nullptr;
 }
 
 QModelIndex Device::index() const
@@ -400,3 +402,5 @@ void Device::updatePercentage(int pc)
 }
 
 #endif // ENABLE_DEVICES_SUPPORT
+
+#include "moc_device.cpp"
