@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * Large portions of this file, mainly the textual descriptions, etc, are taken from
  * Amarok - hence the (c) notice below...
@@ -253,7 +253,7 @@ static void init()
                                       "without compromising on audio quality, FLAC is an excellent choice."),
                                 QObject::tr("The <a href=http://flac.sourceforge.net/documentation_tools_flac.html>"
                                      "compression level</a> is an integer value between 0 and 8 that represents "
-                                     "the tradeoff between file size and compression speed while encoding with <b>FLAC</b>.<br/> "
+                                     "the tradeoff between file size and compression speed while encoding with <b>FLAC</b>.<br/>"
                                      "Setting the compression level to <b>0</b> yields the shortest compression time but "
                                      "generates a comparably big file.<br/>"
                                      "On the other hand, a compression level of <b>8</b> makes compression quite slow but "
@@ -332,7 +332,7 @@ static void init()
             }
 
             QStringList lines=output.split('\n', QString::SkipEmptyParts);
-            foreach (const QString &line, lines) {
+            for (const QString &line: lines) {
                 int pos=line.indexOf(QRegExp(QLatin1String("[\\. D]EA")));
                 if (0==pos || 1==pos) {
                     QList<Encoder>::Iterator it(initial.begin());
@@ -385,7 +385,7 @@ QStringList Encoder::params(int value, const QString &in, const QString &out) co
     if (!param.isEmpty() && values.size()>1) {
         bool increase=values.at(0).value<values.at(1).value;
         int v=values.at(defaultValueIndex).value;
-        foreach (const Setting &s, values) {
+        for (const Setting &s: values) {
             if ((increase && s.value>value) || (!increase && s.value<value)) {
                 break;
             } else {
@@ -417,7 +417,7 @@ QList<Encoder> getAvailable()
 Encoder getEncoder(const QString &codec)
 {
     init();
-    foreach (const Encoder &encoder, installedEncoders) {
+    for (const Encoder &encoder: installedEncoders) {
         if (encoder.codec==codec) {
             return encoder;
         }
