@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -153,13 +153,11 @@ void ExtractJob::run()
         Tags::update(destFile, Song(), song, 3);
 
         if (!stopRequested && !coverFile.isEmpty()) {
-            QString mpdCover=MPDConnection::self()->getDetails().coverName;
-            if (mpdCover.isEmpty()) {
-                mpdCover=Covers::constFileName;
-            }
-            copiedCover=Covers::copyImage(Utils::getDir(coverFile), Utils::getDir(destFile), Utils::getFile(coverFile), mpdCover+coverFile.mid(coverFile.length()-4), 0);
+            copiedCover=Covers::copyImage(Utils::getDir(coverFile), Utils::getDir(destFile), Utils::getFile(coverFile), Covers::albumFileName(song)+coverFile.mid(coverFile.length()-4), 0);
         }
 
         emit result(Device::Ok);
     }
 }
+
+#include "moc_extractjob.cpp"
