@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -224,7 +224,7 @@ void Mounter::timeout()
 
     QSet<int> running;
 
-    foreach (int p, pids) {
+    for (int p: pids) {
         if (0==kill(p, 0)) {
             running.insert(p);
         }
@@ -245,3 +245,5 @@ void Mounter::timeout()
         startTimer();
     }
 }
+
+#include "moc_mounter.cpp"
