@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -93,7 +93,7 @@ static uint16_t fileReceiver(void *params, void *priv, uint32_t sendlen, unsigne
     return LIBMTP_HANDLER_RETURN_OK;
 }
 
-MtpConnection::MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport)
+MtpConnection::MtpConnection(const QString &id, unsigned int bus, unsigned int dev, bool aaSupport)
     : device(0)
     #ifdef MTP_CLEAN_ALBUMS
     , albums(0)
@@ -108,7 +108,7 @@ MtpConnection::MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport)
     size=0;
     used=0;
     LIBMTP_Init();
-    thread=new Thread(metaObject()->className());
+    thread=new Thread(metaObject()->className()+QLatin1String("::")+id);
     moveToThread(thread);
     thread->start();
 }
@@ -443,7 +443,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
             QSet<quint16> tracks;
             QString shortestArtist;
             bool duplicateTrackNumbers=false;
-            foreach (MusicLibraryItemSong *s, (*it).songs) {
+            for (const MusicLibraryItemSong *s: (*it).songs) {
                 if (tracks.contains(s->track())) {
                     duplicateTrackNumbers=true;
                     break;
@@ -461,7 +461,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
                 // Now, check to see if all artists contain 'shortestArtist'. If so then use 'shortestArtist' as the album
                 // artist. This is probably due to songs which have artist set to '${artist} and somebodyelse'
                 QString albumArtist=shortestArtist;
-                foreach (const QString &artist, (*it).artists) {
+                for (const QString &artist: (*it).artists) {
                     if (!artist.contains(shortestArtist)) {
                         // Found an artist that did not contain 'shortestArtist', so use 'Various Artists' for album artist
                         albumArtist=!f.artist.isEmpty() && f.album==it.key() ? f.artist : Song::variousArtists();
@@ -470,7 +470,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
                 }
 
                 // Now move songs to correct artist/album...
-                foreach (MusicLibraryItemSong *s, (*it).songs) {
+                for (MusicLibraryItemSong *s: (*it).songs) {
                     if (s->song().albumartist==albumArtist) {
                         continue;
                     }
@@ -526,7 +526,7 @@ void MtpConnection::setMusicFolder(Storage &store)
 void MtpConnection::updateFilesAndFolders()
 {
     folderMap.clear();
-    foreach (const Storage st, storage) {
+    for (const Storage &st: storage) {
         if (abortRequested) {
             return;
         }
@@ -693,7 +693,7 @@ uint32_t MtpConnection::checkFolderStructure(const QStringList &dirs, Storage &s
     QString path;
     uint32_t parentId=store.musicFolderId;
 
-    foreach (const QString &d, dirs) {
+    for (const QString &d: dirs) {
         path+=d+QChar('/');
         uint32_t folderId=getFolder(path, store.id);
         if (0==folderId) {
@@ -965,11 +965,7 @@ void MtpConnection::getSong(const Song &song, const QString &dest, bool fixVa, b
             File cover=getCoverDetils(song);
 
             if (0!=cover.id) {
-                QString mpdCover=MPDConnection::self()->getDetails().coverName;
-                if (mpdCover.isEmpty()) {
-                    mpdCover="cover";
-                }
-                QString fileName=QString(destDir+mpdCover+(cover.name.endsWith(".jpg", Qt::CaseInsensitive) ? ".jpg" : ".png"));
+                QString fileName=QString(destDir+Covers::albumFileName(song)+(cover.name.endsWith(".jpg", Qt::CaseInsensitive) ? ".jpg" : ".png"));
                 QByteArray fileNameUtf8=fileName.toUtf8();
                 copiedCover=0==LIBMTP_Get_File_To_File(device, cover.id, fileNameUtf8.constData(), 0, 0);
                 if (copiedCover) {
@@ -1034,7 +1030,7 @@ bool MtpConnection::removeFolder(uint32_t folderId)
     if (folderMap.end()!=folder && (*folder).folders.isEmpty() && (*folder).files.isEmpty()) {
         // Delete any cover files...
         QList<uint32_t> toRemove=(*folder).covers.keys();
-        foreach (uint32_t cover, toRemove) {
+        for (uint32_t cover: toRemove) {
             if (0==LIBMTP_Delete_Object(device, cover)) {
                 (*folder).covers.remove(cover);
             }
@@ -1054,7 +1050,7 @@ bool MtpConnection::removeFolder(uint32_t folderId)
 
 void MtpConnection::cleanDirs(const QSet<QString> &dirs)
 {
-    foreach (const QString &d, dirs) {
+    for (const QString &d: dirs) {
         Path path=decodePath(d);
         Storage &store=getStorage(path.storage);
         if (0==store.musicFolderId) {
@@ -1190,7 +1186,7 @@ MtpDevice::MtpDevice(MusicLibraryModel *m, Solid::Device &dev, unsigned int busN
         registeredTypes=true;
     }
 
-    connection=new MtpConnection(busNum, devNum, supportsAlbumArtistTag());
+    connection=new MtpConnection(data(), busNum, devNum, supportsAlbumArtistTag());
     connect(this, SIGNAL(updateLibrary(const DeviceOptions &)), connection, SLOT(updateLibrary(const DeviceOptions &)));
     connect(connection, SIGNAL(libraryUpdated()), this, SLOT(libraryUpdated()));
     connect(connection, SIGNAL(progress(int)), this, SLOT(emitProgress(int)));
@@ -1337,8 +1333,8 @@ void MtpDevice::addSong(const Song &s, bool overwrite, bool copyCover)
     }
 
     transcoding = !opts.transcoderCodec.isEmpty() &&
-                     (!opts.transcoderWhenDifferent || encoder.isDifferent(s.file)) &&
-                     (!opts.transcoderWhenSourceIsLosssless || Device::isLossless(s.file));
+            (DeviceOptions::TW_IfDifferent!=opts.transcoderWhen || encoder.isDifferent(s.file)) &&
+            (DeviceOptions::TW_IfLossess!=opts.transcoderWhen || Device::isLossless(s.file));
 
     if (transcoding) {
         deleteTemp();
@@ -1632,3 +1628,5 @@ void MtpDevice::deleteTemp()
         tempFile=0;
     }
 }
+
+#include "moc_mtpdevice.cpp"
