@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,6 +22,7 @@
  */
 
 #include "coverdialog.h"
+#include "apikeys.h"
 #include "support/messagebox.h"
 #include "widgets/listview.h"
 #include "network/networkaccessmanager.h"
@@ -158,7 +159,7 @@ public:
         f.setBold((true));
         setFont(f);
     }
-    bool isExisting() const { return true; }
+    bool isExisting() const override { return true; }
     const QImage & image() const { return img; }
 private:
     QImage img;
@@ -170,7 +171,7 @@ public:
     LocalCover(const QString &u, const QImage &i, QListWidget *parent)
         : CoverItem(u, QString(), i, Utils::getFile(u), parent, i.width(), i.height(), QFileInfo(u).size())
         , img(i) { }
-    bool isLocal() const { return true; }
+    bool isLocal() const override { return true; }
     const QImage & image() const { return img; }
 private:
     QImage img;
@@ -327,17 +328,17 @@ void CoverPreview::wheelEvent(QWheelEvent *event)
 
 CoverDialog::CoverDialog(QWidget *parent)
     : Dialog(parent, "CoverDialog")
-    , existing(0)
+    , existing(nullptr)
     , currentQueryProviders(0)
-    , preview(0)
+    , preview(nullptr)
     , saving(false)
     , isArtist(false)
-    , spinner(0)
-    , msgOverlay(0)
+    , spinner(nullptr)
+    , msgOverlay(nullptr)
     , page(0)
-    , menu(0)
-    , showAction(0)
-    , removeAction(0)
+    , menu(nullptr)
+    , showAction(nullptr)
+    , removeAction(nullptr)
 {
     Configuration cfg(objectName());
     enabledProviders=cfg.get("enabledProviders", (int)Prov_All);
@@ -440,7 +441,7 @@ void CoverDialog::show(const Song &s, const Covers::Image &current)
         QString dirName=MPDConnection::self()->getDetails().dir+Utils::getDir(s.file);
         QStringList files=QDir(dirName).entryList(QStringList() << QLatin1String("*.jpg") << QLatin1String("*.png"), QDir::Files|QDir::Readable);
         qWarning() << files;
-        foreach (const QString &f, files) {
+        for (const QString &f: files) {
             QString fileName=dirName+f;
             if (fileName!=img.fileName) {
                 QImage i(fileName);
@@ -542,7 +543,7 @@ void CoverDialog::downloadJobFinished()
         QImage img=QImage::fromData(data, format);
         if (!img.isNull()) {
             bool isLarge=reply->property(constThumbProperty).toString().isEmpty();
-            QTemporaryFile *temp=0;
+            QTemporaryFile *temp=nullptr;
 
             if (isLarge || (reply->property(constThumbProperty).toString()==reply->property(constLargeProperty).toString())) {
                 temp=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX."+(format ? QString(QLatin1String(format)).toLower() : "png"));
@@ -563,7 +564,7 @@ void CoverDialog::downloadJobFinished()
                     tempFiles.prepend(temp);
                 } else {
                     delete temp;
-                    temp=0;
+                    temp=nullptr;
                 }
             }
             if (isLarge) {
@@ -577,7 +578,7 @@ void CoverDialog::downloadJobFinished()
                     }
                 }
             } else {
-                CoverItem *item=0;
+                CoverItem *item=nullptr;
                 img=cropImage(img, isArtist);
                 if (constLastFmHost==host) {
                     item=new LastFmCover(reply->property(constLargeProperty).toString(), url, img, list);
@@ -673,7 +674,7 @@ void CoverDialog::sendQuery()
             }
         }
 
-        foreach (CoverItem *item, keep) {
+        for (CoverItem *item: keep) {
             list->addItem(item);
         }
 
@@ -709,7 +710,7 @@ void CoverDialog::sendLastFmQuery(const QString &fixedQuery, int page)
     url.setScheme("https");
     url.setHost(constLastFmHost);
     url.setPath("/2.0/");
-    query.addQueryItem("api_key", Covers::constLastFmApiKey);
+    ApiKeys::self()->addKey(query, ApiKeys::LastFm);
     query.addQueryItem("limit", QString::number(20));
     query.addQueryItem("page", QString::number(page+1));
     query.addQueryItem(isArtist ? "artist" : "album", fixedQuery);
@@ -793,7 +794,7 @@ void CoverDialog::checkStatus()
 
 void CoverDialog::cancelQuery()
 {
-    foreach (NetworkJob *job, currentQuery) {
+    for (NetworkJob *job: currentQuery) {
         job->cancelAndDelete();
     }
     currentQuery.clear();
@@ -833,7 +834,7 @@ void CoverDialog::menuRequested(const QPoint &pos)
     showAction->setEnabled(1==items.count());
     removeAction->setEnabled(!items.isEmpty());
     if (removeAction->isEnabled()) {
-        foreach (QListWidgetItem *i, items) {
+        for (QListWidgetItem *i: items) {
             if (static_cast<CoverItem *>(i)->isExisting()) {
                 removeAction->setEnabled(false);
             }
@@ -853,7 +854,7 @@ void CoverDialog::showImage()
 void CoverDialog::removeImages()
 {
     QList<QListWidgetItem*> items=list->selectedItems();
-    foreach (QListWidgetItem *i, items) {
+    for (QListWidgetItem *i: items) {
         delete i;
     }
 }
@@ -871,7 +872,7 @@ void CoverDialog::updateProviders()
         }
     }
 
-    foreach (const QAction *act, configureButton->menu()->actions()) {
+    for (const QAction *act: configureButton->menu()->actions()) {
         if (act->isChecked()) {
             enabledProviders|=act->data().toUInt();
         }
@@ -881,7 +882,7 @@ void CoverDialog::updateProviders()
 
 void CoverDialog::clearTempFiles()
 {
-    foreach (QTemporaryFile *file, tempFiles) {
+    for (QTemporaryFile *file: tempFiles) {
         file->remove();
         delete file;
     }
@@ -902,11 +903,11 @@ NetworkJob * CoverDialog::downloadImage(const QString &url, DownloadType dlType)
     DBUG << url << dlType;
     if (DL_Thumbnail==dlType) {
         if (currentUrls.contains(url)) {
-            return 0;
+            return nullptr;
         }
         currentUrls.insert(url);
     } else {
-        foreach (QTemporaryFile *tmp, tempFiles) {
+        for (QTemporaryFile *tmp: tempFiles) {
             if (tmp->property(constLargeProperty).toString()==url) {
                 QImage img;
                 if (img.load(tmp->fileName())) {
@@ -918,7 +919,7 @@ NetworkJob * CoverDialog::downloadImage(const QString &url, DownloadType dlType)
                             accept();
                         }
                     }
-                    return 0;
+                    return nullptr;
                 }
                 tmp->remove();
                 delete tmp;
@@ -1007,18 +1008,18 @@ void CoverDialog::parseLastFmQueryResponse(const QByteArray &resp)
     QStringList largeUrls=QStringList() << "extralarge" << "large";
     QStringList thumbUrls=QStringList() << "large" << "medium" << "small";
 
-    foreach (const SizeMap &urls, entries) {
+    for (const SizeMap &urls: entries) {
         QString largeUrl;
         QString thumbUrl;
 
-        foreach (const QString &u, largeUrls) {
+        for (const QString &u: largeUrls) {
             if (urls.contains(u)) {
                 largeUrl=urls[u];
                 break;
             }
         }
 
-        foreach (const QString &u, thumbUrls) {
+        for (const QString &u: thumbUrls) {
             if (urls.contains(u)) {
                 thumbUrl=urls[u];
                 break;
@@ -1028,7 +1029,7 @@ void CoverDialog::parseLastFmQueryResponse(const QByteArray &resp)
     }
 
     if (enabledProviders&Prov_CoverArt) {
-        foreach (const QString &id, musibBrainzIds) {
+        for (const QString &id: musibBrainzIds) {
             QUrl coverartUrl;
             coverartUrl.setScheme("http");
             coverartUrl.setHost(constCoverArtArchiveHost);
@@ -1069,7 +1070,7 @@ void CoverDialog::parseCoverArtArchiveQueryResponse(const QByteArray &resp)
 
     if (ok && parsed.contains("images")) {
         QVariantList images=parsed["images"].toList();
-        foreach (const QVariant &i, images) {
+        for (const QVariant &i: images) {
             QVariantMap im=i.toMap();
             if (im.contains("front") && im["front"].toBool() && im.contains("image") && im.contains("thumbnails")) {
                 QVariantMap thumb=im["thumbnails"].toMap();
@@ -1099,7 +1100,7 @@ void CoverDialog::parseSpotifyQueryResponse(const QByteArray &resp)
             const QString baseUrl=QLatin1String("https://embed.spotify.com/oembed/?url=http://open.spotify.com/")+QLatin1String(isArtist ? "artist/" : "album/");
             if (parsed.contains(key)) {
                 QVariantList results=parsed[key].toList();
-                foreach (const QVariant &res, results) {
+                for (const QVariant &res: results) {
                     QVariantMap item=res.toMap();
                     if (item.contains("href")) {
                         QString url=item["href"].toString();
@@ -1125,7 +1126,7 @@ void CoverDialog::parseITunesQueryResponse(const QByteArray &resp)
 
     if (ok && parsed.contains("results")) {
         QVariantList results=parsed["results"].toList();
-        foreach (const QVariant &res, results) {
+        for (const QVariant &res: results) {
             QVariantMap item=res.toMap();
             if (item.contains("artworkUrl100")) {
                 QString thumbUrl=item["artworkUrl100"].toString();
@@ -1144,7 +1145,7 @@ void CoverDialog::parseDeezerQueryResponse(const QByteArray &resp)
 
     if (ok && parsed.contains("data")) {
         QVariantList results=parsed["data"].toList();
-        foreach (const QVariant &res, results) {
+        for (const QVariant &res: results) {
             QVariantMap item=res.toMap();
             if (item.contains(key)) {
                 QString thumbUrl=item[key].toString();
@@ -1216,7 +1217,7 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
     QString destName;
     QString mpdDir;
     QString dirName;
-    bool saveInMpd=Settings::self()->storeCoversInMpdDir();
+    bool saveInMpd=!isArtist && Settings::self()->storeCoversInMpdDir();
 
     if (saveInMpd) {
         bool haveAbsPath=filePath.startsWith('/');
@@ -1232,7 +1233,7 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
         if (saveInMpd && !mpdDir.isEmpty() && dirName.startsWith(mpdDir) && 2==dirName.mid(mpdDir.length()).split('/', QString::SkipEmptyParts).count()) {
             QDir d(dirName);
             d.cdUp();
-            destName=d.absolutePath()+'/'+Covers::artistFileName(song)+src.mid(src.length()-4);
+            destName=d.absolutePath()+'/'+Covers::constArtistImage+src.mid(src.length()-4);
         } else {
             destName=Utils::cacheDir(Covers::constCoverDir, true)+Covers::encodeName(song.albumArtist())+src.mid(src.length()-4);
         }
@@ -1279,7 +1280,7 @@ void CoverDialog::dropEvent(QDropEvent *event)
         event->acceptProposedAction();
 
         QList<QUrl> urls(event->mimeData()->urls());
-        foreach (const QUrl &url, urls) {
+        for (const QUrl &url: urls) {
             if (url.scheme().isEmpty() || "file"==url.scheme()) {
                 QString path=url.path();
                 if (!currentLocalCovers.contains(path) && (path.endsWith(".jpg", Qt::CaseInsensitive) || path.endsWith(".png", Qt::CaseInsensitive))) {
@@ -1326,3 +1327,5 @@ void CoverDialog::addProvider(QMenu *mnu, const QString &name, int bit, int valu
     connect(act, SIGNAL(toggled(bool)), this, SLOT(updateProviders()));
     providers.insert(bit, act);
 }
+
+#include "moc_coverdialog.cpp"
