@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,6 +31,9 @@
 #ifdef ENABLE_SIMPLE_MPD_SUPPORT
 #include "mpd-interface/mpduser.h"
 #endif
+#ifdef AVAHI_FOUND
+#include "findmpddialog.h"
+#endif
 #include <QDir>
 #include <QStandardPaths>
 #include <QTimer>
@@ -38,7 +41,7 @@
 enum Pages {
     PAGE_INTRO,
     PAGE_CONNECTION,
-    PAGE_FILES,
+    PAGE_COVERS,
     PAGE_END
 };
 
@@ -70,13 +73,12 @@ InitialSettingsWizard::InitialSettingsWizard(QWidget *p)
     groupWarningIcon->setVisible(showGroupWarning);
     int iconSize=Icon::dlgIconSize();
     groupWarningIcon->setPixmap(Icon("dialog-warning").pixmap(iconSize, iconSize));
-    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
-    storeLyricsInMpdDir->setChecked(Settings::self()->storeLyricsInMpdDir());
-    storeBackdropsInMpdDir->setChecked(Settings::self()->storeBackdropsInMpdDir());
     introPage->setBackground(Icons::self()->appIcon);
     connectionPage->setBackground(Icons::self()->audioFileIcon);
-    filesPage->setBackground(Icons::self()->filesIcon);
+    coversPage->setBackground(Icons::self()->albumIconLarge);
     finishedPage->setBackground(Icon("dialog-ok"));
+    fetchCovers->setChecked(Settings::self()->fetchCovers());
+    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
 
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     introStack->setCurrentIndex(MPDUser::self()->isSupported() ? 1 : 0);
@@ -89,11 +91,12 @@ InitialSettingsWizard::InitialSettingsWizard(QWidget *p)
     #endif
 
     #ifndef Q_OS_WIN
-    QSize sz=size();
+    ensurePolished();
+    QSize sz=size()+QSize(64, 32);
     // Adjust size for high-DPI setups...
     bool highDpi=fontMetrics().height()>20;
     if (highDpi) {
-        foreach (int id, pageIds()) {
+        for (int id: pageIds()) {
             QWizardPage *p=QWizard::page(id);
             p->adjustSize();
             QSize ps=p->size();
@@ -112,13 +115,33 @@ InitialSettingsWizard::InitialSettingsWizard(QWidget *p)
     resize(sz);
     setMinimumSize(sz);
     #endif
-    httpNote->setOn(true);
+
+    #ifdef AVAHI_FOUND
+    discoveryButton = new QPushButton(tr("Discover..."), this);
+    hostLayout->insertWidget(hostLayout->count(), discoveryButton);
+    connect(discoveryButton, &QPushButton::clicked, this, &InitialSettingsWizard::detectMPDs);
+    #endif
 }
 
 InitialSettingsWizard::~InitialSettingsWizard()
 {
 }
 
+#ifdef AVAHI_FOUND
+void InitialSettingsWizard::adoptServerSettings(QString ip, QString p)
+{
+    host->setText(ip);
+    port->setValue(p.toInt());
+}
+
+void InitialSettingsWizard::detectMPDs()
+{
+    FindMpdDialog findMpdDlg(this);
+    QObject::connect(&findMpdDlg, &FindMpdDialog::serverChosen, this, &InitialSettingsWizard::adoptServerSettings);
+    findMpdDlg.exec();
+}
+#endif
+
 MPDConnectionDetails InitialSettingsWizard::getDetails()
 {
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
@@ -175,17 +198,13 @@ void InitialSettingsWizard::pageChanged(int p)
         controlNextButton();
         return;
     }
-    if (PAGE_FILES==p) {
+    if (PAGE_COVERS==p) {
         if (dir->text().trimmed().startsWith(QLatin1String("http:/"))) {
             storeCoversInMpdDir->setChecked(false);
-            storeLyricsInMpdDir->setChecked(false);
-            storeBackdropsInMpdDir->setChecked(false);
-            httpNote->setVisible(true);
+            fetchCovers->setChecked(true);
         } else {
             storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
-            storeLyricsInMpdDir->setChecked(Settings::self()->storeLyricsInMpdDir());
-            storeBackdropsInMpdDir->setChecked(Settings::self()->storeBackdropsInMpdDir());
-            httpNote->setVisible(false);
+            fetchCovers->setChecked(Settings::self()->fetchCovers());
         }
     }
     button(NextButton)->setEnabled(PAGE_END!=p);
@@ -217,8 +236,6 @@ void InitialSettingsWizard::controlNextButton()
 void InitialSettingsWizard::accept()
 {
     Settings::self()->saveConnectionDetails(getDetails());
-    Settings::self()->saveStoreCoversInMpdDir(storeCoversInMpdDir->isChecked());
-    Settings::self()->saveStoreLyricsInMpdDir(storeLyricsInMpdDir->isChecked());
 
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (basic->isChecked()) {
@@ -241,3 +258,5 @@ void InitialSettingsWizard::reject()
     QTimer::singleShot(0, qApp, SLOT(quit()));
     QDialog::reject();
 }
+
+#include "moc_initialsettingswizard.cpp"
