@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -60,7 +60,7 @@ static QString viewTypeString(ItemView::Mode mode)
 
 static void addViewTypes(QComboBox *box, QList<ItemView::Mode> modes)
 {
-    foreach (ItemView::Mode m, modes) {
+    for (ItemView::Mode m: modes) {
         box->addItem(viewTypeString(m), m);
     }
 }
@@ -105,6 +105,24 @@ static inline QString getStrValue(QComboBox *box)
 static const char * constValueProperty="value";
 static const char * constSep=",";
 
+class CoverNameValidator : public QValidator
+{
+    public:
+
+    CoverNameValidator(QObject *parent) : QValidator(parent) { }
+
+    State validate(QString &input, int &) const override
+    {
+        for (int i=0; i<input.length(); ++i) {
+            if (!input[i].isLetterOrNumber() && '%'!=input[i] && ' '!=input[i] && '-'!=input[i]) {
+                return Invalid;
+            }
+        }
+
+        return Acceptable;
+    }
+};
+
 InterfaceSettings::InterfaceSettings(QWidget *p)
     : QWidget(p)
     , loaded(false)
@@ -127,6 +145,8 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     setupUi(this);
     addCueSupportTypes(cueSupport);
     addViewTypes(playQueueView, QList<ItemView::Mode>() << ItemView::Mode_GroupedTree << ItemView::Mode_Table);
+    yearTag->addItem(tr("Use 'Year' tag to display & sort"));
+    yearTag->addItem(tr("Use 'Original Year' tag to display & sort, fallback to 'Year' if not set"));
 
     addView(tr("Play Queue"), QLatin1String("PlayQueuePage"));
     addView(tr("Library"), QLatin1String("LibraryPage"));
@@ -165,7 +185,7 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     connect(playQueueBackground_none, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_cover, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_custom, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
-
+    connect(storeCoversInMpdDir, SIGNAL(toggled(bool)), this, SLOT(storeCoversInMpdDirToggled()));
     if (!enableNotifications) {
         REMOVE(systemTrayPopup)
     }
@@ -193,6 +213,17 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
         connect(systemTrayPopup, SIGNAL(toggled(bool)), SLOT(systemTrayPopupToggled()));
     }
     #endif
+    coverFilenameLabel->setToolTip(coverFilename->toolTip());
+    coverFilename->setValidator(new CoverNameValidator(this));
+    #ifdef ENABLE_DEVICES_SUPPORT
+    coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
+                                   "<code>cover</code>. This filename is used when downloading covers, or when adding "
+                                   "music to your library from devices."));
+    #else
+    coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
+                                   "<code>cover</code>. This filename is used when downloading covers."));
+    #endif
+    setMinimumSize(750, 650);
 }
 
 void InterfaceSettings::load()
@@ -201,6 +232,7 @@ void InterfaceSettings::load()
     composerGenres->setText(QStringList(Settings::self()->composerGenres().toList()).join(QString(constSep)));
     singleTracksFolders->setText(QStringList(Settings::self()->singleTracksFolders().toList()).join(QString(constSep)));
     selectEntry(cueSupport, Settings::self()->cueSupport());
+    yearTag->setCurrentIndex(Settings::self()->useOriginalYear() ? 1 : 0);
     #ifdef ENABLE_DEVICES_SUPPORT
     showDeleteAction->setChecked(Settings::self()->showDeleteAction());
     #endif
@@ -226,6 +258,7 @@ void InterfaceSettings::load()
     showStopButton->setChecked(Settings::self()->showStopButton());
     showCoverWidget->setChecked(Settings::self()->showCoverWidget());
     showRatingWidget->setChecked(Settings::self()->showRatingWidget());
+    showTechnicalInfo->setChecked(Settings::self()->showTechnicalInfo());
     if (systemTrayCheckBox) {
         systemTrayCheckBox->setChecked(Settings::self()->useSystemTray());
         if (minimiseOnClose) {
@@ -246,6 +279,8 @@ void InterfaceSettings::load()
         systemTrayPopup->setChecked(Settings::self()->showPopups());
     }
     fetchCovers->setChecked(Settings::self()->fetchCovers());
+    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
+    coverFilename->setText(Settings::self()->coverFilename());
 
     QStringList hiddenPages=Settings::self()->hiddenPages();
     for (int i=0; i<views->count(); ++i) {
@@ -258,6 +293,7 @@ void InterfaceSettings::load()
     sbIconsOnly->setChecked(sidebar&FancyTabWidget::IconOnly);
     sbAutoHide->setChecked(Settings::self()->splitterAutoHide());
     sbAutoHideChanged();
+    responsiveSidebar->setChecked(Settings::self()->responsiveSidebar());
     viewItemChanged(views->item(0));
     setPlayQueueBackgroundOpacityLabel();
     setPlayQueueBackgroundBlurLabel();
@@ -271,7 +307,7 @@ static QSet<QString> toSet(const QString &str)
 {
     QStringList parts=str.split(constSep, QString::SkipEmptyParts);
     QSet<QString> set;
-    foreach (QString s, parts) {
+    for (QString s: parts) {
         set.insert(s.trimmed());
     }
     return set;
@@ -283,6 +319,7 @@ void InterfaceSettings::save()
     Settings::self()->saveComposerGenres(toSet(composerGenres->text()));
     Settings::self()->saveSingleTracksFolders(toSet(singleTracksFolders->text()));
     Settings::self()->saveCueSupport((MPDParseUtils::CueSupport)(cueSupport->itemData(cueSupport->currentIndex()).toInt()));
+    Settings::self()->saveUseOriginalYear(1==yearTag->currentIndex());
     #ifdef ENABLE_DEVICES_SUPPORT
     Settings::self()->saveShowDeleteAction(showDeleteAction->isChecked());
     #endif
@@ -309,6 +346,7 @@ void InterfaceSettings::save()
     Settings::self()->saveShowStopButton(showStopButton->isChecked());
     Settings::self()->saveShowCoverWidget(showCoverWidget->isChecked());
     Settings::self()->saveShowRatingWidget(showRatingWidget->isChecked());
+    Settings::self()->saveShowTechnicalInfo(showTechnicalInfo->isChecked());
     Settings::self()->saveUseSystemTray(systemTrayCheckBox && systemTrayCheckBox->isChecked());
     Settings::self()->saveShowPopups(systemTrayPopup && systemTrayPopup->isChecked());
     Settings::self()->saveMinimiseOnClose(minimiseOnClose && minimiseOnClose->isChecked());
@@ -320,6 +358,8 @@ void InterfaceSettings::save()
         Settings::self()->saveStartupState(Settings::SS_Previous);
     }
     Settings::self()->saveFetchCovers(fetchCovers->isChecked());
+    Settings::self()->saveStoreCoversInMpdDir(storeCoversInMpdDir->isChecked());
+    Settings::self()->saveCoverFilename(coverFilename->text().trimmed());
     if (loaded && lang) {
         Settings::self()->saveLang(lang->itemData(lang->currentIndex()).toString());
     }
@@ -340,6 +380,7 @@ void InterfaceSettings::save()
     }
     Settings::self()->saveSidebar(sidebar);
     Settings::self()->saveSplitterAutoHide(sbAutoHide->isChecked());
+    Settings::self()->saveResponsiveSidebar(responsiveSidebar->isChecked());
     if (enableMpris) {
         Settings::self()->saveMpris(enableMpris->isChecked());
     }
@@ -356,7 +397,7 @@ static QSet<QString> translationCodes(const QString &dir)
     QDir d(dir);
     QStringList installed(d.entryList(QStringList() << "*.qm"));
     QRegExp langRegExp("^cantata_(.*).qm$");
-    foreach (const QString &filename, installed) {
+    for (const QString &filename: installed) {
         if (langRegExp.exactMatch(filename)) {
             codes.insert(langRegExp.cap(1));
         }
@@ -378,7 +419,7 @@ void InterfaceSettings::showEvent(QShowEvent *e)
         transCodes+=translationCodes(CANTATA_SYS_TRANS_DIR);
         #endif
 
-        foreach (const QString &code, transCodes) {
+        for (const QString &code: transCodes) {
             QString langName = QLocale::languageToString(QLocale(code).language());
             QString nativeName = QLocale(code).nativeLanguageName();
             if (!nativeName.isEmpty()) {
@@ -394,7 +435,7 @@ void InterfaceSettings::showEvent(QShowEvent *e)
         qStableSort(names.begin(), names.end(), localeAwareCompare);
         lang->addItem(tr("System default"), QString());
         lang->setCurrentIndex(0);
-        foreach (const QString &name, names) {
+        for (const QString &name: names) {
             lang->addItem(name, langMap[name]);
             if (langMap[name]==current) {
                 lang->setCurrentIndex(lang->count()-1);
@@ -544,3 +585,10 @@ void InterfaceSettings::systemTrayPopupToggled()
         systemTrayCheckBox->setChecked(true);
     }
 }
+
+void InterfaceSettings::storeCoversInMpdDirToggled()
+{
+    storeCoversInMpdDirLabel->setOn(storeCoversInMpdDir->isChecked() && storeCoversInMpdDir->isChecked()!=Settings::self()->storeCoversInMpdDir());
+}
+
+#include "moc_interfacesettings.cpp"
