@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,7 +31,7 @@ class MultiMediaKeysInterface : public QObject
     Q_OBJECT
 public:
     MultiMediaKeysInterface(QObject *p) : QObject(p) { }
-    ~MultiMediaKeysInterface() { }
+    ~MultiMediaKeysInterface() override { }
 
     virtual bool activate()=0;
     virtual void deactivate()=0;
