@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,6 +31,9 @@
 #include "mpd-interface/mpduser.h"
 #endif
 #include "support/utils.h"
+#ifdef AVAHI_FOUND
+#include "findmpddialog.h"
+#endif
 #include <QDir>
 #include <QComboBox>
 #include <QPushButton>
@@ -43,23 +46,7 @@
     w->deleteLater(); \
     w=0;
 
-class CoverNameValidator : public QValidator
-{
-    public:
 
-    CoverNameValidator(QObject *parent) : QValidator(parent) { }
-
-    State validate(QString &input, int &) const
-    {
-        for (int i=0; i<input.length(); ++i) {
-            if (!input[i].isLetterOrNumber() && '%'!=input[i] && ' '!=input[i] && '-'!=input[i]) {
-                return Invalid;
-            }
-        }
-
-        return Acceptable;
-    }
-};
 
 #ifdef ENABLE_SIMPLE_MPD_SUPPORT
 class CollectionNameValidator : public QValidator
@@ -68,7 +55,7 @@ class CollectionNameValidator : public QValidator
 
     CollectionNameValidator(QObject *parent) : QValidator(parent) { }
 
-    State validate(QString &input, int &) const
+    State validate(QString &input, int &) const override
     {
         return input.startsWith(MPDUser::constName) ? Invalid : Acceptable;
     }
@@ -90,6 +77,9 @@ ServerSettings::ServerSettings(QWidget *p)
     musicFolderNoteLabel->appendText(QLatin1String("<i> ")+tr("This folder will also be used to locate music files "
                                      "for tag-editing, replay gain, etc.")+QLatin1String("</i>"));
     #endif
+    musicFolderNoteLabel->appendText(QLatin1String("<br/><i><a href=\"https://github.com/CDrummond/cantata/wiki/Accessing-music-files,-and-covers\">")+
+                                     tr("More information")+QLatin1String("</a></i>"));
+
     connect(combo, SIGNAL(activated(int)), SLOT(showDetails(int)));
     connect(removeButton, SIGNAL(clicked(bool)), SLOT(remove()));
     connect(addButton, SIGNAL(clicked(bool)), SLOT(add()));
@@ -103,11 +93,7 @@ ServerSettings::ServerSettings(QWidget *p)
     #if defined Q_OS_WIN
     hostLabel->setText(tr("Host:"));
     #endif
-    basicCoverName->setToolTip(coverName->toolTip());
-    basicCoverNameLabel->setToolTip(coverName->toolTip());
-    coverNameLabel->setToolTip(coverName->toolTip());
-    coverName->setValidator(new CoverNameValidator(this));
-    basicCoverName->setValidator(new CoverNameValidator(this));
+
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     name->setValidator(new CollectionNameValidator(this));
     #endif
@@ -116,10 +102,24 @@ ServerSettings::ServerSettings(QWidget *p)
     REMOVE(streamUrl)
     REMOVE(streamUrlNoteLabel)
     #endif
+    #ifdef ENABLE_HTTP_SERVER
+    allowLocalStreamingLabel->setToolTip(allowLocalStreaming->toolTip());
+    #else
+    REMOVE(allowLocalStreaming)
+    REMOVE(allowLocalStreamingLabel)
+    REMOVE(allowLocalStreamingNoteLabel)
+    #endif
+    autoUpdate->setToolTip(autoUpdate->toolTip());
 
     #ifdef Q_OS_MAC
     expandingSpacer->changeSize(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
     #endif
+
+    #ifdef AVAHI_FOUND
+    discoveryButton = new QPushButton(tr("Discover..."), this);
+    hostLayout->insertWidget(hostLayout->count(), discoveryButton);
+    connect(discoveryButton, &QPushButton::clicked, this, &ServerSettings::detectMPDs);
+    #endif
 }
 
 void ServerSettings::load()
@@ -131,7 +131,7 @@ void ServerSettings::load()
     combo->clear();
     int idx=0;
     haveBasicCollection=false;
-    foreach (MPDConnectionDetails d, all) {
+    for (MPDConnectionDetails d: all) {
         combo->addItem(d.getName(), d.name);
         if (d.name==currentCon) {
             prevIndex=idx;
@@ -166,7 +166,7 @@ void ServerSettings::save()
     QList<MPDConnectionDetails> existingInConfig=Settings::self()->allConnections();
     QList<Collection> toAdd;
 
-    foreach (const Collection &c, collections) {
+    for (const Collection &c: collections) {
         bool found=false;
         for (int i=0; i<existingInConfig.count(); ++i) {
             MPDConnectionDetails e=existingInConfig.at(i);
@@ -174,10 +174,12 @@ void ServerSettings::save()
                 existingInConfig.removeAt(i);
                 found=true;
                 if (c.details.hostname!=e.hostname || c.details.port!=e.port || c.details.password!=e.password ||
-                    c.details.dir!=e.dir || c.details.coverName!=e.coverName
+                    c.details.dir!=e.dir
                     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
                     || c.details.streamUrl!=e.streamUrl
                     #endif
+                    || c.details.allowLocalStreaming!=e.allowLocalStreaming
+                    || c.details.autoUpdate!=e.autoUpdate
                     ) {
                     toAdd.append(c);
                 }
@@ -193,11 +195,11 @@ void ServerSettings::save()
         #endif
     }
 
-    foreach (const MPDConnectionDetails &c, existingInConfig) {
+    for (const MPDConnectionDetails &c: existingInConfig) {
         Settings::self()->removeConnectionDetails(c.name);
     }
 
-    foreach (const Collection &c, toAdd) {
+    for (const Collection &c: toAdd) {
         Settings::self()->saveConnectionDetails(c.details);
         c.namingOpts.save(MPDConnectionDetails::configGroupName(c.details.name), true);
     }
@@ -279,6 +281,9 @@ void ServerSettings::add()
         details.port=6600;
         details.hostname=QLatin1String("localhost");
         details.dir=QLatin1String("/var/lib/mpd/music/");
+        // disable local streaming by default
+        details.allowLocalStreaming=false;
+        details.autoUpdate=false;
         combo->addItem(details.name);
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     } else {
@@ -337,6 +342,21 @@ void ServerSettings::basicDirChanged()
     }
 }
 
+#ifdef AVAHI_FOUND
+void ServerSettings::adoptServerSettings(QString ip, QString p)
+{
+    host->setText(ip);
+    port->setValue(p.toInt());
+}
+
+void ServerSettings::detectMPDs()
+{
+    FindMpdDialog findMpdDlg(this);
+    QObject::connect(&findMpdDlg, &FindMpdDialog::serverChosen, this, &ServerSettings::adoptServerSettings);
+    findMpdDlg.exec();
+}
+#endif
+
 QString ServerSettings::generateName(int ignore) const
 {
     QString n;
@@ -362,7 +382,6 @@ void ServerSettings::setDetails(const MPDConnectionDetails &details)
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (details.name==MPDUser::constName) {
         basicDir->setText(Utils::convertPathForDisplay(details.dir));
-        basicCoverName->setText(details.coverName);
         stackedWidget->setCurrentIndex(1);
     } else {
     #endif
@@ -371,10 +390,13 @@ void ServerSettings::setDetails(const MPDConnectionDetails &details)
         port->setValue(details.port);
         password->setText(details.password);
         dir->setText(Utils::convertPathForDisplay(details.dir));
-        coverName->setText(details.coverName);
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         streamUrl->setText(details.streamUrl);
         #endif
+        #ifdef ENABLE_HTTP_SERVER
+        allowLocalStreaming->setChecked(details.allowLocalStreaming);
+        #endif
+        autoUpdate->setChecked(details.autoUpdate);
         stackedWidget->setCurrentIndex(0);
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     }
@@ -395,19 +417,23 @@ MPDConnectionDetails ServerSettings::getDetails() const
         details.port=port->value();
         details.password=password->text();
         details.dir=Utils::convertPathFromDisplay(dir->text());
-        details.coverName=coverName->text().trimmed();
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=streamUrl->text().trimmed();
         #endif
+        #ifdef ENABLE_HTTP_SERVER
+        details.allowLocalStreaming=allowLocalStreaming->checkState() == Qt::Checked;
+        #endif
+        details.autoUpdate=autoUpdate->checkState() == Qt::Checked;
     }
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     else {
         details=MPDUser::self()->details(true);
         details.dir=Utils::convertPathFromDisplay(basicDir->text());
-        details.coverName=basicCoverName->text().trimmed();
         MPDUser::self()->setMusicFolder(details.dir);
     }
     #endif
     details.setDirReadable();
     return details;
 }
+
+#include "moc_serversettings.cpp"
