@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -166,20 +166,24 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
         details.port=grp.get("port", name.isEmpty() ? mpdDefaults.port : 6600);
         details.dir=grp.getDirPath("dir", name.isEmpty() ? mpdDefaults.dir : "/var/lib/mpd/music");
         details.password=grp.get("passwd", name.isEmpty() ? mpdDefaults.passwd : QString());
-        details.coverName=grp.get("coverName", QString());
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=grp.get("streamUrl", QString());
         #endif
         details.replayGain=grp.get("replayGain", QString());
+        // if the setting hasn't been set before, we set it to true to allow
+        // for easy migration of existing settings
+        details.allowLocalStreaming=grp.get("allowLocalStreaming", true);
+        details.autoUpdate=grp.get("autoUpdate", false);
     } else {
         details.hostname=mpdDefaults.host;
         details.port=mpdDefaults.port;
         details.dir=mpdDefaults.dir;
         details.password=mpdDefaults.passwd;
-        details.coverName=QString();
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=QString();
         #endif
+        details.allowLocalStreaming=true;
+        details.autoUpdate=false;
     }
     details.setDirReadable();
     return details;
@@ -189,7 +193,7 @@ QList<MPDConnectionDetails> Settings::allConnections()
 {
     QStringList groups=cfg.childGroups();
     QList<MPDConnectionDetails> connections;
-    foreach (const QString &grp, groups) {
+    for (const QString &grp: groups) {
         if (cfg.hasGroup(grp) && grp.startsWith("Connection")) {
             connections.append(connectionDetails(grp=="Connection" ? QString() : grp.mid(11)));
         }
@@ -240,6 +244,11 @@ QSize Settings::mainWindowCollapsedSize()
     return cfg.get("mainWindowCollapsedSize", QSize());
 }
 
+bool Settings::maximized()
+{
+    return cfg.get("maximized", false);
+}
+
 bool Settings::useSystemTray()
 {
     return cfg.get("useSystemTray", false);
@@ -262,27 +271,29 @@ bool Settings::stopOnExit()
 
 bool Settings::storeCoversInMpdDir()
 {
-    return cfg.get("storeCoversInMpdDir", true);
+    return cfg.get("storeCoversInMpdDir", false);
 }
 
-bool Settings::storeLyricsInMpdDir()
+QString Settings::coverFilename()
 {
-    return cfg.get("storeLyricsInMpdDir", true);
-}
-
-bool Settings::storeStreamsInMpdDir()
-{
-    #ifdef Q_OS_WIN
-    bool def=false;
-    #else
-    bool def=version()>=CANTATA_MAKE_VERSION(0, 9, 50);
-    #endif
-    return cfg.get("storeStreamsInMpdDir", def);
-}
-
-bool Settings::storeBackdropsInMpdDir()
-{
-    return cfg.get("storeBackdropsInMpdDir", false);
+    QString name=cfg.get("coverFilename", QString());
+    // name is empty, so for old configs try to get from MPD settings
+    if (name.isEmpty() && version()<CANTATA_MAKE_VERSION(2, 2, 51)) {
+        QStringList groups=cfg.childGroups();
+        QList<MPDConnectionDetails> connections;
+        for (const QString &grp: groups) {
+            if (cfg.hasGroup(grp) && grp.startsWith("Connection")) {
+                Configuration cfg(grp);
+                name=cfg.get("coverName", QString());
+                if (!name.isEmpty()) {
+                    // Use 1st non-empty
+                    saveCoverFilename(name);
+                    return name;
+                }
+            }
+        }
+    }
+    return name;
 }
 
 int Settings::sidebar()
@@ -602,11 +613,6 @@ int Settings::podcastAutoDownloadLimit()
     return cfg.get("podcastAutoDownloadLimit", 0, 0, 1000);
 }
 
-bool Settings::networkAccessEnabled()
-{
-    return cfg.get("networkAccessEnabled", true);
-}
-
 int Settings::volumeStep()
 {
     return cfg.get("volumeStep", 5, 1, 20);
@@ -647,6 +653,11 @@ bool Settings::showRatingWidget()
     return cfg.get("showRatingWidget", false);
 }
 
+bool Settings::showTechnicalInfo()
+{
+    return cfg.get("showTechnicalInfo", false);
+}
+
 bool Settings::infoTooltips()
 {
     return cfg.get("infoTooltips", true);
@@ -667,6 +678,23 @@ QString Settings::style()
     return cfg.get("style", QString());
 }
 
+#if !defined Q_OS_WIN && !defined Q_OS_MAC
+bool Settings::showMenubar()
+{
+    return cfg.get("showMenubar", false);
+}
+#endif
+
+bool Settings::useOriginalYear()
+{
+    return cfg.get("useOriginalYear", false);
+}
+
+bool Settings::responsiveSidebar()
+{
+    return cfg.get("responsiveSidebar", true);
+}
+
 void Settings::removeConnectionDetails(const QString &v)
 {
     if (v==currentConnection()) {
@@ -691,10 +719,11 @@ void Settings::saveConnectionDetails(const MPDConnectionDetails &v)
     grp.set("port", (int)v.port);
     grp.setDirPath("dir", v.dir);
     grp.set("passwd", v.password);
-    grp.set("coverName", v.coverName);
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
     grp.set("streamUrl", v.streamUrl);
     #endif
+    grp.set("allowLocalStreaming", v.allowLocalStreaming);
+    grp.set("autoUpdate", v.autoUpdate);
 }
 
 void Settings::saveCurrentConnection(const QString &v)
@@ -732,6 +761,11 @@ void Settings::saveMainWindowSize(const QSize &v)
     cfg.set("mainWindowSize", v);
 }
 
+void Settings::saveMaximized(bool v)
+{
+    cfg.set("maximized", v);
+}
+
 void Settings::saveMainWindowCollapsedSize(const QSize &v)
 {
     if (v.width()>16 && v.height()>16) {
@@ -764,14 +798,9 @@ void Settings::saveStoreCoversInMpdDir(bool v)
     cfg.set("storeCoversInMpdDir", v);
 }
 
-void Settings::saveStoreLyricsInMpdDir(bool v)
+void Settings::saveCoverFilename(const QString &v)
 {
-    cfg.set("storeLyricsInMpdDir", v);
-}
-
-void Settings::saveStoreBackdropsInMpdDir(bool v)
-{
-    cfg.set("storeBackdropsInMpdDir", v);
+    cfg.set("coverFilename", v);
 }
 
 void Settings::saveSidebar(int v)
@@ -1059,6 +1088,11 @@ void Settings::savePodcastAutoDownloadLimit(int v)
     cfg.set("podcastAutoDownloadLimit", v);
 }
 
+void Settings::saveVolumeStep(int v)
+{
+    cfg.set("volumeStep", v);
+}
+
 void Settings::saveStartupState(int v)
 {
     cfg.set("startupState", getStartupStateStr((StartupState)v));
@@ -1094,6 +1128,11 @@ void Settings::saveShowRatingWidget(bool v)
     cfg.set("showRatingWidget", v);
 }
 
+void Settings::saveShowTechnicalInfo(bool v)
+{
+    cfg.set("showTechnicalInfo", v);
+}
+
 void Settings::saveInfoTooltips(bool v)
 {
     cfg.set("infoTooltips", v);
@@ -1121,6 +1160,23 @@ void Settings::saveStyle(const QString &v)
     cfg.set("style", v);
 }
 
+#if !defined Q_OS_WIN && !defined Q_OS_MAC
+void Settings::saveShowMenubar(bool v)
+{
+    cfg.set("showMenubar", v);
+}
+#endif
+
+void Settings::saveUseOriginalYear(bool v)
+{
+    cfg.set("useOriginalYear", v);
+}
+
+void Settings::saveResponsiveSidebar(bool v)
+{
+    cfg.set("responsiveSidebar", v);
+}
+
 void Settings::save()
 {
     if (AP_NotConfigured!=state) {
