@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -54,14 +54,13 @@ public:
     bool splitterAutoHide();
     QSize mainWindowSize();
     QSize mainWindowCollapsedSize();
+    bool maximized();
     bool useSystemTray();
     bool minimiseOnClose();
     bool showPopups();
     bool stopOnExit();
     bool storeCoversInMpdDir();
-    bool storeLyricsInMpdDir();
-    bool storeStreamsInMpdDir();
-    bool storeBackdropsInMpdDir();
+    QString coverFilename();
     int sidebar();
     QSet<QString> composerGenres();
     QSet<QString> singleTracksFolders();
@@ -129,7 +128,6 @@ public:
     QDateTime lastRssUpdate();
     QString podcastDownloadPath();
     int podcastAutoDownloadLimit();
-    bool networkAccessEnabled();
     int volumeStep();
     StartupState startupState();
     QString searchCategory();
@@ -138,10 +136,16 @@ public:
     bool showCoverWidget();
     bool showStopButton();
     bool showRatingWidget();
+    bool showTechnicalInfo();
     bool infoTooltips();
     QSet<QString> ignorePrefixes();
     bool mpris();
     QString style();
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    bool showMenubar();
+    #endif
+    bool useOriginalYear();
+    bool responsiveSidebar();
 
     void removeConnectionDetails(const QString &v);
     void saveConnectionDetails(const MPDConnectionDetails &v);
@@ -154,12 +158,12 @@ public:
     void saveSplitterAutoHide(bool v);
     void saveMainWindowSize(const QSize &v);
     void saveMainWindowCollapsedSize(const QSize &v);
+    void saveMaximized(bool v);
     void saveUseSystemTray(bool v);
     void saveMinimiseOnClose(bool v);
     void saveShowPopups(bool v);
     void saveStoreCoversInMpdDir(bool v);
-    void saveStoreLyricsInMpdDir(bool v);
-    void saveStoreBackdropsInMpdDir(bool v);
+    void saveCoverFilename(const QString &v);
     void saveLibraryArtistImage(bool v);
     void saveSidebar(int v);
     void saveComposerGenres(const QSet<QString> &v);
@@ -227,6 +231,7 @@ public:
     void saveLastRssUpdate(const QDateTime &v);
     void savePodcastDownloadPath(const QString &v);
     void savePodcastAutoDownloadLimit(int v);
+    void saveVolumeStep(int v);
     void saveStartupState(int v);
     void saveSearchCategory(const QString &v);
     void saveFetchCovers(bool v);
@@ -234,11 +239,17 @@ public:
     void saveShowCoverWidget(bool v);
     void saveShowStopButton(bool v);
     void saveShowRatingWidget(bool v);
+    void saveShowTechnicalInfo(bool v);
     void saveInfoTooltips(bool v);
     void saveIgnorePrefixes(const QSet<QString> &v);
     void saveMpris(bool v);
     void saveReplayGain(const QString &conn, const QString &v);
     void saveStyle(const QString &v);
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    void saveShowMenubar(bool v);
+    #endif
+    void saveUseOriginalYear(bool v);
+    void saveResponsiveSidebar(bool v);
     void save();
     void clearVersion();
 
