@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -46,7 +46,7 @@ class VolumeSliderEventHandler : public QObject
 public:
     VolumeSliderEventHandler(QObject *p) : QObject(p) { }
 protected:
-    bool eventFilter(QObject *obj, QEvent *event)
+    bool eventFilter(QObject *obj, QEvent *event) override
     {
         if (QEvent::Wheel==event->type()) {
             int numDegrees = static_cast<QWheelEvent *>(event)->delta() / 8;
@@ -72,13 +72,13 @@ TrayItem::TrayItem(MainWindow *p)
     : QObject(p)
     #ifndef Q_OS_MAC
     , mw(p)
-    , trayItem(0)
-    , trayItemMenu(0)
+    , trayItem(nullptr)
+    , trayItemMenu(nullptr)
     #ifdef QT_QTDBUS_FOUND
-    , notification(0)
+    , notification(nullptr)
     #endif
-    , connectionsAction(0)
-    , outputsAction(0)
+    , connectionsAction(nullptr)
+    , outputsAction(nullptr)
     #endif
 {
 }
@@ -117,9 +117,9 @@ void TrayItem::setup()
         if (trayItem) {
             trayItem->setVisible(false);
             trayItem->deleteLater();
-            trayItem=0;
+            trayItem=nullptr;
             trayItemMenu->deleteLater();
-            trayItemMenu=0;
+            trayItemMenu=nullptr;
         }
         return;
     }
@@ -146,7 +146,7 @@ void TrayItem::setup()
 
     trayItem = new QSystemTrayIcon(this);
     trayItem->installEventFilter(new VolumeSliderEventHandler(this));
-    trayItemMenu = new QMenu(0);
+    trayItemMenu = new QMenu(nullptr);
     trayItemMenu->addAction(StdActions::self()->prevTrackAction);
     trayItemMenu->addAction(StdActions::self()->playPauseTrackAction);
     trayItemMenu->addAction(StdActions::self()->stopPlaybackAction);
@@ -168,7 +168,7 @@ void TrayItem::setup()
     // add icon files here...
     if (icon.isNull()) {
         QStringList sizes=QStringList() << "16" << "22" << "24" << "32" << "48" << "64";
-        foreach (const QString &s, sizes) {
+        for (const QString &s: sizes) {
             icon.addFile(QLatin1String(ICON_INSTALL_PREFIX "/")+s+QLatin1Char('x')+s+QLatin1String("/apps/cantata.png"));
         }
 
@@ -212,11 +212,7 @@ void TrayItem::songChanged(const Song &song, bool isPlaying)
                         ? !song.title.isEmpty() && !song.name().isEmpty()
                         : !song.title.isEmpty() && !song.artist.isEmpty() && !song.album.isEmpty();
         if (useable) {
-            QString text=song.describe(false);
-            if (song.time>0) {
-                text+=QString(" – ")+Utils::formatTime(song.time);
-            }
-            MacNotify::showMessage(tr("Now playing"), text, CurrentCover::self()->image());
+            MacNotify::showMessage(song.mainText(), song.subText(), CurrentCover::self()->image());
         }
     }
     #else
@@ -225,20 +221,13 @@ void TrayItem::songChanged(const Song &song, bool isPlaying)
                         ? !song.title.isEmpty() && !song.name().isEmpty()
                         : !song.title.isEmpty() && !song.artist.isEmpty() && !song.album.isEmpty();
         if (useable) {
-            QString text=song.describe(false);
-            if (song.time>0) {
-                text+=QString(" – ")+Utils::formatTime(song.time);
-            }
-
             if (trayItem) {
+                trayItem->setToolTip(QLatin1String("Cantata\n\n")+song.mainText()+"\n"+song.subText());
                 #if defined Q_OS_WIN || defined Q_OS_MAC || !defined QT_QTDBUS_FOUND
-                trayItem->setToolTip(tr("Cantata")+"\n\n"+text);
                 // The pure Qt implementation needs both the tray icon and the setting checked.
                 if (Settings::self()->showPopups() && isPlaying) {
-                    trayItem->showMessage(tr("Now playing"), text, QSystemTrayIcon::Information, 5000);
+                    trayItem->showMessage(song.mainText(), song.subText(), QSystemTrayIcon::Information, 5000);
                 }
-                #else
-                trayItem->setToolTip(tr("Cantata")+"\n\n"+text);
                 #endif
             }
             #ifdef QT_QTDBUS_FOUND
@@ -246,11 +235,11 @@ void TrayItem::songChanged(const Song &song, bool isPlaying)
                 if (!notification) {
                     notification=new Notify(this);
                 }
-                notification->show(tr("Now playing"), text, CurrentCover::self()->image());
+                notification->show(song.mainText(), song.subText(), CurrentCover::self()->image());
             }
             #endif
         } else if (trayItem) {
-            trayItem->setToolTip(tr("Cantata"));
+            trayItem->setToolTip(QLatin1String("Cantata"));
         }
     }
     #endif
@@ -265,12 +254,12 @@ static void copyMenu(Action *from, Action *to)
     to->setVisible(from->isVisible());
     if (to->isVisible()) {
         if (!to->menu()) {
-            to->setMenu(new QMenu(0));
+            to->setMenu(new QMenu(nullptr));
         }
         QMenu *m=to->menu();
         m->clear();
 
-        foreach (QAction *act, from->menu()->actions()) {
+        for (QAction *act: from->menu()->actions()) {
             m->addAction(act);
         }
     }
@@ -290,3 +279,5 @@ void TrayItem::updateOutputs()
     copyMenu(mw->outputsAction, outputsAction);
     #endif
 }
+
+#include "moc_trayitem.cpp"
