@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -60,10 +60,10 @@ GLOBAL_STATIC(HttpServer, instance)
 
 #ifdef ENABLE_HTTP_SERVER
 HttpServer::HttpServer()
-    : QObject(0)
-    , thread(0)
-    , socket(0)
-    , closeTimer(0)
+    : QObject(nullptr)
+    , thread(nullptr)
+    , socket(nullptr)
+    , closeTimer(nullptr)
 {
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QList<Song>,bool)), this, SLOT(cantataStreams(QList<Song>,bool)));
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QStringList)), this, SLOT(cantataStreams(QStringList)));
@@ -71,6 +71,12 @@ HttpServer::HttpServer()
     connect(MPDConnection::self(), SIGNAL(ifaceIp(QString)), this, SLOT(ifaceIp(QString)));
 }
 
+bool HttpServer::isAlive() const
+{
+    // started on demand, but only start if allowed
+    return Settings::self()->connectionDetails().allowLocalStreaming;
+}
+
 bool HttpServer::start()
 {
     if (closeTimer) {
@@ -85,7 +91,7 @@ bool HttpServer::start()
 
     DBUG << "open new socket";
     quint16 prevPort=Settings::self()->httpAllocatedPort();
-    bool newThread=0==thread;
+    bool newThread=nullptr==thread;
     if (newThread) {
         thread=new Thread("HttpServer");
     }
@@ -108,7 +114,7 @@ void HttpServer::stop()
     if (socket) {
         DBUG;
         emit terminateSocket();
-        socket=0;
+        socket=nullptr;
     }
 }
 
@@ -120,7 +126,7 @@ void HttpServer::readConfig()
         return;
     }
 
-    bool wasStarted=0!=socket;
+    bool wasStarted=nullptr!=socket;
     stop();
     if (wasStarted) {
         start();
@@ -139,11 +145,11 @@ QString HttpServer::address() const
 
 bool HttpServer::isOurs(const QString &url) const
 {
-    if (0==socket || !url.startsWith(QLatin1String("http://"))) {
+    if (nullptr==socket || !url.startsWith(QLatin1String("http://"))) {
         return false;
     }
 
-    foreach (const QString &ip, ipAddresses) {
+    for (const QString &ip: ipAddresses) {
         if (url.startsWith(serverUrl(ip, socket->serverPort()))) {
             return true;
         }
@@ -229,7 +235,7 @@ QByteArray HttpServer::encodeUrl(const QString &file)
     #endif
     s.file=file;
     #endif
-    return encodeUrl(s);
+    return /*s.isEmpty() ? QByteArray() :*/ encodeUrl(s);
 }
 
 Song HttpServer::decodeUrl(const QString &url) const
@@ -309,7 +315,7 @@ void HttpServer::startCloseTimer()
 void HttpServer::cantataStreams(const QStringList &files)
 {
     DBUG << files;
-    foreach (const QString &f, files) {
+    for (const QString &f: files) {
         Song s=HttpServer::self()->decodeUrl(f);
         if (s.isCantataStream() || s.isCdda()) {
             start();
@@ -325,7 +331,7 @@ void HttpServer::cantataStreams(const QList<Song> &songs, bool isUpdate)
         streamIds.clear();
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         streamIds.insert(s.id);
     }
 
@@ -355,3 +361,5 @@ void HttpServer::ifaceIp(const QString &ip)
 }
 
 #endif
+
+#include "moc_httpserver.cpp"
