@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -156,7 +156,7 @@ static QList<QByteArray> split(const QByteArray &a)
 
 static bool isFromMpd(const QStringList &params)
 {
-    foreach (const QString &str, params) {
+    for (const QString &str: params) {
         if (str.startsWith("User-Agent:") && str.contains("Music Player Daemon")) {
             return true;
         }
@@ -166,7 +166,7 @@ static bool isFromMpd(const QStringList &params)
 
 static void getRange(const QStringList &params, qint32 &from, qint32 &to)
 {
-    foreach (const QString &str, params) {
+    for (const QString &str: params) {
         if (str.startsWith("Range:")) {
             int start=str.indexOf("bytes=");
             if (start>0) {
@@ -184,7 +184,7 @@ static void getRange(const QStringList &params, qint32 &from, qint32 &to)
 }
 
 HttpSocket::HttpSocket(const QString &iface, quint16 port)
-    : QTcpServer(0)
+    : QTcpServer(nullptr)
     , cfgInterface(iface)
     , terminated(false)
 {
@@ -261,6 +261,9 @@ void HttpSocket::readClient()
     }
 
     QTcpSocket *socket = static_cast<QTcpSocket *>(sender());
+    if (!socket) {
+        return;
+    }
 
     if (socket->bytesAvailable() >= constMaxBuffer) {
         // Request too large, reject
@@ -412,7 +415,7 @@ void HttpSocket::readClient()
             socket->close();
 
             if (QTcpSocket::UnconnectedState==socket->state()) {
-                delete socket;
+                socket->deleteLater();
             }
         } else {
             // Bad Request
@@ -452,7 +455,7 @@ void HttpSocket::sendErrorResponse(QTcpSocket *socket, int code)
 void HttpSocket::cantataStreams(const QStringList &files)
 {
     DBUG << files;
-    foreach (const QString &f, files) {
+    for (const QString &f: files) {
         Song s=HttpServer::self()->decodeUrl(f);
         if (s.isCantataStream() || s.isCdda()) {
             DBUG << s.file;
@@ -468,16 +471,21 @@ void HttpSocket::cantataStreams(const QList<Song> &songs, bool isUpdate)
         streamIds.clear();
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         DBUG << s.file;
-        streamIds.insert(s.id, s.file);
+        if (s.isCantataStream()) {
+            streamIds.insert(s.id, HttpServer::self()->decodeUrl(s.file).file);
+        } else {
+            streamIds.insert(s.id, s.file);
+        }
         newlyAddedFiles.remove(s.file);
     }
+    DBUG << streamIds;
 }
 
 void HttpSocket::removedIds(const QSet<qint32> &ids)
 {
-    foreach (qint32 id, ids) {
+    for (qint32 id: ids) {
         streamIds.remove(id);
     }
 }
@@ -507,3 +515,5 @@ bool HttpSocket::write(QTcpSocket *socket, char *buffer, qint32 bytesRead, bool
     }
     return true;
 }
+
+#include "moc_httpsocket.cpp"
