@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2017-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -42,7 +42,7 @@ QStringList BrowseModel::FolderItem::allEntries(bool allowPlaylists) const
     if (children.isEmpty()) {
         entries << MPDConnection::constDirPrefix+path;
     } else {
-        foreach (Item *i, children) {
+        for (Item *i: children) {
             if (i->isFolder()) {
                 entries+=static_cast<FolderItem *>(i)->allEntries(allowPlaylists);
             } else if (allowPlaylists || Song::Playlist!=static_cast<TrackItem *>(i)->getSong().type) {
@@ -55,14 +55,30 @@ QStringList BrowseModel::FolderItem::allEntries(bool allowPlaylists) const
 
 BrowseModel::BrowseModel(QObject *p)
     : ActionModel(p)
-    , root(new FolderItem("/", 0))
+    , root(new FolderItem("/", nullptr))
     , enabled(false)
     , dbVersion(0)
 {
+    icn.addFile(":mpd.svg");
     connect(this, SIGNAL(listFolder(QString)), MPDConnection::self(), SLOT(listFolder(QString)));
     folderIndex.insert(root->getPath(), root);
 }
 
+QString BrowseModel::name() const
+{
+    return QLatin1String("mpdbrowse");
+}
+
+QString BrowseModel::title() const
+{
+    return tr("Server Folders");
+}
+
+QString BrowseModel::descr() const
+{
+    return tr("MPD virtual file-system");
+}
+
 void BrowseModel::clear()
 {
     beginResetModel();
@@ -114,7 +130,7 @@ QModelIndex BrowseModel::index(int row, int column, const QModelIndex &parent) c
         return QModelIndex();
     }
     const FolderItem * p = parent.isValid() ? static_cast<FolderItem *>(parent.internalPointer()) : root;
-    const Item * c = row<p->getChildCount() ? p->getChildren().at(row) : 0;
+    const Item * c = row<p->getChildCount() ? p->getChildren().at(row) : nullptr;
     return c ? createIndex(row, column, (void *)c) : QModelIndex();
 }
 
@@ -126,7 +142,7 @@ QModelIndex BrowseModel::parent(const QModelIndex &child) const
 
     const Item * const item = static_cast<Item *>(child.internalPointer());
     Item * const parentItem = item->getParent();
-    if (parentItem == root || 0==parentItem) {
+    if (parentItem == root || nullptr==parentItem) {
         return QModelIndex();
     }
 
@@ -181,6 +197,14 @@ void BrowseModel::fetchMore(const QModelIndex &index)
 QVariant BrowseModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
+        switch (role) {
+        case Cantata::Role_TitleText:
+            return title();
+        case Cantata::Role_SubText:
+            return descr();
+        case Qt::DecorationRole:
+            return icon();
+        }
         return QVariant();
     }
 
@@ -235,7 +259,7 @@ QMimeData * BrowseModel::mimeData(const QModelIndexList &indexes) const
 {
     QMimeData *mimeData = new QMimeData();
     QStringList files;
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         Item *item=toItem(idx);
         if (item) {
             if (item->isFolder()) {
@@ -253,7 +277,7 @@ QMimeData * BrowseModel::mimeData(const QModelIndexList &indexes) const
 QList<Song> BrowseModel::songs(const QModelIndexList &indexes, bool allowPlaylists) const
 {
     QList<Song> songList;
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         Item *item=toItem(idx);
         if (item && !item->isFolder() && (allowPlaylists || Song::Standard==static_cast<TrackItem *>(item)->getSong().type)) {
             songList.append(static_cast<TrackItem *>(item)->getSong());
@@ -293,15 +317,17 @@ void BrowseModel::folderContents(const QString &path, const QStringList &folders
 		QModelIndex idx=it.value()==root ? QModelIndex() : createIndex(it.value()->getRow(), 0, it.value());
 
 		beginInsertRows(idx, 0, folders.count() + songs.count() - 1);
-		foreach(const QString &folder, folders) {
+        for (const QString &folder: folders) {
 			FolderItem *item = new FolderItem(folder.split("/", QString::SkipEmptyParts).last(), folder, it.value());
 			it.value()->add(item);
 			folderIndex.insert(folder, item);
 		}
-		foreach(const Song &song, songs) {
+        for (const Song &song: songs) {
 			it.value()->add(new TrackItem(song, it.value()));
 		}
 		it.value()->setState(FolderItem::State_Fetched);
 		endInsertRows();
 	}
 }
+
+#include "moc_browsemodel.cpp"
