@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -46,7 +46,7 @@ public:
     virtual ~MusicLibraryItem() { }
 
     MusicLibraryItemContainer * parentItem() const { return m_parentItem; }
-    virtual MusicLibraryItem * childItem(int) const { return 0; }
+    virtual MusicLibraryItem * childItem(int) const { return nullptr; }
     virtual int childCount() const { return 0; }
     int row() const;
     void setRow(int r) const { m_row=r+1; }
@@ -70,15 +70,15 @@ class MusicLibraryItemContainer : public MusicLibraryItem
 {
 public:
     MusicLibraryItemContainer(const QString &data, MusicLibraryItemContainer *parent) : MusicLibraryItem(parent), m_itemData(data), m_isNew(false), m_rowsSet(false) { }
-    virtual ~MusicLibraryItemContainer() { clear(); }
+    ~MusicLibraryItemContainer() override { clear(); }
 
     virtual void append(MusicLibraryItem *i) { m_childItems.append(i); }
-    virtual MusicLibraryItem * childItem(int row) const { return m_childItems.value(row); }
+    MusicLibraryItem * childItem(int row) const override { return m_childItems.value(row); }
     MusicLibraryItem * childItem(const QString &name) const;
 
-    QString data() const { return m_itemData; }
+    QString data() const override { return m_itemData; }
     void setData(const QString &d) { m_itemData=d; }
-    int childCount() const { return m_childItems.count(); }
+    int childCount() const override { return m_childItems.count(); }
     const QList<MusicLibraryItem *> & childItems() const { return m_childItems; }
 
     void resetRows();
