@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -56,9 +56,12 @@ QString PlaylistsModel::headerText(int col)
     case COL_ALBUM:     return PlayQueueModel::headerText(PlayQueueModel::COL_ALBUM);
     case COL_LENGTH:    return PlayQueueModel::headerText(PlayQueueModel::COL_LENGTH);
     case COL_YEAR:      return PlayQueueModel::headerText(PlayQueueModel::COL_YEAR);
+    case COL_ORIGYEAR:  return PlayQueueModel::headerText(PlayQueueModel::COL_ORIGYEAR);
     case COL_GENRE:     return PlayQueueModel::headerText(PlayQueueModel::COL_GENRE);
     case COL_COMPOSER:  return PlayQueueModel::headerText(PlayQueueModel::COL_COMPOSER);
     case COL_PERFORMER: return PlayQueueModel::headerText(PlayQueueModel::COL_PERFORMER);
+    case COL_FILENAME:  return PlayQueueModel::headerText(PlayQueueModel::COL_FILENAME);
+    case COL_PATH:      return PlayQueueModel::headerText(PlayQueueModel::COL_PATH);
     default:            return QString();
     }
 }
@@ -68,7 +71,7 @@ GLOBAL_STATIC(PlaylistsModel, instance)
 PlaylistsModel::PlaylistsModel(QObject *parent)
     : ActionModel(parent)
     , multiCol(false)
-    , itemMenu(0)
+    , itemMenu(nullptr)
     , dropAdjust(0)
 {
     icn.addFile(":playlist.svg");
@@ -90,15 +93,16 @@ PlaylistsModel::PlaylistsModel(QObject *parent)
     newAction=new QAction(MonoIcon::icon(FontAwesome::asterisk, Utils::monoIconColor()), tr("New Playlist..."), this);
     connect(newAction, SIGNAL(triggered()), this, SIGNAL(addToNew()));
     Action::initIcon(newAction);
-    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=alignments[COL_PERFORMER]=int(Qt::AlignVCenter|Qt::AlignLeft);
-    alignments[COL_LENGTH]=alignments[COL_YEAR]=int(Qt::AlignVCenter|Qt::AlignRight);
+    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=
+            alignments[COL_PERFORMER]=alignments[COL_FILENAME]=alignments[COL_PATH]=int(Qt::AlignVCenter|Qt::AlignLeft);
+    alignments[COL_LENGTH]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=int(Qt::AlignVCenter|Qt::AlignRight);
 }
 
 PlaylistsModel::~PlaylistsModel()
 {
     if (itemMenu) {
         itemMenu->deleteLater();
-        itemMenu=0;
+        itemMenu=nullptr;
     }
 }
 
@@ -210,18 +214,23 @@ QVariant PlaylistsModel::headerData(int section, Qt::Orientation orientation, in
         case Qt::TextAlignmentRole:
             return alignments[section];
         case Cantata::Role_InitiallyHidden:
+            return COL_YEAR==section || COL_ORIGYEAR==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section ||
+                   COL_FILENAME==section || COL_PATH==section;
         case Cantata::Role_Hideable:
-            return COL_YEAR==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section;
+            return COL_LENGTH!=section;
         case Cantata::Role_Width:
             switch (section) {
             case COL_TITLE:     return 0.4;
             case COL_ARTIST:    return 0.15;
             case COL_ALBUM:     return 0.15;
             case COL_YEAR:      return 0.05;
+            case COL_ORIGYEAR:  return 0.05;
             case COL_GENRE:     return 0.125;
             case COL_LENGTH:    return 0.125;
             case COL_COMPOSER:  return 0.15;
             case COL_PERFORMER: return 0.15;
+            case COL_FILENAME:  return 0.15;
+            case COL_PATH:      return 0.15;
             }
         default:
             break;
@@ -316,6 +325,7 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                     }
                     return pl->loaded && !pl->isSmartPlaylist ? Utils::formatTime(pl->totalTime()) : QVariant();
                 case COL_YEAR:
+                case COL_ORIGYEAR:
                 case COL_GENRE:
                     return QVariant();
                 default:
@@ -434,12 +444,23 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                         return QVariant();
                     }
                     return s->year;
+                case COL_ORIGYEAR:
+                    if (s->origYear <= 0) {
+                        return QVariant();
+                    }
+                    return s->origYear;
                 case COL_GENRE:
                     return s->displayGenre();
                 case COL_COMPOSER:
                     return s->composer();
                 case COL_PERFORMER:
                     return s->performer();
+                case COL_FILENAME:
+                    return Utils::getFile(QUrl(s->file).path());
+                case COL_PATH: {
+                    QString dir=Utils::getDir(QUrl(s->file).path());
+                    return dir.endsWith("/") ? dir.left(dir.length()-1) : dir;
+                }
                 default:
                     break;
                 }
@@ -501,12 +522,12 @@ QStringList PlaylistsModel::filenames(const QModelIndexList &indexes, bool files
 {
     QStringList fnames;
     QSet<Item *> selectedPlaylists;
-    foreach(QModelIndex index, indexes) {
+    for(QModelIndex index: indexes) {
         Item *item=static_cast<Item *>(index.internalPointer());
 
         if (item->isPlaylist()) {
             selectedPlaylists.insert(item);
-            foreach (const SongItem *s, static_cast<PlaylistItem*>(item)->songs) {
+            for (const SongItem *s: static_cast<PlaylistItem*>(item)->songs) {
                 if (!filesOnly || !s->file.contains(':')) {
                     fnames << s->file;
                 }
@@ -526,17 +547,17 @@ QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
     QList<Song> songs;
     QSet<Item *> selectedPlaylists;
 
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         Item *item=static_cast<Item *>(idx.internalPointer());
         if (item->isPlaylist()) {
             PlaylistItem *playlist=static_cast<PlaylistItem *>(item);
-            foreach (const SongItem *song, playlist->songs) {
+            for (const SongItem *song: playlist->songs) {
                 selectedPlaylists.insert(item);
                 songs.append(*song);
             }
         }
     }
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(idx.internalPointer());
         if (!item->isPlaylist()) {
             SongItem *song=static_cast<SongItem *>(item);
@@ -559,7 +580,7 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
     QStringList playlists;
     QList<quint32> positions;
     QSet<Item *> selectedPlaylists;
-    foreach(QModelIndex index, indexes) {
+    for (QModelIndex index: indexes) {
         if (!index.isValid() || 0!=index.column()) {
             continue;
         }
@@ -575,7 +596,7 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
                 playlists << playlist->name;
                 positions << pos++;
             } else {
-                foreach (const SongItem *s, playlist->songs) {
+                for (const SongItem *s: playlist->songs) {
                     filenames << s->file;
                     playlists << playlist->name;
                     positions << pos++;
@@ -645,7 +666,7 @@ bool PlaylistsModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 
             bool fromThis=false;
             bool fromOthers=false;
-            foreach (const QString &p, playlists) {
+            for (const QString &p: playlists) {
                 if (p==pl->name) {
                     fromThis=true;
                 } else {
@@ -706,7 +727,7 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
             return;
         }
         beginResetModel();
-        foreach (const Playlist &p, playlists) {
+        for (const Playlist &p: playlists) {
             items.append(new PlaylistItem(p, allocateKey()));
         }
         endResetModel();
@@ -720,11 +741,11 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         QSet<QString> removed;
         QSet<QString> added;
 
-        foreach (PlaylistItem *p, items) {
+        for (PlaylistItem *p: items) {
             existing.insert(p->name);
         }
 
-        foreach (const Playlist &p, playlists) {
+        for (const Playlist &p: playlists) {
             retreived.insert(p.name);
             PlaylistItem *pl=getPlaylist(p.name);
 
@@ -740,7 +761,7 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         added=retreived-existing;
 
         if (removed.count()) {
-            foreach (const QString &p, removed) {
+            for (const QString &p: removed) {
                 PlaylistItem *pl=getPlaylist(p);
                 if (pl) {
                     int index=items.indexOf(pl);
@@ -754,7 +775,7 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         }
         if (added.count()) {
             beginInsertRows(parent, items.count(), items.count()+added.count()-1);
-            foreach (const QString &p, added) {
+            for (const QString &p: added) {
                 int idx=playlists.indexOf(Playlist(p));
                 if (-1!=idx) {
                     items.append(new PlaylistItem(playlists.at(idx), allocateKey()));
@@ -788,7 +809,7 @@ void PlaylistsModel::playlistInfoRetrieved(const QString &name, const QList<Song
 
         if (!songs.isEmpty()) {
             beginInsertRows(idx, 0, songs.count()-1);
-            foreach (const Song &s, songs) {
+            for (const Song &s: songs) {
                 pl->songs.append(new SongItem(s, pl));
             }
             endInsertRows();
@@ -803,7 +824,7 @@ void PlaylistsModel::playlistInfoRetrieved(const QString &name, const QList<Song
 
 void PlaylistsModel::removedFromPlaylist(const QString &name, const QList<quint32> &positions)
 {
-    PlaylistItem *pl=0;
+    PlaylistItem *pl=nullptr;
     if (0==positions.count() || !(pl=getPlaylist(name))) {
         emit listPlaylists();
         return;
@@ -839,13 +860,13 @@ void PlaylistsModel::removedFromPlaylist(const QString &name, const QList<quint3
 
 void PlaylistsModel::movedInPlaylist(const QString &name, const QList<quint32> &idx, quint32 pos)
 {
-    PlaylistItem *pl=0;
+    PlaylistItem *pl=nullptr;
     if (!(pl=getPlaylist(name)) || idx.count()>pl->songs.count()) {
         emit listPlaylists();
         return;
     }
 
-    foreach (quint32 i, idx) {
+    for (quint32 i: idx) {
         if (i>=(quint32)pl->songs.count()) {
             emit listPlaylists();
             return;
@@ -899,10 +920,10 @@ void PlaylistsModel::coverLoaded(const Song &song, int s)
     Q_UNUSED(s)
     if (!song.isArtistImageRequest() && !song.isComposerImageRequest()) {
         int plRow=0;
-        foreach (const PlaylistItem *pl, items) {
+        for (const PlaylistItem *pl: items) {
             QModelIndex plIdx;
             int songRow=0;
-            foreach (const SongItem *s, pl->songs) {
+            for (const SongItem *s: pl->songs) {
                 if (s->albumArtist()==song.albumArtist() && s->album==song.album) {
                     if (!plIdx.isValid()) {
                         plIdx=index(plRow, 0, QModelIndex());
@@ -923,37 +944,37 @@ void PlaylistsModel::updateItemMenu(bool create)
         if (!create) {
             return;
         }
-        itemMenu = new MirrorMenu(0);
+        itemMenu = new MirrorMenu(nullptr);
     }
 
     itemMenu->clear();
     itemMenu->addAction(newAction);
     QStringList names;
-    foreach (const PlaylistItem *p, items) {
+    for (const PlaylistItem *p: items) {
         if (!p->isSmartPlaylist) {
             names << p->name;
         }
     }
     qSort(names.begin(), names.end(), PlaylistsProxyModel::compareNames);
-    foreach (const QString &n, names) {
+    for (const QString &n: names) {
         itemMenu->addAction(n, this, SLOT(emitAddToExisting()));
     }
 }
 
 PlaylistsModel::PlaylistItem * PlaylistsModel::getPlaylist(const QString &name)
 {
-    foreach (PlaylistItem *p, items) {
+    for (PlaylistItem *p: items) {
         if (p->name==name) {
             return p;
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 void PlaylistsModel::clearPlaylists()
 {
-    foreach (PlaylistItem *p, items) {
+    for (PlaylistItem *p: items) {
         usedKeys.remove(p->key);
         emit playlistRemoved(p->key);
     }
@@ -1000,7 +1021,7 @@ void PlaylistsModel::PlaylistItem::clearSongs()
 PlaylistsModel::SongItem * PlaylistsModel::PlaylistItem::getSong(const Song &song, int offset)
 {
     int i=0;
-    foreach (SongItem *s, songs) {
+    for (SongItem *s: songs) {
         if (++i<offset) {
             continue;
         }
@@ -1009,15 +1030,17 @@ PlaylistsModel::SongItem * PlaylistsModel::PlaylistItem::getSong(const Song &son
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 quint32 PlaylistsModel::PlaylistItem::totalTime()
 {
     if (0==time) {
-        foreach (SongItem *s, songs) {
+        for (SongItem *s: songs) {
             time+=s->time;
         }
     }
     return time;
 }
+
+#include "moc_playlistsmodel.cpp"
