@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -33,10 +33,10 @@ class PlaylistsProxyModel : public ProxyModel
     Q_OBJECT
 
 public:
-    PlaylistsProxyModel(QObject *parent = 0);
-    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
+    PlaylistsProxyModel(QObject *parent = nullptr);
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
     static bool compareNames(const QString &l, const QString &r) { return l.localeAwareCompare(r)<0; }
-    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
+    bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
 };
 
 #endif
