@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -25,9 +25,9 @@
  */
 
 #include "playqueuemodel.h"
-#include "mpd-interface/mpdconnection.h"
 #include "mpd-interface/mpdparseutils.h"
 #include "mpd-interface/mpdstats.h"
+#include "mpd-interface/cuefile.h"
 #include "streams/streamfetcher.h"
 #include "streamsmodel.h"
 #include "http/httpserver.h"
@@ -51,14 +51,18 @@
 #endif
 #include <QPalette>
 #include <QFont>
+#include <QFile>
+#include <QDir>
 #include <QModelIndex>
 #include <QMimeData>
 #include <QTextStream>
 #include <QSet>
 #include <QUrl>
+#include <QUrlQuery>
 #include <QTimer>
 #include <QApplication>
 #include <QMenu>
+#include <QXmlStreamReader>
 
 GLOBAL_STATIC(PlayQueueModel, instance)
 
@@ -77,24 +81,184 @@ static const QLatin1String constSortByPerformerKey("performer");
 static const QLatin1String constSortByTitleKey("title");
 static const QLatin1String constSortByNumberKey("track");
 
-static bool checkExtension(const QString &file)
-{
-    static QSet<QString> constExtensions=QSet<QString>()
-            << QLatin1String("mp3") << QLatin1String("ogg") << QLatin1String("flac") << QLatin1String("wma") << QLatin1String("m4a")
-            << QLatin1String("m4b") << QLatin1String("mp4") << QLatin1String("m4p") << QLatin1String("wav") << QLatin1String("wv")
-            << QLatin1String("wvp") << QLatin1String("aiff") << QLatin1String("aif") << QLatin1String("aifc") << QLatin1String("ape")
-            << QLatin1String("spx") << QLatin1String("tta") << QLatin1String("mpc") << QLatin1String("mpp") << QLatin1String("mp+")
-            << QLatin1String("dff") << QLatin1String("dsf");
+static QSet<QString> constM3uPlaylists = QSet<QString>() << QLatin1String("m3u") << QLatin1String("m3u8");
+static const QString constPlsPlaylist = QLatin1String("pls");
+static const QString constXspfPlaylist = QLatin1String("xspf");
+QSet<QString> PlayQueueModel::constFileExtensions = QSet<QString>()
+                                                  << QLatin1String("mp3") << QLatin1String("ogg") << QLatin1String("flac") << QLatin1String("wma") << QLatin1String("m4a")
+                                                  << QLatin1String("m4b") << QLatin1String("mp4") << QLatin1String("m4p") << QLatin1String("wav") << QLatin1String("wv")
+                                                  << QLatin1String("wvp") << QLatin1String("aiff") << QLatin1String("aif") << QLatin1String("aifc") << QLatin1String("ape")
+                                                  << QLatin1String("spx") << QLatin1String("tta") << QLatin1String("mpc") << QLatin1String("mpp") << QLatin1String("mp+")
+                                                  << QLatin1String("dff") << QLatin1String("dsf")
+                                                  // And playlists...
+                                                  << QLatin1String("m3u") << QLatin1String("m3u8") << constPlsPlaylist << constXspfPlaylist;
 
+static inline QString getExtension(const QString &file)
+{
     int pos=file.lastIndexOf('.');
-    return pos>1 ? constExtensions.contains(file.mid(pos+1).toLower()) : false;
+    return pos>0 ? file.mid(pos+1).toLower() : QString();
 }
 
-static QStringList parseUrls(const QStringList &urls, bool percentEncoded)
+static inline bool checkExtension(const QString &file, const QSet<QString> &exts = PlayQueueModel::constFileExtensions)
+{
+    return exts.contains(getExtension(file));
+}
+
+static QString fileUrl(const QString &file, bool useServer, bool useLocal)
+{
+    if (useServer) {
+        QByteArray path = HttpServer::self()->encodeUrl(file);
+        if (!path.isEmpty()) {
+            return path;
+        }
+    } else if (useLocal && !file.startsWith(QLatin1String("/media/"))) {
+        return QLatin1String("file://")+file;
+    }
+    return QString();
+}
+
+static QString checkUrl(const QString &url, const QDir &dir, bool useServer, bool useLocal, const QSet<QString> &handlers)
+{
+    int pos = url.indexOf(QLatin1String("://"));
+    QString handler = pos>0 ? url.left(pos+3).toLower() : QString();
+    if (!handler.isEmpty() && (QLatin1String("http://")==handler || QLatin1String("https://")==handler)) {
+        // Radio stream?
+        return StreamsModel::constPrefix+url;
+    } else if (handlers.contains(handler)) {
+        return url;
+    } else if (checkExtension(url)) {
+        QString path = dir.filePath(url);
+        if (QFile::exists(path)) { // Relative
+            return fileUrl(path, useServer, useLocal);
+        } else if (QFile::exists(url)) { // Absolute
+            return fileUrl(url, useServer, useLocal);
+        }
+    }
+    return QString();
+}
+
+static QStringList expandM3uPlaylist(const QString &playlist, bool useServer, bool useLocal, const QSet<QString> &handlers)
+{
+    QStringList files;
+    QFile f(playlist);
+    QDir dir(Utils::getDir(playlist));
+
+    if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
+        QTextStream in(&f);
+        while (!in.atEnd()) {
+            QString line = in.readLine();
+            if (!line.startsWith(QLatin1Char('#'))) {
+                QString url = checkUrl(line, dir, useServer, useLocal, handlers);
+                if (!url.isEmpty()) {
+                    files.append(url);
+                }
+            }
+        }
+        f.close();
+    }
+    return files;
+}
+
+static QStringList expandPlsPlaylist(const QString &playlist, bool useServer, bool useLocal, const QSet<QString> &handlers)
+{
+    QStringList files;
+    QFile f(playlist);
+    QDir dir(Utils::getDir(playlist));
+    QMap<unsigned int, QString> titles;
+    QMap<unsigned int, QString> urls;
+
+    if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
+        QTextStream in(&f);
+        while (!in.atEnd()) {
+            QString line = in.readLine();
+            if (line.startsWith(QLatin1String("File"))) {
+                QStringList parts=line.split("=");
+                if (2==parts.length()) {
+                    QString path=parts[1].trimmed();
+                    unsigned int num=parts[0].left(4).toUInt();
+                    QString url=checkUrl(path, dir, useServer, useLocal, handlers);
+                    if (!url.isEmpty()) {
+                        urls.insert(num, url);
+                    }
+                }
+            } else if (line.startsWith(QLatin1String("Title"))) {
+                QStringList parts=line.split("=");
+                if (2==parts.length()) {
+                    titles.insert(parts[0].left(5).toUInt(), parts[1].trimmed());
+                }
+            }
+        }
+        f.close();
+    }
+
+    auto it = urls.constBegin();
+    auto end = urls.constEnd();
+    for (; it!=end; ++it) {
+        if (titles.contains(it.key()) && (it.value().startsWith(QLatin1String("http://")) || it.value().startsWith("https://"))) {
+            files.append(it.value()+"#"+titles[it.key()]);
+        } else {
+            files.append(it.value());
+        }
+    }
+    return files;
+}
+
+static QStringList expandXspfPlaylist(const QString &playlist, bool useServer, bool useLocal, const QSet<QString> &handlers)
+{
+    QStringList files;
+    QFile f(playlist);
+
+    if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
+        QXmlStreamReader reader(&f);
+        QDir dir(Utils::getDir(playlist));
+
+        while (!reader.atEnd()) {
+            reader.readNext();
+            if (QXmlStreamReader::StartElement==reader.tokenType() && QLatin1String("location")==reader.name()) {
+                QString url=checkUrl(reader.readElementText().trimmed(), dir, useServer, useLocal, handlers);
+                if (!url.isEmpty()) {
+                    files.append(url);
+                }
+            }
+        }
+        f.close();
+    }
+
+    return files;
+}
+
+static QStringList listFiles(const QDir &d, bool useServer, bool useLocal, const QSet<QString> &handlers, int level=5)
+{
+    QStringList files;
+    if (level) {
+        for (const auto &f: d.entryInfoList(QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot|QDir::NoSymLinks, QDir::LocaleAware|QDir::IgnoreCase)) {
+            if (f.isDir()) {
+                files += listFiles(QDir(f.absoluteFilePath()), useServer, useLocal, handlers, level-1);
+            } else if (checkExtension(f.fileName(), constM3uPlaylists)) {
+                files += expandM3uPlaylist(f.absoluteFilePath(), useServer, useLocal, handlers);
+            } else if (constPlsPlaylist == getExtension(f.fileName())) {
+                files += expandPlsPlaylist(f.absoluteFilePath(), useServer, useLocal, handlers);
+            } else if (constXspfPlaylist == getExtension(f.fileName())) {
+                files += expandXspfPlaylist(f.absoluteFilePath(), useServer, useLocal, handlers);
+            } else if (checkExtension(f.fileName())) {
+                QString fUrl=fileUrl(f.absoluteFilePath(), useServer, useLocal);
+                if (!fUrl.isEmpty()) {
+                    files.append(fUrl);
+                }
+            }
+        }
+    }
+    return files;
+}
+
+static QStringList parseUrls(const QStringList &urls)
 {
     QStringList useable;
-    foreach (const QString &path, urls) {
-        QUrl u=percentEncoded ? QUrl::fromPercentEncoding(path.toUtf8()) : QUrl(path);
+    bool useServer = HttpServer::self()->isAlive();
+    bool useLocal = MPDConnection::self()->localFilePlaybackSupported();
+    QSet<QString> handlers = MPDConnection::self()->urlHandlers();
+    for (const auto &path: urls) {
+        QUrl u=path.indexOf("://")>2 ? QUrl(path) : QUrl::fromLocalFile(path);
         #if defined ENABLE_DEVICES_SUPPORT && (defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND)
         QString cdDevice=AudioCdDevice::getDevice(u);
         if (!cdDevice.isEmpty()) {
@@ -103,15 +267,37 @@ static QStringList parseUrls(const QStringList &urls, bool percentEncoded)
         #endif
         if (QLatin1String("http")==u.scheme()) {
             useable.append(u.toString());
-        } else if ((u.scheme().isEmpty() || QLatin1String("file")==u.scheme()) && checkExtension(u.path())) {
-            if (MPDConnection::self()->localFilePlaybackSupported() && !u.path().startsWith(QLatin1String("/media/"))) {
-                useable.append(QLatin1String("file://")+u.path());
-            } else if (HttpServer::self()->isAlive()) {
-                useable.append(HttpServer::self()->encodeUrl(u.path()));
+        } else if (u.scheme().isEmpty() || QLatin1String("file")==u.scheme()) {
+            QDir d(u.path());
+
+            if (d.exists()) {
+                useable = listFiles(d, useServer, useLocal, handlers);
+            } else if (checkExtension(u.path(), constM3uPlaylists)) {
+                useable += expandM3uPlaylist(u.path(), useServer, useLocal, handlers);
+            } else if (constPlsPlaylist == getExtension(u.path())) {
+                useable += expandPlsPlaylist(u.path(), useServer, useLocal, handlers);
+            } else if (constXspfPlaylist == getExtension(u.path())) {
+                useable += expandXspfPlaylist(u.path(), useServer, useLocal, handlers);
+            } else if (checkExtension(u.path())) {
+                QString fUrl = fileUrl(u.path(), useServer, useLocal);
+                if (!fUrl.isEmpty()) {
+                    useable.append(fUrl);
+                }
             }
         }
     }
-    return useable;
+
+    // Ensure we only have unqiue URLs...
+    QSet<QString> unique;
+    QStringList use;
+    for (const auto &u: useable) {
+        if (!unique.contains(u)) {
+            unique.insert(u);
+            use.append(u);
+        }
+    }
+
+    return use;
 }
 
 void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QStringList &values)
@@ -119,7 +305,7 @@ void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QStr
     QByteArray encodedData;
     QTextStream stream(&encodedData, QIODevice::WriteOnly);
 
-    foreach (const QString &v, values) {
+    for (const QString &v: values) {
         stream << v << endl;
     }
 
@@ -130,7 +316,7 @@ void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QLis
 {
     QByteArray encodedData;
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
-    foreach (quint32 v, values) {
+    for (quint32 v: values) {
         stream << v;
     }
     mimeData.setData(mime, encodedData);
@@ -178,6 +364,8 @@ QString PlayQueueModel::headerText(int col)
     case COL_COMPOSER:  return tr("Composer");
     case COL_PERFORMER: return tr("Performer");
     case COL_RATING:    return tr("Rating");
+    case COL_FILENAME:  return tr("Filename");
+    case COL_PATH:      return tr("Path");
     default:            return QString();
     }
 }
@@ -234,7 +422,7 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     connect(removeDuplicatesAction, SIGNAL(triggered()), this, SLOT(removeDuplicates()));
 
     shuffleAction=new Action(tr("Shuffle"), this);
-    shuffleAction->setMenu(new QMenu(0));
+    shuffleAction->setMenu(new QMenu(nullptr));
     Action *shuffleTracksAction = new Action(tr("Tracks"), shuffleAction);
     Action *shuffleAlbumsAction = new Action(tr("Albums"), shuffleAction);
     connect(shuffleTracksAction, SIGNAL(triggered()), MPDConnection::self(), SLOT(shuffle()));
@@ -243,7 +431,7 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     shuffleAction->menu()->addAction(shuffleAlbumsAction);
 
     sortAction=new Action(tr("Sort By"), this);
-    sortAction->setMenu(new QMenu(0));
+    sortAction->setMenu(new QMenu(nullptr));
     addSortAction(tr("Artist"), constSortByArtistKey);
     addSortAction(tr("Album Artist"), constSortByAlbumArtistKey);
     addSortAction(tr("Album"), constSortByAlbumKey);
@@ -256,8 +444,10 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     controlActions();
     shuffleAction->setEnabled(false);
     sortAction->setEnabled(false);
-    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=alignments[COL_PERFORMER]=int(Qt::AlignVCenter|Qt::AlignLeft);
-    alignments[COL_TRACK]=alignments[COL_LENGTH]=alignments[COL_DISC]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=alignments[COL_PRIO]=int(Qt::AlignVCenter|Qt::AlignRight);
+    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=
+            alignments[COL_PERFORMER]=alignments[COL_FILENAME]=alignments[COL_PATH]=int(Qt::AlignVCenter|Qt::AlignLeft);
+    alignments[COL_TRACK]=alignments[COL_LENGTH]=alignments[COL_DISC]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=
+            alignments[COL_PRIO]=int(Qt::AlignVCenter|Qt::AlignRight);
     alignments[COL_RATING]=int(Qt::AlignVCenter|Qt::AlignHCenter);
 }
 
@@ -286,9 +476,9 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             return alignments[section];
         case Cantata::Role_InitiallyHidden:
             return COL_YEAR==section || COL_ORIGYEAR==section || COL_DISC==section || COL_GENRE==section || COL_PRIO==section ||
-                   COL_COMPOSER==section || COL_PERFORMER==section || COL_RATING==section;
+                   COL_COMPOSER==section || COL_PERFORMER==section || COL_RATING==section || COL_FILENAME==section || COL_PATH==section;
         case Cantata::Role_Hideable:
-            return COL_TITLE!=section && COL_ARTIST!=section;
+            return COL_LENGTH!=section;
         case Cantata::Role_Width:
             switch (section) {
             case COL_TRACK:     return 0.075;
@@ -304,6 +494,8 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             case COL_COMPOSER:  return 0.2;
             case COL_PERFORMER: return 0.2;
             case COL_RATING:    return 0.08;
+            case COL_FILENAME:  return 0.27;
+            case COL_PATH:      return 0.27;
             }
         case Cantata::Role_ContextMenuText:
             return COL_TRACK==section ? tr("# (Track Number)") : headerText(section);
@@ -546,6 +738,12 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
 //            var.setValue(s.rating);
 //            return var;
 //        }
+        case COL_FILENAME:
+            return Utils::getFile(QUrl(song.file).path());
+        case COL_PATH: {
+            QString dir=Utils::getDir(QUrl(song.file).path());
+            return dir.endsWith("/") ? dir.left(dir.length()-1) : dir;
+        }
         default:
             break;
         }
@@ -628,7 +826,7 @@ QMimeData *PlayQueueModel::mimeData(const QModelIndexList &indexes) const
     QList<quint32> positions;
     QStringList filenames;
 
-    foreach(QModelIndex index, indexes) {
+    for (const QModelIndex &index: indexes) {
         if (index.isValid() && 0==index.column()) {
             positions.append(index.row());
             filenames.append(static_cast<Song *>(index.internalPointer())->file);
@@ -671,7 +869,7 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
         addItems(decode(*data, constFileNameMimeType), row, false, 0, false);
         return true;
     } else if(data->hasFormat(constUriMimeType)/* && MPDConnection::self()->getDetails().isLocal()*/) {
-        QStringList useable=parseUrls(decode(*data, constUriMimeType), true);
+        QStringList useable=parseUrls(decode(*data, constUriMimeType));
         if (useable.count()) {
             addItems(useable, row, false, 0, false);
             return true;
@@ -680,11 +878,11 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
     return false;
 }
 
-void PlayQueueModel::load(const QStringList &urls)
+void PlayQueueModel::load(const QStringList &urls, int action, quint8 priority, bool decreasePriority)
 {
-    QStringList useable=parseUrls(urls, false);
+    QStringList useable=parseUrls(urls);
     if (useable.count()) {
-        addItems(useable, songs.count(), songs.isEmpty(), 0, false);
+        addItems(useable, songs.count(), action, priority, decreasePriority);
     }
 }
 
@@ -692,9 +890,8 @@ void PlayQueueModel::addItems(const QStringList &items, int row, int action, qui
 {
     bool haveRadioStream=false;
 
-    foreach (const QString &f, items) {
+    for (const QString &f: items) {
         QUrl u(f);
-
         if (u.scheme().startsWith(StreamsModel::constPrefix)) {
             haveRadioStream=true;
             break;
@@ -726,14 +923,14 @@ void PlayQueueModel::prioritySet(const QMap<qint32, quint8> &prio)
 {
     QList<Song> prev;
     if (undoEnabled) {
-        foreach (Song s, songs) {
+        for (const Song &s: songs) {
             prev.append(s);
         }
     }
     QMap<qint32, quint8> copy = prio;
     int row=0;
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         QMap<qint32, quint8>::ConstIterator it = copy.find(s.id);
         if (copy.end()!=it) {
             s.priority=it.value();
@@ -757,9 +954,31 @@ qint32 PlayQueueModel::getIdByRow(qint32 row) const
 
 qint32 PlayQueueModel::getSongId(const QString &file) const
 {
-    foreach (const Song &s, songs) {
-        if (s.file==file) {
-            return s.id;
+    if (CueFile::isCue(file)) {
+        QUrl u(file);
+        QUrlQuery q(u);
+        Song check;
+        check.album=q.queryItemValue("album");
+        check.artist=q.queryItemValue("artist");
+        check.albumartist=q.queryItemValue("albumartist");
+        check.album=q.queryItemValue("album");
+        check.title=q.queryItemValue("title");
+        check.disc=q.queryItemValue("disc").toUInt();
+        check.track=q.queryItemValue("track").toUInt();
+        check.time=q.queryItemValue("time").toUInt();
+        check.year=q.queryItemValue("year").toUInt();
+        check.origYear=q.queryItemValue("origYear").toUInt();
+
+        for (const Song &s: songs) {
+            if (s.sameMetadata(check)) {
+                return s.id;
+            }
+        }
+    } else {
+        for (const Song &s: songs) {
+            if (s.file==file) {
+                return s.id;
+            }
         }
     }
 
@@ -789,7 +1008,7 @@ Song PlayQueueModel::getSongByRow(const qint32 row) const
 
 Song PlayQueueModel::getSongById(qint32 id) const
 {
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (s.id==id) {
             return s;
         }
@@ -873,12 +1092,13 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
     }
 
     QSet<qint32> newIds;
-    foreach (const Song &s, songList) {
+    for (const Song &s: songList) {
         newIds.insert(s.id);
     }
 
     // If we have too many changes UI can hang, so it is sometimes better just to do a complete reset!
-    if (isComplete && songs.count()>MPDConnection::constMaxPqChanges) {
+    if (isComplete && (songs.count()>MPDConnection::constMaxPqChanges
+                       || !MPDConnection::self()->isPlayQueueIdValid())) {
         songs.clear();
     }
 
@@ -894,7 +1114,7 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
         time = 0;
 
         QSet<qint32> removed=ids-newIds;
-        foreach (qint32 id, removed) {
+        for (qint32 id: removed) {
             qint32 row=getRowById(id);
             if (row!=-1) {
                 beginRemoveRows(QModelIndex(), row, row);
@@ -979,7 +1199,7 @@ void PlayQueueModel::setStopAfterTrack(qint32 track)
 bool PlayQueueModel::removeCantataStreams(bool cdOnly)
 {
     QList<qint32> ids;
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (s.isCdda() || (!cdOnly && s.isCantataStream())) {
             ids.append(s.id);
         }
@@ -1000,7 +1220,7 @@ void PlayQueueModel::removeAll()
 void PlayQueueModel::remove(const QList<int> &rowsToRemove)
 {
     QList<qint32> removeIds;
-    foreach (const int &r, rowsToRemove) {
+    for (const int &r: rowsToRemove) {
         if (r>-1 && r<songs.count()) {
             removeIds.append(songs.at(r).id);
         }
@@ -1014,12 +1234,12 @@ void PlayQueueModel::remove(const QList<int> &rowsToRemove)
 void PlayQueueModel::crop(const QList<int> &rowsToKeep)
 {
     QSet<qint32> allIds;
-    foreach(const Song &song, songs) {
+    for (const Song &song: songs) {
         allIds.insert(song.id);
     }
 
     QSet<qint32> keepIds;
-    foreach (const int &r, rowsToKeep) {
+    for (const int &r: rowsToKeep) {
         if (r>-1 && r<songs.count()) {
             keepIds.insert(songs.at(r).id);
         }
@@ -1034,7 +1254,7 @@ void PlayQueueModel::crop(const QList<int> &rowsToKeep)
 void PlayQueueModel::setRating(const QList<int> &rows, quint8 rating) const
 {
     QSet<QString> files;
-    foreach (const int &r, rows) {
+    for (const int &r: rows) {
         if (r>-1 && r<songs.count()) {
             const Song &s=songs.at(r);
             if (Song::Standard==s.type && !files.contains(s.file)) {
@@ -1061,7 +1281,7 @@ void PlayQueueModel::enableUndo(bool e)
 static PlayQueueModel::UndoItem getState(const QList<Song> &songs)
 {
     PlayQueueModel::UndoItem item;
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         item.files.append(s.file);
         item.priority.append(s.priority);
     }
@@ -1222,7 +1442,7 @@ void PlayQueueModel::sortBy()
     if (act) {
         QString key=act->property(constSortByKey).toString();
         QList<const Song *> copy;
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             ((Song &)s).populateSorts();
             copy.append(&s);
         }
@@ -1247,7 +1467,7 @@ void PlayQueueModel::sortBy()
             qSort(copy.begin(), copy.end(), trackSort);
         }
         QList<quint32> positions;
-        foreach (const Song *s, copy) {
+        for (const Song *s: copy) {
             positions.append(getRowById(s->id));
         }
         emit setOrder(positions);
@@ -1257,12 +1477,12 @@ void PlayQueueModel::sortBy()
 void PlayQueueModel::removeDuplicates()
 {
     QMap<QString, QList<Song> > map;
-    foreach (const Song &song, songs) {
+    for (const Song &song: songs) {
         map[song.albumKey()+"-"+song.artistSong()+"-"+song.track].append(song);
     }
 
     QList<qint32> toRemove;
-    foreach (const QString &key, map.keys()) {
+    for (const QString &key: map.keys()) {
         QList<Song> values=map.value(key);
         if (values.size()>1) {
             Song::sortViaType(values);
@@ -1297,7 +1517,7 @@ void PlayQueueModel::stickerDbChanged()
 {
     // Sticker DB changed, need to re-request ratings...
     QSet<QString> requests;
-    foreach (const Song &song, songs) {
+    for (const Song &song: songs) {
         if (Song::Standard==song.type && song.rating<=Song::Rating_Max && !requests.contains(song.file)) {
             emit getRating(song.file);
             requests.insert(song.file);
@@ -1339,7 +1559,7 @@ void PlayQueueModel::stats()
 {
     time = 0;
     //Loop over all songs
-    foreach(const Song &song, songs) {
+    for (const Song &song: songs) {
         time += song.time;
     }
 
@@ -1359,7 +1579,7 @@ static bool songSort(const Song *s1, const Song *s2)
 void PlayQueueModel::shuffleAlbums()
 {
     QMap<quint32, QList<const Song *> > albums;
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         albums[s.key].append(&s);
     }
 
@@ -1373,7 +1593,7 @@ void PlayQueueModel::shuffleAlbums()
         quint32 key=keys.takeAt(Utils::random(keys.count()));
         QList<const Song *> albumSongs=albums[key];
         qSort(albumSongs.begin(), albumSongs.end(), songSort);
-        foreach (const Song *song, albumSongs) {
+        for (const Song *song: albumSongs) {
             positions.append(getRowById(song->id));
         }
     }
@@ -1393,11 +1613,11 @@ void PlayQueueModel::remove(const QList<Song> &rem)
     QSet<QString> s;
     QList<qint32> ids;
 
-    foreach (const Song &song, rem) {
+    for (const Song &song: rem) {
         s.insert(song.file);
     }
 
-    foreach (const Song &song, songs) {
+    for (const Song &song: songs) {
         if (s.contains(song.file)) {
             ids.append(song.id);
             s.remove(song.file);
@@ -1419,7 +1639,7 @@ void PlayQueueModel::updateDetails(const QList<Song> &updated)
     bool currentUpdated=false;
     Song currentSong;
 
-    foreach (const Song &song, updated) {
+    for (const Song &song: updated) {
         songMap[song.file]=song;
     }
 
@@ -1451,7 +1671,7 @@ void PlayQueueModel::updateDetails(const QList<Song> &updated)
             beginResetModel();
             endResetModel();
         } else {
-            foreach (int row, updatedRows) {
+            for (int row: updatedRows) {
                 emit dataChanged(index(row, 0), index(row, columnCount(QModelIndex())-1));
             }
         }
@@ -1465,8 +1685,10 @@ void PlayQueueModel::updateDetails(const QList<Song> &updated)
 QStringList PlayQueueModel::filenames()
 {
     QStringList names;
-    foreach(const Song &song, songs) {
+    for (const Song &song: songs) {
         names << song.file;
     }
     return names;
 }
+
+#include "moc_playqueuemodel.cpp"
