@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -66,7 +66,7 @@ QModelIndex StreamSearchModel::index(int row, int column, const QModelIndex &par
     }
 
     const StreamsModel::CategoryItem * p = parent.isValid() ? static_cast<StreamsModel::CategoryItem *>(parent.internalPointer()) : root;
-    const StreamsModel::Item * c = row<p->children.count() ? p->children.at(row) : 0;
+    const StreamsModel::Item * c = row<p->children.count() ? p->children.at(row) : nullptr;
     return c ? createIndex(row, column, (void *)c) : QModelIndex();
 }
 
@@ -217,7 +217,7 @@ void StreamSearchModel::fetchMore(const QModelIndex &index)
 QStringList StreamSearchModel::filenames(const QModelIndexList &indexes, bool addPrefix) const
 {
     QStringList fnames;
-    foreach(QModelIndex index, indexes) {
+    for (const QModelIndex &index: indexes) {
         StreamsModel::Item *item=static_cast<StreamsModel::Item *>(index.internalPointer());
         if (!item->isCategory() && !fnames.contains(item->url)) {
             fnames << StreamsModel::modifyUrl(item->url, addPrefix, item->name);
@@ -244,7 +244,7 @@ QStringList StreamSearchModel::mimeTypes() const
 void StreamSearchModel::clear()
 {
     cancelAll();
-    foreach (StreamsModel::Item *item, root->children) {
+    for (StreamsModel::Item *item: root->children) {
         StreamsModel::CategoryItem *cat=static_cast<StreamsModel::CategoryItem *>(item);
 		if (cat->children.count()) {
 			QModelIndex index = createIndex(root->children.indexOf(cat), 0, (void *)cat);
@@ -287,7 +287,7 @@ void StreamSearchModel::search(const QString &searchTerm, bool stationsOnly)
     clear();
     currentSearch=searchTerm;
 
-    foreach (StreamsModel::Item *item, root->children) {
+    for (StreamsModel::Item *item: root->children) {
         QUrl searchUrl;
         QUrlQuery query;
         switch (root->children.indexOf(item)) {
@@ -311,7 +311,7 @@ void StreamSearchModel::search(const QString &searchTerm, bool stationsOnly)
             if (0==bitrate) {
                 bitrate=getParam("bitrate", search);
             }
-            query.addQueryItem("k", StreamsModel::constShoutCastApiKey);
+            ApiKeys::self()->addKey(query, ApiKeys::ShoutCast);
             query.addQueryItem("search", search);
             query.addQueryItem("limit", QString::number(limit<1 ? 100 : limit));
             if (bitrate>=32 && bitrate<=512) {
@@ -321,7 +321,7 @@ void StreamSearchModel::search(const QString &searchTerm, bool stationsOnly)
         }
         case Dirble:
             searchUrl=QUrl(item->url+searchTerm);
-            query.addQueryItem("token", StreamsModel::constDirbleApiKey);
+            ApiKeys::self()->addKey(query, ApiKeys::Dirble);
             break;
         }
 
@@ -339,7 +339,7 @@ void StreamSearchModel::cancelAll()
 {
     if (!jobs.isEmpty()) {
         QList<NetworkJob *> jobList=jobs.keys();
-        foreach (NetworkJob *j, jobList) {
+        for (NetworkJob *j: jobList) {
             j->cancelAndDelete();
         }
         jobs.clear();
@@ -361,13 +361,12 @@ void StreamSearchModel::jobFinished()
         cat->state=StreamsModel::CategoryItem::Fetched;
         jobs.remove(job);
         QModelIndex index=cat==root ? QModelIndex() : createIndex(cat->parent->children.indexOf(cat), 0, (void *)cat);
+        StreamsModel::Item *i=cat;
+        while (i->parent && i->parent!=root) {
+            i=i->parent;
+        }
         if (job->ok()) {
             QList<StreamsModel::Item *> newItems;
-
-            StreamsModel::Item *i=cat;
-            while (i->parent && i->parent!=root) {
-                i=i->parent;
-            }
             switch(root->children.indexOf(i)) {
             case TuneIn:    newItems=StreamsModel::parseRadioTimeResponse(job->actualJob(), cat, true); break;
             case ShoutCast: newItems=StreamsModel::parseShoutCastSearchResponse(job->actualJob(), cat); break;
@@ -379,6 +378,12 @@ void StreamSearchModel::jobFinished()
                 cat->children+=newItems;
                 endInsertRows();
             }
+        } else {
+            switch(root->children.indexOf(i)) {
+            case ShoutCast: ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::ShoutCast); break;
+            case Dirble:    ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::Dirble); break;
+            default : break;
+            }
         }
         emit dataChanged(index, index);
         if (jobs.isEmpty()) {
@@ -391,7 +396,7 @@ QList<StreamsModel::Item *> StreamSearchModel::getStreams(StreamsModel::Category
 {
     QList<StreamsModel::Item *> streams;
     if (cat) {
-        foreach (StreamsModel::Item *i, cat->children) {
+        for (StreamsModel::Item *i: cat->children) {
             if (i->isCategory()) {
                 streams+=getStreams(static_cast<StreamsModel::CategoryItem *>(i));
             } else {
@@ -401,3 +406,5 @@ QList<StreamsModel::Item *> StreamSearchModel::getStreams(StreamsModel::Category
     }
     return streams;
 }
+
+#include "moc_streamsearchmodel.cpp"
