@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -64,13 +64,8 @@ GLOBAL_STATIC(StreamsModel, instance)
 
 const QString StreamsModel::constSubDir=QLatin1String("streams");
 const QString StreamsModel::constCacheExt=QLatin1String(".xml.gz");
-
-const QString StreamsModel::constShoutCastApiKey=QLatin1String("fa1669MuiRPorUBw");
 const QString StreamsModel::constShoutCastHost=QLatin1String("api.shoutcast.com");
-
-const QString StreamsModel::constDirbleApiKey=QLatin1String("1035d2834bdc7195b8929ad7f70a8410f02c633e");
 const QString StreamsModel::constDirbleHost=QLatin1String("api.dirble.com");
-
 const QString StreamsModel::constCompressedXmlFile=QLatin1String("streams.xml.gz");
 const QString StreamsModel::constXmlFile=QLatin1String("streams.xml");
 const QString StreamsModel::constSettingsFile=QLatin1String("settings.json");
@@ -88,8 +83,8 @@ static const QString constDiChannelListHost=QLatin1String("api.v2.audioaddict.co
 static const QString constDiChannelListUrl=QLatin1String("http://")+constDiChannelListHost+("/v1/%1/mobile/batch_update?asset_group_key=mobile_icons&stream_set_key=");
 static const QString constDiStdUrl=QLatin1String("http://%1/public3/%2.pls");
 
-static QString constShoutCastUrl=QLatin1String("http://")+StreamsModel::constShoutCastHost+QLatin1String("/genre/primary?f=xml&k=")+StreamsModel::constShoutCastApiKey;
-static QString constDirbleUrl=QLatin1String("http://")+StreamsModel::constDirbleHost+QLatin1String("/v2/categories/primary?token=")+StreamsModel::constDirbleApiKey;
+static QString constShoutCastUrl=QLatin1String("http://")+StreamsModel::constShoutCastHost+QLatin1String("/genre/primary?f=xml");
+static QString constDirbleUrl=QLatin1String("http://")+StreamsModel::constDirbleHost+QLatin1String("/v2/categories/primary");
 
 static const QLatin1String constBookmarksDir=QLatin1String("bookmarks");
 
@@ -102,7 +97,7 @@ static QIcon getIcon(const QString &name)
 
 static Icon getExternalIcon(const QString &path, QStringList files=QStringList() << StreamsModel::constSvgIcon <<  StreamsModel::constPngIcon)
 {
-    foreach (const QString &file, files) {
+    for (const QString &file: files) {
         QString iconFile=path+Utils::constDirSep+file;
         if (QFile::exists(iconFile)) {
             Icon icon;
@@ -142,7 +137,7 @@ StreamsModel::CategoryItem * StreamsModel::Item::getTopLevelCategory() const
     while (item->parent && item->parent->parent) {
         item=item->parent;
     }
-    return item && item->isCategory() ? static_cast<CategoryItem *>(item) : 0;
+    return item && item->isCategory() ? static_cast<CategoryItem *>(item) : nullptr;
 }
 
 void StreamsModel::CategoryItem::removeBookmarks()
@@ -162,7 +157,7 @@ void StreamsModel::CategoryItem::saveBookmarks()
         return;
     }
 
-    foreach (Item *child, children) {
+    for (Item *child: children) {
         if (child->isCategory()) {
              CategoryItem *cat=static_cast<CategoryItem *>(child);
              if (cat->isBookmarks) {
@@ -176,10 +171,10 @@ void StreamsModel::CategoryItem::saveBookmarks()
                          doc.writeStartElement("bookmarks");
                          doc.writeAttribute("version", "1.0");
                          doc.setAutoFormatting(false);
-                         foreach (Item *i, cat->children) {
+                         for (Item *i: cat->children) {
                              doc.writeStartElement("bookmark");
                              doc.writeAttribute("name", i->name);
-                             doc.writeAttribute("url", i->url);
+                             doc.writeAttribute("url", i->fullUrl());
                              doc.writeEndElement();
                          }
                          doc.writeEndElement();
@@ -226,12 +221,12 @@ QList<StreamsModel::Item *> StreamsModel::CategoryItem::loadBookmarks()
 
 StreamsModel::CategoryItem * StreamsModel::CategoryItem::getBookmarksCategory()
 {
-    foreach (Item *i, children) {
+    for (Item *i: children) {
         if (i->isCategory() && static_cast<CategoryItem *>(i)->isBookmarks) {
             return static_cast<CategoryItem *>(i);
         }
     }
-    return 0;
+    return nullptr;
 }
 
 StreamsModel::CategoryItem * StreamsModel::CategoryItem::createBookmarksCategory()
@@ -282,7 +277,7 @@ QList<StreamsModel::Item *> StreamsModel::XmlCategoryItem::loadCache()
     if (QFile::exists(cacheName)) {
         newItems=loadXml(cacheName);
         QString dir=Utils::getDir(cacheName);
-        foreach (Item *i, newItems) {
+        for (Item *i: newItems) {
             if (i->isCategory()) {
                 StreamsModel::CategoryItem *cat=static_cast<StreamsModel::CategoryItem *>(i);
                 QString name=cat->name;
@@ -317,7 +312,7 @@ static void saveStream(QXmlStreamWriter &doc, const StreamsModel::Item *item)
 {
     doc.writeStartElement("stream");
     doc.writeAttribute("name", item->name);
-    doc.writeAttribute("url", item->url);
+    doc.writeAttribute("url", item->fullUrl());
     doc.writeEndElement();
 }
 
@@ -331,7 +326,7 @@ static void saveCategory(QXmlStreamWriter &doc, const StreamsModel::CategoryItem
     if (cat->isAll) {
         doc.writeAttribute("isAll", "true");
     }
-    foreach (const StreamsModel::Item *i, cat->children) {
+    for (const StreamsModel::Item *i: cat->children) {
         if (i->isCategory()) {
             saveCategory(doc, static_cast<const StreamsModel::CategoryItem *>(i));
         } else {
@@ -354,7 +349,7 @@ bool StreamsModel::CategoryItem::saveXml(QIODevice *dev, bool format) const
         doc.setAutoFormatting(false);
     }
 
-    foreach (const Item *i, children) {
+    for (const Item *i: children) {
         if (i->isCategory()) {
             ::saveCategory(doc, static_cast<const CategoryItem *>(i));
         } else {
@@ -434,8 +429,8 @@ QList<StreamsModel::Item *> StreamsModel::FavouritesCategoryItem::loadXml(QIODev
     QSet<QString> existingUrls;
     QSet<QString> existingNames;
 
-    foreach (Item *i, children) {
-        existingUrls.insert(i->url);
+    for (Item *i: children) {
+        existingUrls.insert(i->fullUrl());
         existingNames.insert(i->name);
     }
 
@@ -477,11 +472,11 @@ void StreamsModel::ShoutCastCategoryItem::addHeaders(QNetworkRequest &req)
 
 NetworkJob * StreamsModel::ShoutCastCategoryItem::fetchSecondardyUrl()
 {
-    if (url!=constShoutCastUrl) {
+    if (!url.startsWith(constShoutCastUrl)) {
         // Get stations...
         QUrl url(QLatin1String("http://")+constShoutCastHost+QLatin1String("/legacy/genresearch"));
         QUrlQuery query;
-        query.addQueryItem("k", constShoutCastApiKey);
+        ApiKeys::self()->addKey(query, ApiKeys::ShoutCast);
         query.addQueryItem("genre", name);
         url.setQuery(query);
 
@@ -491,7 +486,7 @@ NetworkJob * StreamsModel::ShoutCastCategoryItem::fetchSecondardyUrl()
         job->setProperty(constOrigUrlProperty, url.toString());
         return job;
     }
-    return 0;
+    return nullptr;
 }
 
 void StreamsModel::DiCategoryItem::addHeaders(QNetworkRequest &req)
@@ -523,7 +518,7 @@ StreamsModel::StreamsModel(QObject *parent)
     reloadAction = new Action(Icons::self()->reloadIcon, tr("Reload"), this);
 
     QSet<QString> hidden=Settings::self()->hiddenStreamCategories().toSet();
-    foreach (Item *c, root->children) {
+    for (Item *c: root->children) {
         if (c!=favourites) {
             CategoryItem *cat=static_cast<CategoryItem *>(c);
             if (hidden.contains(cat->configName)) {
@@ -570,7 +565,7 @@ QModelIndex StreamsModel::index(int row, int column, const QModelIndex &parent)
     }
 
     const CategoryItem * p = parent.isValid() ? static_cast<CategoryItem *>(parent.internalPointer()) : root;
-    const Item * c = row<p->children.count() ? p->children.at(row) : 0;
+    const Item * c = row<p->children.count() ? p->children.at(row) : nullptr;
     return c ? createIndex(row, column, (void *)c) : QModelIndex();
 }
 
@@ -639,7 +634,7 @@ QVariant StreamsModel::data(const QModelIndex &index, int role) const
         if (!Settings::self()->infoTooltips()) {
             return QVariant();
         }
-        return item->isCategory() ? item->name : (item->name+QLatin1String("<br><small><i>")+item->url+QLatin1String("</i></small>"));
+        return item->isCategory() ? item->name : (item->name+QLatin1String("<br><small><i>")+item->fullUrl()+QLatin1String("</i></small>"));
     case Cantata::Role_SubText:
         if (item->isCategory()) {
             const CategoryItem *cat=static_cast<const CategoryItem *>(item);
@@ -725,13 +720,13 @@ void StreamsModel::fetchMore(const QModelIndex &index)
         if (!cat->isFavourites() && !loadCache(cat)) {
             QNetworkRequest req;
             if (cat->isDi()) {
-                req=QNetworkRequest(constDiChannelListUrl.arg(cat->url.split(".").at(1)));
+                req=QNetworkRequest(constDiChannelListUrl.arg(cat->fullUrl().split(".").at(1)));
             } else {
-                req=QNetworkRequest(cat->url);
+                req=QNetworkRequest(cat->fullUrl());
             }
             cat->addHeaders(req);
             NetworkJob *job=NetworkAccessManager::self()->get(req);
-            job->setProperty(constOrigUrlProperty, cat->url);
+            job->setProperty(constOrigUrlProperty, cat->fullUrl());
             if (jobs.isEmpty()) {
                 emit loading();
             }
@@ -776,8 +771,8 @@ void StreamsModel::importIntoFavourites(const QString &fileName)
 {
     QList<Item *> newItems=favourites->CategoryItem::loadXml(fileName);
     if (!newItems.isEmpty()) {
-        foreach (Item *i, newItems) {
-            emit saveFavouriteStream(i->url, i->name);
+        for (Item *i: newItems) {
+            emit saveFavouriteStream(i->fullUrl(), i->name);
         }
         qDeleteAll(newItems);
     }
@@ -786,7 +781,7 @@ void StreamsModel::importIntoFavourites(const QString &fileName)
 void StreamsModel::removeFromFavourites(const QModelIndexList &indexes)
 {
     QList<quint32> rows;
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         rows.append(idx.row());
     }
 
@@ -799,8 +794,8 @@ bool StreamsModel::addToFavourites(const QString &url, const QString &name)
 {
     QSet<QString> existingNames;
 
-    foreach (Item *i, favourites->children) {
-        if (i->url==url) {
+    for (Item *i: favourites->children) {
+        if (i->fullUrl()==url) {
             return false;
         }
         existingNames.insert(i->name);
@@ -821,8 +816,8 @@ bool StreamsModel::addToFavourites(const QString &url, const QString &name)
 
 QString StreamsModel::favouritesNameForUrl(const QString &u)
 {
-    foreach (Item *i, favourites->children) {
-        if (i->url==u) {
+    for (Item *i: favourites->children) {
+        if (i->fullUrl()==u) {
             return i->name;
         }
     }
@@ -831,7 +826,7 @@ QString StreamsModel::favouritesNameForUrl(const QString &u)
 
 bool StreamsModel::nameExistsInFavourites(const QString &n)
 {
-    foreach (Item *i, favourites->children) {
+    for (Item *i: favourites->children) {
         if (i->name==n) {
             return true;
         }
@@ -860,8 +855,8 @@ bool StreamsModel::addBookmark(const QString &url, const QString &name, Category
             endInsertRows();
         }
 
-        foreach (Item *i, bookmarkCat->children) {
-            if (i->url==url) {
+        for (Item *i: bookmarkCat->children) {
+            if (i->fullUrl()==url) {
                 return false;
             }
         }
@@ -923,14 +918,14 @@ QModelIndex StreamsModel::favouritesIndex() const
 
 static QString addDiHash(const StreamsModel::Item *item)
 {
-    return ( (item->parent && item->parent->isDi()) || DigitallyImported::self()->isDiUrl(item->url) )
-            ? DigitallyImported::self()->modifyUrl(item->url) : item->url;
+    return ( (item->parent && item->parent->isDi()) || DigitallyImported::self()->isDiUrl(item->fullUrl()) )
+            ? DigitallyImported::self()->modifyUrl(item->fullUrl()) : item->fullUrl();
 }
 
 QStringList StreamsModel::filenames(const QModelIndexList &indexes, bool addPrefix) const
 {
     QStringList fnames;
-    foreach(QModelIndex index, indexes) {
+    for (const QModelIndex &index: indexes) {
         Item *item=static_cast<Item *>(index.internalPointer());
         if (!item->isCategory()) {
             fnames << modifyUrl(addDiHash(item), addPrefix, addPrefix ? item->modifiedName() : item->name);
@@ -957,7 +952,7 @@ QStringList StreamsModel::mimeTypes() const
 void StreamsModel::save()
 {
     QStringList disabled;
-    foreach (Item *i, hiddenCategories) {
+    for (Item *i: hiddenCategories) {
         disabled.append(static_cast<CategoryItem *>(i)->configName);
     }
     disabled.sort();
@@ -967,11 +962,11 @@ void StreamsModel::save()
 QList<StreamsModel::Category> StreamsModel::getCategories() const
 {
     QList<StreamsModel::Category> categories;
-    foreach (Item *i, hiddenCategories) {
+    for (Item *i: hiddenCategories) {
         categories.append(Category(i->name, static_cast<CategoryItem *>(i)->icon, static_cast<CategoryItem *>(i)->configName,
                                    static_cast<CategoryItem *>(i)->isBuiltIn(), true, static_cast<CategoryItem *>(i)->isDi()));
     }
-    foreach (Item *i, root->children) {
+    for (Item *i: root->children) {
         if (i!=favourites) {
             categories.append(Category(i->name, static_cast<CategoryItem *>(i)->icon, static_cast<CategoryItem *>(i)->configName,
                                        static_cast<CategoryItem *>(i)->isBuiltIn(), false, static_cast<CategoryItem *>(i)->isDi()));
@@ -983,7 +978,7 @@ QList<StreamsModel::Category> StreamsModel::getCategories() const
 void StreamsModel::setHiddenCategories(const QSet<QString> &cats)
 {
     bool changed=false;
-    foreach (Item *i, hiddenCategories) {
+    for (Item *i: hiddenCategories) {
         if (!cats.contains(static_cast<CategoryItem *>(i)->configName)) {
             beginInsertRows(QModelIndex(), root->children.count(), root->children.count());
             root->children.append(i);
@@ -993,7 +988,7 @@ void StreamsModel::setHiddenCategories(const QSet<QString> &cats)
         }
     }
 
-    foreach (Item *i, root->children) {
+    for (Item *i: root->children) {
         if (cats.contains(static_cast<CategoryItem *>(i)->configName)) {
             int row=root->children.indexOf(i);
             if (row>0) {
@@ -1044,7 +1039,7 @@ void StreamsModel::jobFinished()
         // ShoutCast and Dirble have two jobs when listing a category - child categories and station list.
         // So, only set as fetched if both are finished.
         bool haveOtherJob=false;
-        foreach (CategoryItem *c, jobs.values()) {
+        for (CategoryItem *c: jobs.values()) {
             if (c==cat) {
                 haveOtherJob=true;
                 break;
@@ -1056,10 +1051,11 @@ void StreamsModel::jobFinished()
         }
 
         QModelIndex index=createIndex(cat->parent->children.indexOf(cat), 0, (void *)cat);
+        QString url=job->origUrl().toString();
+
         if (job->ok()) {
             QList<Item *> newItems;
             if (cat!=favourites && QLatin1String("http")==job->url().scheme()) {
-                QString url=job->origUrl().toString();
                 if (constRadioTimeHost==job->origUrl().host()) {
                     newItems=parseRadioTimeResponse(job->actualJob(), cat);
                 } else if (constIceCastUrl==url) {
@@ -1084,11 +1080,11 @@ void StreamsModel::jobFinished()
 
                     if (bookmarksCat) {
                         QList<Item *> newBookmarks;
-                        foreach (Item *bm, bookmarks) {
-                            foreach (Item *ex, bookmarksCat->children) {
-                                if (ex->url==bm->url) {
+                        for (Item *bm: bookmarks) {
+                            for (Item *ex: bookmarksCat->children) {
+                                if (ex->fullUrl()==bm->fullUrl()) {
                                     delete bm;
-                                    bm=0;
+                                    bm=nullptr;
                                     break;
                                 }
                             }
@@ -1105,7 +1101,7 @@ void StreamsModel::jobFinished()
                         }
                     } else {
                         bookmarksCat=cat->createBookmarksCategory();
-                        foreach (Item *i, bookmarks) {
+                        for (Item *i: bookmarks) {
                             i->parent=bookmarksCat;
                         }
                         bookmarksCat->children=bookmarks;
@@ -1122,6 +1118,12 @@ void StreamsModel::jobFinished()
                     cat->saveCache();
                 }
             }
+        } else {
+            if (constShoutCastHost==job->origUrl().host()) {
+                ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::ShoutCast);
+            } else if (constDirbleHost==job->origUrl().host()) {
+                ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::Dirble);
+            }
         }
         emit dataChanged(index, index);
         if (jobs.isEmpty()) {
@@ -1179,16 +1181,16 @@ void StreamsModel::removedFavouriteStreams(const QList<quint32> &removed)
 static StreamsModel::Item * getStream(StreamsModel::FavouritesCategoryItem *fav, const Stream &stream, int offset)
 {
     int i=0;
-    foreach (StreamsModel::Item *s, fav->children) {
+    for (StreamsModel::Item *s: fav->children) {
         if (++i<offset) {
             continue;
         }
-        if (s->name==stream.name && s->url==stream.url) {
+        if (s->name==stream.name && s->fullUrl()==stream.url) {
             return s;
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 void StreamsModel::favouriteStreams(const QList<Stream> &streams)
@@ -1203,7 +1205,7 @@ void StreamsModel::favouriteStreams(const QList<Stream> &streams)
             return;
         }
         beginInsertRows(idx, 0, streams.count()-1);
-        foreach (const Stream &s, streams) {
+        for (const Stream &s: streams) {
             favourites->children.append(new Item(s.url, s.name, favourites));
         }
         endInsertRows();
@@ -1215,9 +1217,9 @@ void StreamsModel::favouriteStreams(const QList<Stream> &streams)
     } else {
         for (qint32 i=0; i<streams.count(); ++i) {
             Stream s=streams.at(i);
-            Item *si=i<favourites->children.count() ? favourites->children.at(i) : 0;
-            if (!si || si->name!=s.name || si->url!=s.url) {
-                si=i<favourites->children.count() ? getStream(favourites, s, i) : 0;
+            Item *si=i<favourites->children.count() ? favourites->children.at(i) : nullptr;
+            if (!si || si->name!=s.name || si->fullUrl()!=s.url) {
+                si=i<favourites->children.count() ? getStream(favourites, s, i) : nullptr;
                 if (!si) {
                     beginInsertRows(idx, i, i);
                     favourites->children.insert(i, new Item(s.url, s.name, favourites));
@@ -1241,8 +1243,6 @@ void StreamsModel::favouriteStreams(const QList<Stream> &streams)
         }
     }
 
-    importOldFavourites();
-
     if (CategoryItem::Fetched!=favourites->state) {
         favourites->state=CategoryItem::Fetched;
         emit dataChanged(idx, idx);
@@ -1278,7 +1278,7 @@ static QStringList fixGenres(const QString &genre)
 {
     QStringList allGenres=Song::capitalize(genre).split(' ', QString::SkipEmptyParts);
     QStringList fixed;
-    foreach (const QString &genre, allGenres) {
+    for (const QString &genre: allGenres) {
         if (genre.length() < 2 ||
             genre.contains("ÃÂ") ||  // Broken unicode.
             genre.contains(QRegExp("^#x[0-9a-f][0-9a-f]"))) { // Broken XML entities.
@@ -1301,7 +1301,7 @@ static void trimGenres(QMap<QString, QList<StreamsModel::Item *> > &genres)
 {
     QString other=QObject::tr("Other");
     QSet<QString> genreSet = genres.keys().toSet();
-    foreach (const QString &genre, genreSet) {
+    for (const QString &genre: genreSet) {
         if (other!=genre && genres[genre].count() < 2) {
             genres[other]+=genres[genre];
             genres.remove(genre);
@@ -1344,7 +1344,7 @@ QList<StreamsModel::Item *> StreamsModel::parseIceCastResponse(QIODevice *dev, C
 
             if (!name.isEmpty() && !url.isEmpty() && !names.contains(name)) {
                 names.insert(name);
-                foreach (const QString &g, stationGenres) {
+                for (const QString &g: stationGenres) {
                     genres[g].append(new Item(url, name, cat));
                 }
             }
@@ -1357,7 +1357,7 @@ QList<StreamsModel::Item *> StreamsModel::parseIceCastResponse(QIODevice *dev, C
     for (; it!=end; ++it) {
         CategoryItem *genre=new CategoryItem(QString(), it.key(), cat);
         genre->state=CategoryItem::Fetched;
-        foreach (Item *i, it.value()) {
+        for (Item *i: it.value()) {
             i->parent=genre;
             genre->children.append(i);
         }
@@ -1387,12 +1387,12 @@ QList<StreamsModel::Item *> StreamsModel::parseDigitallyImportedResponse(QIODevi
 {
     QList<Item *> newItems;
     QVariantMap data=QJsonDocument::fromJson(dev->readAll()).toVariant().toMap();
-    QString listenHost=QLatin1String("listen.")+QUrl(cat->url).host().remove("www.");
+    QString listenHost=QLatin1String("listen.")+QUrl(cat->fullUrl()).host().remove("www.");
 
     if (data.contains("channel_filters")) {
         QVariantList filters = data["channel_filters"].toList();
 
-        foreach (const QVariant &filter, filters) {
+        for (const QVariant &filter: filters) {
             // Find the filter called "All"
             QVariantMap filterMap = filter.toMap();
             if (filterMap.value("name", QString()).toString() != "All") {
@@ -1401,7 +1401,7 @@ QList<StreamsModel::Item *> StreamsModel::parseDigitallyImportedResponse(QIODevi
 
             // Add all its stations to the result
             QVariantList channels = filterMap.value("channels", QVariantList()).toList();
-            foreach (const QVariant &channel, channels) {
+            for (const QVariant &channel: channels) {
                 QVariantMap channelMap = channel.toMap();
                 QString url=constDiStdUrl.arg(listenHost).arg(channelMap.value("key").toString());
                 newItems.append(new Item(url, channelMap.value("name").toString(), cat));
@@ -1583,8 +1583,8 @@ QList<StreamsModel::Item *> StreamsModel::parseShoutCastLinks(QXmlStreamReader &
     while (!doc.atEnd()) {
         doc.readNext();
         if (doc.isStartElement() && QLatin1String("genre")==doc.name()) {
-            newItems.append(new ShoutCastCategoryItem(QLatin1String("http://")+constShoutCastHost+QLatin1String("/genre/secondary?parentid=")+
-                                                      doc.attributes().value("id").toString()+QLatin1String("&f=xml&k=")+constShoutCastApiKey,
+            newItems.append(new ShoutCastCategoryItem(ApiKeys::self()->addKey(QLatin1String("http://")+constShoutCastHost+QLatin1String("/genre/secondary?parentid=")+
+                                                                              doc.attributes().value("id").toString()+QLatin1String("&f=xml"), ApiKeys::ShoutCast),
                                                       doc.attributes().value("name").toString(), cat));
         } else if (doc.isEndElement() && QLatin1String("genrelist")==doc.name()) {
             return newItems;
@@ -1624,10 +1624,11 @@ QList<StreamsModel::Item *> StreamsModel::parseDirbleResponse(QIODevice *dev, Ca
 
     // Get categories...::f
     if (!data.isEmpty()) {
-        foreach (const QVariant &d, data) {
+        for (const QVariant &d: data) {
             QVariantMap map = d.toMap();
-            newItems.append(new DirbleCategoryItem(QLatin1String("http://")+constDirbleHost+QLatin1String("/v2/category/")+map["id"].toString()+
-                                                   QLatin1String("/stations?token=")+StreamsModel::constDirbleApiKey, map["title"].toString(), cat));
+            newItems.append(new DirbleCategoryItem(ApiKeys::self()->addKey(QLatin1String("http://")+constDirbleHost+QLatin1String("/v2/category/")+map["id"].toString()+
+                                                                           QLatin1String("/stations"), ApiKeys::Dirble),
+                                                   map["title"].toString(), cat));
         }
     }
     return newItems;
@@ -1639,14 +1640,14 @@ QList<StreamsModel::Item *> StreamsModel::parseDirbleStations(QIODevice *dev, Ca
     QVariantList data=QJsonDocument::fromJson(dev->readAll()).toVariant().toList();
 
     QSet<QString> added;
-    foreach (const QVariant &d, data) {
+    for (const QVariant &d: data) {
         QVariantMap map = d.toMap();
         QString name=map["name"].toString().trimmed().simplified();
         if (!name.isEmpty()) {
             QVariantList streams=map["streams"].toList();
             int bitrate=0;
             QString url;
-            foreach (const QVariant &s, streams) {
+            for (const QVariant &s: streams) {
                 QVariantMap streamMap=s.toMap();
                 int br=streamMap["bitrate"].toInt();
                 QString u=streamMap["stream"].toString().trimmed().simplified();
@@ -1667,8 +1668,8 @@ QList<StreamsModel::Item *> StreamsModel::parseDirbleStations(QIODevice *dev, Ca
 
 StreamsModel::Item * StreamsModel::parseRadioTimeEntry(QXmlStreamReader &doc, CategoryItem *parent, bool parseSubText)
 {
-    Item *item=0;
-    CategoryItem *cat=0;
+    Item *item=nullptr;
+    CategoryItem *cat=nullptr;
     while (!doc.atEnd()) {
         if (doc.isStartElement()) {
             QString text=doc.attributes().value("text").toString();
@@ -1730,25 +1731,7 @@ StreamsModel::Item * StreamsModel::parseSomaFmEntry(QXmlStreamReader &doc, Categ
         }
     }
 
-    return name.isEmpty() || url.isEmpty() ? 0 : new Item(url, name, parent);
-}
-
-void StreamsModel::importOldFavourites()
-{
-    if (!favourites->importedOld) {
-        if (Settings::self()->version()<CANTATA_MAKE_VERSION(1,3,54)) {
-            QString prevFile=Settings::self()->storeStreamsInMpdDir()
-                                ? MPDConnection::self()->getDetails().dir : Utils::dataDir(QString(), false);
-
-            if (!prevFile.isEmpty()) {
-                prevFile+="streams.xml.gz";
-                if (QFile::exists(prevFile)) {
-                    importIntoFavourites(prevFile);
-                }
-            }
-        }
-        favourites->importedOld=true;
-    }
+    return name.isEmpty() || url.isEmpty() ? nullptr : new Item(url, name, parent);
 }
 
 void StreamsModel::loadInstalledProviders()
@@ -1757,9 +1740,9 @@ void StreamsModel::loadInstalledProviders()
     QString dir=Utils::dataDir(StreamsModel::constSubDir);
     QStringList subDirs=QDir(dir).entryList(QStringList() << "*", QDir::Dirs|QDir::Readable|QDir::NoDot|QDir::NoDotDot);
     QStringList streamFiles=QStringList() << constCompressedXmlFile << constXmlFile << constSettingsFile;
-    foreach (const QString &sub, subDirs) {
+    for (const QString &sub: subDirs) {
         if (!added.contains(sub)) {
-            foreach (const QString &streamFile, streamFiles) {
+            for (const QString &streamFile: streamFiles) {
                 if (QFile::exists(dir+sub+Utils::constDirSep+streamFile)) {
                     addInstalledProvider(sub, getExternalIcon(dir+sub), dir+sub+Utils::constDirSep+streamFile, false);
                     added.insert(sub);
@@ -1772,7 +1755,7 @@ void StreamsModel::loadInstalledProviders()
 
 StreamsModel::CategoryItem * StreamsModel::addInstalledProvider(const QString &name, const Icon &icon, const QString &streamsFileName, bool replace)
 {
-    CategoryItem *cat=0;
+    CategoryItem *cat=nullptr;
     if (streamsFileName.endsWith(constSettingsFile)) {
         QFile file(streamsFileName);
         if (file.open(QIODevice::ReadOnly)) {
@@ -1783,7 +1766,7 @@ StreamsModel::CategoryItem * StreamsModel::addInstalledProvider(const QString &n
             if (!url.isEmpty() && !type.isEmpty()) {
                 QStringList toRemove=QStringList() << " " << "." << "/" << "\\" << "(" << ")";
                 QString cacheName=name;
-                foreach (const QString &rem, toRemove) {
+                for (const QString &rem: toRemove) {
                     cacheName=cacheName.replace(rem, "");
                 }
                 cacheName=cacheName.toLower();
@@ -1801,7 +1784,7 @@ StreamsModel::CategoryItem * StreamsModel::addInstalledProvider(const QString &n
     }
 
     if (!cat) {
-        return 0;
+        return nullptr;
     }
 
     cat->configName="x-"+name;
@@ -1823,7 +1806,7 @@ StreamsModel::CategoryItem * StreamsModel::addInstalledProvider(const QString &n
 
 void StreamsModel::removeInstalledProvider(const QString &key)
 {
-    foreach (Item *i, root->children) {
+    for (Item *i: root->children) {
         if (key==static_cast<CategoryItem *>(i)->configName) {
             int row=root->children.indexOf(i);
             if (row>=0) {
@@ -1849,3 +1832,5 @@ QString StreamsModel::modifyUrl(const QString &u, bool addPrefix, const QString
 {
     return MPDParseUtils::addStreamName(!addPrefix || !u.startsWith("http:") ? u : (constPrefix+u), name);
 }
+
+#include "moc_streamsmodel.cpp"
