@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,6 +25,8 @@
 #include "mpdconnection.h"
 #include "mpdstatus.h"
 #include "gui/settings.h"
+#include "support/globalstatic.h"
+#include "support/configuration.h"
 #ifndef LIBVLC_FOUND
 #include <QtMultimedia/QMediaPlayer>
 #endif
@@ -41,16 +43,27 @@ void HttpStream::enableDebug()
     debugEnabled=true;
 }
 
+GLOBAL_STATIC(HttpStream, instance)
+
 HttpStream::HttpStream(QObject *p)
     : QObject(p)
     , enabled(false)
     , state(MPDState_Inactive)
+    , muted(false)
     , playStateChecks(0)
-    , playStateCheckTimer(0)
-    , player(0)
+    , currentVolume(50)
+    , unmuteVol(50)
+    , playStateCheckTimer(nullptr)
+    , player(nullptr)
 {
 }
 
+void HttpStream::save() const
+{
+    Configuration config(metaObject()->className());
+    config.set("volume", currentVolume);
+}
+
 void HttpStream::setEnabled(bool e)
 {
     if (e==enabled) {
@@ -66,6 +79,7 @@ void HttpStream::setEnabled(bool e)
         disconnect(MPDConnection::self(), SIGNAL(streamUrl(QString)), this, SLOT(streamUrl(QString)));
         disconnect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(updateStatus()));
         if (player) {
+            save();
             #ifdef LIBVLC_FOUND
             libvlc_media_player_stop(player);
             #else
@@ -73,6 +87,54 @@ void HttpStream::setEnabled(bool e)
             #endif
         }
     }
+    emit isEnabled(enabled);
+}
+
+void HttpStream::setVolume(int vol)
+{
+    DBUG << vol;
+    if (player) {
+        currentVolume=vol;
+        #ifdef LIBVLC_FOUND
+        libvlc_audio_set_volume(player, vol);
+        #else
+        player->setVolume(vol);
+        #endif
+        emit update();
+    }
+}
+
+int HttpStream::volume()
+{
+    int vol=currentVolume;
+    if (player && !isMuted()) {
+        #ifdef LIBVLC_FOUND
+        vol=libvlc_audio_get_volume(player);
+        #else
+        vol=player->volume();
+        #endif
+        if (vol<0) {
+            vol=currentVolume;
+        } else {
+            currentVolume=vol;
+        }
+    }
+    DBUG << vol;
+    return vol;
+}
+
+void HttpStream::toggleMute()
+{
+    DBUG << isMuted();
+    if (player) {
+        muted=!muted;
+        #ifdef LIBVLC_FOUND
+        libvlc_audio_set_mute(player, muted);
+        #else
+        player->setMuted(!muted);
+        #endif
+        emit update();
+    }
 }
 
 void HttpStream::streamUrl(const QString &url)
@@ -91,7 +153,7 @@ void HttpStream::streamUrl(const QString &url)
     if (player && player->property(constUrlProperty).toString()!=url) {
         player->stop();
         player->deleteLater();
-        player=0;
+        player=nullptr;
     }
     #endif
     if (!url.isEmpty() && !player) {
@@ -106,14 +168,16 @@ void HttpStream::streamUrl(const QString &url)
         player->setMedia(QUrl(url));
         player->setProperty(constUrlProperty, url);
         #endif
+        muted=false;
+        setVolume(Configuration(metaObject()->className()).get("volume", currentVolume));
     }
-
     if (player) {
         state=0xFFFF; // Force an update...
         updateStatus();
     } else {
         state=MPDState_Inactive;
     }
+    emit update();
 }
 
 void HttpStream::updateStatus()
@@ -206,3 +270,5 @@ void HttpStream::stopTimer()
     }
     playStateChecks=0;
 }
+
+#include "moc_httpstream.cpp"
