@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -195,6 +195,8 @@ QString MPDConnection::Response::getError(const QByteArray &command)
 MPDConnectionDetails::MPDConnectionDetails()
     : port(6600)
     , dirReadable(false)
+    , allowLocalStreaming(false)
+    , autoUpdate(false)
 {
 }
 
@@ -225,7 +227,7 @@ void MPDConnectionDetails::setDirReadable()
 
 MPDConnection::MPDConnection()
     : isInitialConnect(true)
-    , thread(0)
+    , thread(nullptr)
     , ver(0)
     , canUseStickers(false)
     , sock(this)
@@ -234,15 +236,14 @@ MPDConnection::MPDConnection()
     , lastUpdatePlayQueueVersion(0)
     , state(State_Blank)
     , isListingMusic(false)
-    , reconnectTimer(0)
+    , reconnectTimer(nullptr)
     , reconnectStart(0)
     , stopAfterCurrent(false)
     , currentSongId(-1)
     , songPos(0)
     , unmuteVol(-1)
-    , mopidy(false)
     , isUpdatingDb(false)
-    , volumeFade(0)
+    , volumeFade(nullptr)
     , fadeDuration(0)
     , restoreVolume(-1)
 {
@@ -308,9 +309,9 @@ void MPDConnection::stop()
 
     if (thread) {
         thread->deleteTimer(connTimer);
-        connTimer=0;
+        connTimer=nullptr;
         thread->stop();
-        thread=0;
+        thread=nullptr;
     }
 }
 
@@ -475,7 +476,7 @@ void MPDConnection::disconnectFromMPD()
     lastUpdatePlayQueueVersion=0;
     currentSongId=0;
     songPos=0;
-    mopidy=false;
+    serverInfo.reset();
     isUpdatingDb=false;
     emit socketAddress(QString());
 }
@@ -486,7 +487,6 @@ void MPDConnection::reconnect()
     if (reconnectTimer && reconnectTimer->isActive()) {
         return;
     }
-    qWarning() << reconnectStart;
     if (0==reconnectStart && isConnected()) {
         disconnectFromMPD();
     }
@@ -495,8 +495,7 @@ void MPDConnection::reconnect()
         reconnectStart=0;
         return;
     }
-qWarning() << "TRY TO CONNECT";
-    time_t now=time(NULL);
+    time_t now=time(nullptr);
     ConnectionReturn status=connectToMPD();
     switch (status) {
     case Success:        
@@ -580,7 +579,6 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
         DBUG << "call connectToMPD";
         unmuteVol=-1;
         toggleStopAfterCurrent(false);
-        mopidy=false;
         #ifdef ENABLE_SIMPLE_MPD_SUPPORT
         if (isUser) {
             MPDUser::self()->start();
@@ -597,6 +595,7 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
             if (replaygainSupported() && !details.replayGain.isEmpty()) {
                 sendCommand("replay_gain_mode "+details.replayGain.toLatin1());
             }
+            serverInfo.detect();
             getStatus();
             getStats();
             getUrlHandlers();
@@ -694,7 +693,7 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
             bool emitError=true;
             // Mopidy returns "incorrect arguments" for commands it does not support. The docs state that crossfade and replaygain mode
             // setting commands are not supported. So, if we get this error then just ignore it.
-            if (mopidy && (command.startsWith("crossfade ") || command.startsWith("replay_gain_mode "))) {
+            if (!isMpd() && (command.startsWith("crossfade ") || command.startsWith("replay_gain_mode "))) {
                 emitError=false;
             }
             if (emitError) {
@@ -738,9 +737,12 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
  */
 bool MPDConnection::isPlaylist(const QString &file)
 {
-    return file.endsWith(QLatin1String(".asx"), Qt::CaseInsensitive) || file.endsWith(QLatin1String(".cue"), Qt::CaseInsensitive) ||
-           file.endsWith(QLatin1String(".m3u"), Qt::CaseInsensitive) || file.endsWith(QLatin1String(".pls"), Qt::CaseInsensitive) ||
-           file.endsWith(QLatin1String(".xspf"), Qt::CaseInsensitive);
+    static QSet<QString> extensions = QSet<QString>() << QLatin1String("asx") << QLatin1String("cue")
+                                                      << QLatin1String("m3u") << QLatin1String("m3u8")
+                                                      << QLatin1String("pls") << QLatin1String("xspf");
+
+    int pos=file.lastIndexOf('.');
+    return pos>0 ? extensions.contains(file.mid(pos+1).toLower()) : false;
 }
 
 void MPDConnection::add(const QStringList &files, int action, quint8 priority, bool decreasePriority)
@@ -787,7 +789,7 @@ void MPDConnection::add(const QStringList &origList, quint32 pos, quint32 size,
     }
 
     QStringList files;
-    foreach (const QString &file, origList) {
+    for (const QString &file: origList) {
         if (file.startsWith(constDirPrefix)) {
             files+=getAllFiles(file.mid(constDirPrefix.length()));
         } else if (file.startsWith(constPlaylistPrefix)) {
@@ -832,7 +834,7 @@ void MPDConnection::add(const QStringList &origList, quint32 pos, quint32 size,
         curPos=playQueueIds.size();
     }
 
-    foreach (const QStringList &files, fileLists) {
+    for (const QStringList &files: fileLists) {
         QByteArray send = "command_list_begin\n";
 
         for (int i = 0; i < files.size(); i++) {
@@ -900,7 +902,11 @@ void MPDConnection::addAndPlay(const QString &file)
     if (response.ok) {
         MPDStatusValues sv=MPDParseUtils::parseStatus(response.data);
         QByteArray send = "command_list_begin\n";
-        send+="add "+encodeName(file)+'\n';
+        if (CueFile::isCue(file)) {
+            send += "load "+CueFile::getLoadLine(file)+'\n';
+        } else {
+            send+="add "+encodeName(file)+'\n';
+        }
         send+="play "+quote(sv.playlistLength)+'\n';
         send+="command_list_end";
         sendCommand(send);
@@ -921,7 +927,7 @@ void MPDConnection::removeSongs(const QList<qint32> &items)
 {
     toggleStopAfterCurrent(false);
     QByteArray send = "command_list_begin\n";
-    foreach (qint32 i, items) {
+    for (qint32 i: items) {
         send += "deleteid "+quote(i)+'\n';
     }
 
@@ -1049,7 +1055,7 @@ void MPDConnection::playListChanges()
     QByteArray data = "plchangesposid "+quote(lastUpdatePlayQueueVersion);
     Response status=sendCommand("status"); // We need an updated status so as to detect deletes at end of list...
     Response response=sendCommand(data, false);
-    if (response.ok && status.ok) {
+    if (response.ok && status.ok && isPlayQueueIdValid()) {
         MPDStatusValues sv=MPDParseUtils::parseStatus(status.data);
         lastUpdatePlayQueueVersion=lastStatusPlayQueueVersion=sv.playlist;
         emitStatusUpdated(sv);
@@ -1067,7 +1073,7 @@ void MPDConnection::playListChanges()
             QSet<qint32> prevIds=playQueueIds.toSet();
             QSet<qint32> strmIds;
 
-            foreach (const MPDParseUtils::IdPos &idp, changes) {
+            for (const MPDParseUtils::IdPos &idp: changes) {
                 if (first) {
                     first=false;
                     firstPos=idp.pos;
@@ -1164,7 +1170,7 @@ void MPDConnection::playListInfo()
         streamIds.clear();
 
         QList<Song> cStreams;
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             playQueueIds.append(s.id);
             if (s.isCdda()) {
                 cStreams.append(s);
@@ -1403,9 +1409,7 @@ void MPDConnection::getStats()
     if (response.ok) {
         MPDStatsValues stats=MPDParseUtils::parseStats(response.data);
         dbUpdate=stats.dbUpdate;
-        mopidy=0==stats.artists && 0==stats.albums && 0==stats.songs &&
-               0==stats.uptime && 0==stats.playtime && 0==stats.dbPlaytime && 0==dbUpdate;
-        if (mopidy) {
+        if (isMopidy()) {
             // Set version to 1 so that SQL cache is updated - it uses 0 as intial value
             dbUpdate=stats.dbUpdate=1;
         }
@@ -1536,7 +1540,7 @@ void MPDConnection::parseIdleReturn(const QByteArray &data)
      */
     bool playListUpdated=false;
     bool statusUpdated=false;
-    foreach(const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (line.startsWith(constIdleChangedKey)) {
             QByteArray value=line.mid(constIdleChangedKey.length());
             if (constIdleDbValue==value) {
@@ -1595,9 +1599,16 @@ void MPDConnection::enableOutput(int id, bool enable)
 /*
  * Admin commands
  */
+void MPDConnection::updateMaybe()
+{
+    if (!details.autoUpdate) {
+        update();
+    }
+}
+
 void MPDConnection::update()
 {
-    if (mopidy) {
+    if (isMopidy()) {
         // Mopidy does not support MPD's update command. So, when user presses update DB, what we
         // just reload the library.
         loadLibrary();
@@ -1690,8 +1701,11 @@ void MPDConnection::removePlaylist(const QString &name)
     sendCommand("rm "+encodeName(name));
 }
 
-void MPDConnection::savePlaylist(const QString &name)
+void MPDConnection::savePlaylist(const QString &name, bool overwrite)
 {
+    if (overwrite) {
+        sendCommand("rm "+encodeName(name), false);
+    }
     if (!sendCommand("save "+encodeName(name), false).ok) {
         emit error(tr("Failed to save <b>%1</b>").arg(name));
     }
@@ -1704,7 +1718,7 @@ void MPDConnection::addToPlaylist(const QString &name, const QStringList &songs,
     }
 
     if (!name.isEmpty()) {
-        foreach (const QString &song, songs) {
+        for (const QString &song: songs) {
             if (CueFile::isCue(song)) {
                 emit error(tr("You cannot add parts of a cue sheet to a playlist!")+QLatin1String(" (")+song+QLatin1Char(')'));
                 return;
@@ -1716,7 +1730,7 @@ void MPDConnection::addToPlaylist(const QString &name, const QStringList &songs,
     }
 
     QStringList files;
-    foreach (const QString &file, songs) {
+    for (const QString &file: songs) {
         if (file.startsWith(constDirPrefix)) {
             files+=getAllFiles(file.mid(constDirPrefix.length()));
         } else if (file.startsWith(constPlaylistPrefix)) {
@@ -1728,7 +1742,7 @@ void MPDConnection::addToPlaylist(const QString &name, const QStringList &songs,
 
     QByteArray encodedName=encodeName(name);
     QByteArray send = "command_list_begin\n";
-    foreach (const QString &s, files) {
+    for (const QString &s: files) {
         send += "playlistadd "+encodedName+" "+encodeName(s)+'\n';
     }
     send += "command_list_end";
@@ -1783,7 +1797,7 @@ void MPDConnection::setPriority(const QList<qint32> &ids, quint8 priority, bool
         QMap<qint32, quint8> tracks;
         QByteArray send = "command_list_begin\n";
 
-        foreach (quint32 id, ids) {
+        for (quint32 id: ids) {
             tracks.insert(id, priority);
             send += "prioid "+quote(priority)+" "+quote(id)+'\n';
             if (decreasePriority && priority>0) {
@@ -1837,7 +1851,7 @@ void MPDConnection::search(const QByteArray &query, const QString &id)
         Response response=sendCommand("list albumartist", false, false);
         if (response.ok) {
             QList<QByteArray> lines = response.data.split('\n');
-            foreach (const QByteArray &line, lines) {
+            for (const QByteArray &line: lines) {
                 if (line.startsWith("AlbumArtist: ")) {
                     Response resp = sendCommand("find albumartist " + encodeName(QString::fromUtf8(line.mid(13))) , false, false);
                     if (resp.ok) {
@@ -1855,7 +1869,7 @@ void MPDConnection::search(const QByteArray &query, const QString &id)
                 int max = parts.at(2).toInt();
                 QList<MPDParseUtils::Sticker> stickers=MPDParseUtils::parseStickers(response.data, constRatingSticker);
                 if (!stickers.isEmpty()) {
-                    foreach (const MPDParseUtils::Sticker &sticker, stickers) {
+                    for (const MPDParseUtils::Sticker &sticker: stickers) {
                         if (!sticker.file.isEmpty() && !sticker.value.isEmpty()) {
                             int val = sticker.value.toInt();
                             if (val>=min && val<=max) {
@@ -1884,7 +1898,7 @@ void MPDConnection::listStreams()
     QList<Stream> streams;
     if (response.ok) {
         QList<Song> songs=MPDParseUtils::parseSongs(response.data, MPDParseUtils::Loc_Streams);
-        foreach (const Song &song, songs) {
+        for (const Song &song: songs) {
             streams.append(Stream(song.file, song.name()));
         }    
     }
@@ -1957,7 +1971,7 @@ void MPDConnection::sendDynamicMessage(const QStringList &msg)
     }
 
     QByteArray data;
-    foreach (QString part, msg) {
+    for (QString part: msg) {
         if (data.isEmpty()) {
             data+='\"'+part.toUtf8()+':'+dynamicId;
         } else {
@@ -2041,7 +2055,7 @@ bool MPDConnection::recursivelyListDir(const QString &dir, QList<Song> &songs)
 {
     bool topLevel="/"==dir || ""==dir;
 
-    if (topLevel && !mopidy) {
+    if (topLevel && isMpd()) {
         // UPnP database backend does not list separate metadata items, so if "list genre" returns
         // empty response assume this is a UPnP backend and dont attempt to get rest of data...
         // Although we dont use "list XXX", lsinfo will return duplciate items (due to the way most
@@ -2058,7 +2072,7 @@ bool MPDConnection::recursivelyListDir(const QString &dir, QList<Song> &songs)
     }
 
     Response response=sendCommand(topLevel
-                                    ? QByteArray(mopidy ? "lsinfo \"Local media\"" : "lsinfo")
+                                    ? serverInfo.getTopLevelLsinfo()
                                     : ("lsinfo "+encodeName(dir)));
     if (response.ok) {
         QStringList subDirs;
@@ -2070,7 +2084,7 @@ bool MPDConnection::recursivelyListDir(const QString &dir, QList<Song> &songs)
             emit librarySongs(copy);
             songs.clear();
         }
-        foreach (const QString &sub, subDirs) {
+        for (const QString &sub: subDirs) {
             if (!recursivelyListDir(sub, songs)) {
                 return false;
             }
@@ -2094,7 +2108,7 @@ QStringList MPDConnection::getPlaylistFiles(const QString &name)
     if (response.ok) {
         QList<Song> songs=MPDParseUtils::parseSongs(response.data, MPDParseUtils::Loc_Playlists);
         emit playlistInfoRetrieved(name, songs);
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             files.append(s.file);
         }    
     }
@@ -2109,12 +2123,12 @@ QStringList MPDConnection::getAllFiles(const QString &dir)
         QStringList subDirs;
         QList<Song> songs;
         MPDParseUtils::parseDirItems(response.data, details.dir, ver, songs, dir, subDirs, MPDParseUtils::Loc_Browse);
-        foreach (const Song &song, songs) {
+        for (const Song &song: songs) {
             if (Song::Playlist!=song.type) {
                 files.append(song.file);
             }
         }
-        foreach (const QString &sub, subDirs) {
+        for (const QString &sub: subDirs) {
             files+=getAllFiles(sub);
         }
     }
@@ -2181,14 +2195,14 @@ void MPDConnection::readRemoteDynamicMessages()
             MPDParseUtils::MessageMap messages=MPDParseUtils::parseMessages(response.data);
             if (!messages.isEmpty()) {
                 QList<QByteArray> channels=QList<QByteArray>() << constDynamicOut << constDynamicOut+'-'+dynamicId;
-                foreach (const QByteArray &channel, channels) {
+                for (const QByteArray &channel: channels) {
                     if (messages.contains(channel)) {
-                        foreach (const QString &m, messages[channel]) {
+                        for (const QString &m: messages[channel]) {
                             if (!m.isEmpty()) {
                                 DBUG << "Received message " << m;
                                 QStringList parts=m.split(':', QString::SkipEmptyParts);
                                 QStringList message;
-                                foreach (QString part, parts) {
+                                for (QString part: parts) {
                                     part=part.replace("{c}", ":");
                                     part=part.replace("{n}", "\n");
                                     part=part.replace("{cb}", "}");
@@ -2265,10 +2279,10 @@ void MPDConnection::setRating(const QStringList &files, quint8 val)
     }
 
     bool ok=true;
-    foreach (const QStringList &list, fileLists) {
+    for (const QStringList &list: fileLists) {
         QByteArray cmd = "command_list_begin\n";
 
-        foreach (const QString &f, list) {
+        for (const QString &f: list) {
             if (0==val) {
                 cmd+="sticker delete song "+encodeName(f)+' '+constRatingSticker+'\n';
             } else {
@@ -2400,8 +2414,8 @@ void MPDConnection::determineIfaceIp()
 
 MpdSocket::MpdSocket(QObject *parent)
     : QObject(parent)
-    , tcp(0)
-    , local(0)
+    , tcp(nullptr)
+    , local(nullptr)
 {
 }
 
@@ -2446,7 +2460,7 @@ void MpdSocket::deleteTcp()
         disconnect(tcp, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SIGNAL(stateChanged(QAbstractSocket::SocketState)));
         disconnect(tcp, SIGNAL(readyRead()), this, SIGNAL(readyRead()));
         tcp->deleteLater();
-        tcp=0;
+        tcp=nullptr;
     }
 }
 
@@ -2456,6 +2470,94 @@ void MpdSocket::deleteLocal()
         disconnect(local, SIGNAL(stateChanged(QLocalSocket::LocalSocketState)), this, SLOT(localStateChanged(QLocalSocket::LocalSocketState)));
         disconnect(local, SIGNAL(readyRead()), this, SIGNAL(readyRead()));
         local->deleteLater();
-        local=0;
+        local=nullptr;
     }
 }
+
+// ONLY use this method to detect Non-MPD servers. The code which uses this will default to MPD
+MPDServerInfo::ResponseParameter MPDServerInfo::lsinfoResponseParameters[] = {
+    // github
+    { "{lsinfo} Directory info not found for virtual-path '/", true, MPDServerInfo::ForkedDaapd, "forked-daapd" },
+    // { "ACK [50@0] {lsinfo} Not found", false, MPDServerInfo::Mopidy, "Mopidy" },
+    // ubuntu 16.10
+    { "OK", false, MPDServerInfo::ForkedDaapd, "forked-daapd" }
+};
+
+void MPDServerInfo::detect(void) {
+    MPDConnection *conn;
+
+    if (!isUndetermined()) {
+        return;
+    }
+
+    conn = MPDConnection::self();
+
+    if (isUndetermined()) {
+        MPDConnection::Response response=conn->sendCommand("stats");
+        if (response.ok) {
+            MPDStatsValues stats=MPDParseUtils::parseStats(response.data);
+            if (0==stats.artists && 0==stats.albums && 0==stats.songs
+                && 0==stats.uptime && 0==stats.playtime && 0==stats.dbPlaytime
+                && 0==stats.dbUpdate) {
+                setServerType(Mopidy);
+                serverName = "Mopidy";
+            }
+        }
+    }
+
+    if (isUndetermined()) {
+        MPDConnection::Response response=conn->sendCommand(lsinfoCommand, false, false);
+        QList<QByteArray> lines = response.data.split('\n');
+        bool match = false;
+        int indx;
+        for (const QByteArray &line: lines) {
+            for (indx=0; indx<sizeof(lsinfoResponseParameters)/sizeof(ResponseParameter); ++indx) {
+                ResponseParameter &rp = lsinfoResponseParameters[indx];
+                if (rp.isSubstring) {
+                    match = line.toLower().contains(rp.response.toLower());
+                } else {
+                    match = line.toLower() == rp.response.toLower();
+                }
+                if (match) {
+                    setServerType(rp.serverType);
+                    serverName = rp.name;
+                    break;
+                }
+            }
+            // first line is currently enough
+            break;
+        }
+    }
+
+    if (isUndetermined()) {
+        // Default to MPD if cannot determine otherwise. Cantata is an *MPD* client first and foremost.
+        setServerType(Mpd);
+        serverName = "MPD";
+    }
+
+    DBUG << "detected serverType:" << getServerName() << "(" << getServerType() << ")";
+
+    if (isMopidy()) {
+        topLevelLsinfo = "lsinfo \"Local media\"";
+    }
+
+    if (isForkedDaapd()) {
+        topLevelLsinfo = "lsinfo file:";
+
+        QByteArray message = "sendmessage rating \"";
+        message += "rating ";                           // sticker name
+        message += QString().number(Song::Rating_Max);  // max rating
+        message += " ";
+        message += QString().number(Song::Rating_Step); // rating step (optional)
+        message += "\"";
+        conn->sendCommand(message, false, false);
+    }
+}
+
+void MPDServerInfo::reset(void) {
+    setServerType(MPDServerInfo::Undetermined);
+    serverName = "undetermined";
+    topLevelLsinfo = "lsinfo";
+}
+
+#include "moc_mpdconnection.cpp"
