@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -51,7 +51,7 @@ class MpdSocket : public QObject
     Q_OBJECT
 public:
     MpdSocket(QObject *parent);
-    virtual ~MpdSocket();
+    ~MpdSocket() override;
 
     void connectToHost(const QString &hostName, quint16 port, QIODevice::OpenMode mode = QIODevice::ReadWrite);
     void disconnectFromHost() {
@@ -114,7 +114,7 @@ public:
                         : QAbstractSocket::UnconnectedState;
     }
     QNetworkProxy::ProxyType proxyType() const { return tcp ? tcp->proxy().type() : QNetworkProxy::NoProxy; }
-    bool isLocal() const { return 0!=local; }
+    bool isLocal() const { return nullptr!=local; }
     QString address() const { return tcp ? tcp->peerAddress().toString() : QString(); }
     QString errorString() const { return tcp ? tcp->errorString() : local ? local->errorString() : QLatin1String("No socket object?"); }
     QAbstractSocket::SocketError error() const {
@@ -157,11 +157,60 @@ struct MPDConnectionDetails {
     QString password;
     QString dir;
     bool dirReadable;
-    QString coverName;
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
     QString streamUrl;
     #endif
     QString replayGain;
+    bool allowLocalStreaming;
+    bool autoUpdate;
+};
+
+class MPDServerInfo {
+public:
+    enum ServerType {
+        Undetermined,
+        Mpd,
+        Mopidy,
+        ForkedDaapd,
+        Unknown
+    };
+
+    MPDServerInfo() {
+        reset();
+        lsinfoCommand = "lsinfo \"mpd-client://cantata/";
+        lsinfoCommand += PACKAGE_VERSION_STRING;
+        lsinfoCommand += "\"";
+    };
+
+    void reset(void);
+    void detect(void);
+
+    ServerType getServerType(void) const { return serverType;}
+    bool isUndetermined(void) const { return serverType == Undetermined; }
+    bool isMpd(void) const { return serverType == Mpd; }
+    bool isMopidy(void) const { return serverType == Mopidy; }
+    bool isForkedDaapd(void) const { return serverType == ForkedDaapd; }
+    bool isPlayQueueIdValid(void) const { return serverType != ForkedDaapd; }
+
+    const QString &getServerName(void) const { return serverName;}
+    const QByteArray &getTopLevelLsinfo(void) const { return topLevelLsinfo; }
+
+private:
+    void setServerType(ServerType newServerType) { serverType = newServerType; }
+
+    ServerType serverType;
+    QString serverName;
+    QByteArray topLevelLsinfo;
+
+    struct ResponseParameter {
+        QByteArray response;
+        bool isSubstring;
+        ServerType serverType;
+        QString name;
+    };
+
+    QByteArray lsinfoCommand;
+    static ResponseParameter lsinfoResponseParameters[];
 };
 
 class MPDConnection : public QObject
@@ -207,13 +256,13 @@ public:
     static void enableDebug();
 
     MPDConnection();
-    ~MPDConnection();
+    ~MPDConnection() override;
 
     void start();
     const MPDConnectionDetails & getDetails() const { return details; }
     void setDirReadable() { details.setDirReadable(); }
     bool isConnected() const { return State_Connected==state; }
-    bool canUsePriority() const { return ver>=CANTATA_MAKE_VERSION(0, 17, 0) && !mopidy; }
+    bool canUsePriority() const { return ver>=CANTATA_MAKE_VERSION(0, 17, 0) && isMpd(); }
     const QSet<QString> & urlHandlers() const { return handlers; }
     const QSet<QString> & tags() const { return tagTypes; }
     bool composerTagSupported() const { return tagTypes.contains(QLatin1String("Composer")); }
@@ -229,7 +278,10 @@ public:
     static bool isPlaylist(const QString &file);
     int unmuteVolume() { return unmuteVol; }
     bool isMuted() { return -1!=unmuteVol; }
-    bool isMopidy() const { return mopidy; }
+    bool isMpd(void) const { return serverInfo.isMpd(); }
+    bool isMopidy(void) const { return serverInfo.isMopidy(); }
+    bool isForkedDaapd(void) const { return serverInfo.isForkedDaapd(); }
+    bool isPlayQueueIdValid(void) const { return serverInfo.isPlayQueueIdValid(); }
     void setVolumeFadeDuration(int f) { fadeDuration=f; }
     QString ipAddress() const { return details.isLocal() ? QString() : sock.address(); }
 
@@ -292,6 +344,7 @@ public Q_SLOTS:
     void listFolder(const QString &folder);
 
     // Admin
+    void updateMaybe();
     void update();
 
     // Playlists
@@ -301,7 +354,7 @@ public Q_SLOTS:
     void loadPlaylist(const QString &name, bool replace);
     void renamePlaylist(const QString oldName, const QString newName);
     void removePlaylist(const QString &name);
-    void savePlaylist(const QString &name);
+    void savePlaylist(const QString &name, bool overwrite);
     void addToPlaylist(const QString &name, const QStringList &songs) { addToPlaylist(name, songs, 0, 0); }
     void addToPlaylist(const QString &name, const QStringList &songs, quint32 pos, quint32 size);
     void removeFromPlaylist(const QString &name, const QList<quint32> &positions);
@@ -460,7 +513,8 @@ private:
     qint32 currentSongId;
     quint32 songPos; // USe for stop-after-current when we only have 1 songin playqueue!
     int unmuteVol;
-    bool mopidy;
+    friend class MPDServerInfo;
+    MPDServerInfo serverInfo;
     bool isUpdatingDb;
 
     QPropertyAnimation *volumeFade;
@@ -468,5 +522,4 @@ private:
     int restoreVolume;
 };
 
-
 #endif
