@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -190,7 +190,7 @@ MPDStatsValues MPDParseUtils::parseStats(const QByteArray &data)
     MPDStatsValues v;
     QList<QByteArray> lines = data.split('\n');
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (line.startsWith(constStatsArtistsKey)) {
             v.artists=line.mid(constStatsArtistsKey.length()).toUInt();
         } else if (line.startsWith(constStatsAlbumsKey)) {
@@ -215,7 +215,7 @@ MPDStatusValues MPDParseUtils::parseStatus(const QByteArray &data)
     MPDStatusValues v;
     QList<QByteArray> lines = data.split('\n');
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (line.startsWith(constStatusVolumeKey)) {
             v.volume=line.mid(constStatusVolumeKey.length()).toInt();
         } else if (line.startsWith(constStatusConsumeKey)) {
@@ -298,7 +298,7 @@ static QSet<QString> constStdProtocols=QSet<QString>() << constHttpProtocol
 Song MPDParseUtils::parseSong(const QList<QByteArray> &lines, Location location)
 {
     Song song;
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (line.startsWith(constFileKey)) {
             song.file = QString::fromUtf8(line.mid(constFileKey.length()));
         } else if (line.startsWith(constTimeKey) ){
@@ -381,6 +381,7 @@ Song MPDParseUtils::parseSong(const QList<QByteArray> &lines, Location location)
         if (!song.file.isEmpty() && song.file.startsWith(constHttpProtocol) && HttpServer::self()->isOurs(song.file)) {
             song.type=Song::CantataStream;
             Song mod=HttpServer::self()->decodeUrl(song.file);
+            mod.priority=song.priority;
             if (!mod.title.isEmpty()) {
                 mod.id=song.id;
                 song=mod;
@@ -391,7 +392,7 @@ Song MPDParseUtils::parseSong(const QList<QByteArray> &lines, Location location)
             if (song.file.contains(Song::constCddaProtocol)) {
                 song.type=Song::Cdda;
             } else if (song.file.contains(constProtocol)) {
-                foreach (const QString &protocol, constStdProtocols) {
+                for (const QString &protocol: constStdProtocols) {
                     if (song.file.startsWith(protocol)) {
                         song.type=Song::Stream;
                         break;
@@ -517,7 +518,7 @@ QStringList MPDParseUtils::parseList(const QByteArray &data, const QByteArray &k
     QList<QByteArray> lines = data.split('\n');
     int keyLen=key.length();
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         // Skip the "OK" line, this is NOT a valid item!!!
         if (constOkValue==line) {
             continue;
@@ -536,7 +537,7 @@ MPDParseUtils::MessageMap MPDParseUtils::parseMessages(const QByteArray &data)
     QList<QByteArray> lines = data.split('\n');
     QByteArray channel;
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         // Skip the "OK" line, this is NOT a valid item!!!
         if (constOkValue==line) {
             continue;
@@ -572,6 +573,7 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                 continue;
             }
 
+            DBUG << currentSong.file;
             if (Song::Playlist==currentSong.type) {
                 // lsinfo will return all stored playlists - but this is deprecated.
                 if (!parsePlaylists) {
@@ -622,9 +624,10 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                 bool canSplitCue=mpdVersion>=CANTATA_MAKE_VERSION(0,17,0);
                 bool parseCue=canSplitCue && currentSong.isCueFile() && !mpdDir.startsWith(constHttpProtocol) && QFile::exists(mpdDir+currentSong.file);
                 bool cueParseStatus=false;
+                double lastTrackIndex=0.0;
                 if (parseCue) {
                     DBUG << "Parsing cue file:" << currentSong.file << "mpdDir:" << mpdDir;
-                    cueParseStatus=CueFile::parse(currentSong.file, mpdDir, cueSongs, cueFiles);
+                    cueParseStatus=CueFile::parse(currentSong.file, mpdDir, cueSongs, cueFiles, lastTrackIndex);
                     if (!cueParseStatus) {
                         DBUG << "Failed to parse cue file!";
                         continue;
@@ -634,7 +637,7 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                         (cueFiles.count()<cueSongs.count() || (firstSong.albumArtist().isEmpty() && firstSong.album.isEmpty()))) {
 
                     bool canUseThisCueFile=true;
-                    foreach (const Song &s, cueSongs) {
+                    for (const Song &s: cueSongs) {
                         if (!QFile::exists(mpdDir+s.name())) {
                             DBUG << QString(mpdDir+s.name()) << "is referenced in cue file, but does not exist in MPD folder";
                             canUseThisCueFile=false;
@@ -652,7 +655,7 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                     if (songs.size()==cueFiles.size()) {
                         quint32 albumTime=0;
                         QMap<QString, Song> origFiles;
-                        foreach (const Song &s, songs) {
+                        for (const Song &s: songs) {
                             origFiles.insert(s.file, s);
                             albumTime+=s.time;
                         }
@@ -660,8 +663,7 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
 
                         bool setTimeFromSource=origFiles.size()==cueSongs.size();
                         DBUG << "setTimeFromSource" << setTimeFromSource << "at" << albumTime << "#c" << cueFiles.size();
-                        quint32 usedAlbumTime=0;
-                        foreach (const Song &orig, cueSongs) {
+                        for (const Song &orig: cueSongs) {
                             Song s=orig;
                             Song albumSong=origFiles[s.name()];
                             s.setName(QString()); // CueFile has placed source file name here!
@@ -687,14 +689,10 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                             }
                             if (0==s.time && setTimeFromSource) {
                                 s.time=albumSong.time;
-                            } else if (0!=albumTime && 1==cueFiles.size()) {
-                                DBUG << s.title << s.time << albumTime << usedAlbumTime;
+                            } else if (0==s.time && 1==cueFiles.size()) {
+                                DBUG << "Set time of last track" << s.title << s.time << albumTime << (lastTrackIndex/1000.0);
                                 // Try to set duration of last track by subtracting previous track durations from album duration...
-                                if (0==s.time) {
-                                    s.time=albumTime-usedAlbumTime;
-                                } else {
-                                    usedAlbumTime+=s.time;
-                                }
+                                s.time=albumTime-(lastTrackIndex/1000.0);
                             }
                             DBUG << s.title << s.time;
                             fixedCueSongs.append(s);
@@ -705,7 +703,7 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
                     if (!canUseCueFileTracks) {
                         // Album had a different number of source files to the CUE file. If so, then we need to ensure
                         // all tracks have meta data - otherwise just fallback to listing file + cue
-                        foreach (const Song &orig, cueSongs) {
+                        for (const Song &orig: cueSongs) {
                             Song s=orig;
                             s.setName(QString()); // CueFile has placed source file name here!
                             if (s.artist.isEmpty() || s.album.isEmpty()) {
@@ -733,13 +731,20 @@ void MPDParseUtils::parseDirItems(const QByteArray &data, const QString &mpdDir,
             } else {
                 if (setSingleTracks) {
                     currentSong.albumartist=Song::variousArtists();
-                    currentSong.album=QObject::tr("Single Tracks");
+                    currentSong.album=Song::singleTracks();
+                    currentSong.type=Song::SingleTracks;
+                    currentSong.setAlbumArtistSort(QString());
+                    currentSong.setAlbumSort(QString());
+                    currentSong.setMbAlbumId(QString());
                 }
                 currentSong.fillEmptyFields();
                 songs.append(currentSong);
             }
         }
     }
+    if (Loc_Browse==loc) {
+        qSort(songs);
+    }
     songList+=songs;
 }
 
@@ -749,7 +754,7 @@ QList<Output> MPDParseUtils::parseOuputs(const QByteArray &data)
     QList<QByteArray> lines = data.split('\n');
     Output output;
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (constOkValue==line) {
             break;
         }
@@ -779,7 +784,7 @@ QByteArray MPDParseUtils::parseSticker(const QByteArray &data, const QByteArray
 {
     QList<QByteArray> lines = data.split('\n');
     QByteArray key=constSticker+sticker+'=';
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (line.startsWith(key)) {
             return line.mid(key.length());
         }
@@ -794,7 +799,7 @@ QList<MPDParseUtils::Sticker> MPDParseUtils::parseStickers(const QByteArray &dat
     Sticker s;
     QByteArray key=constSticker+sticker+'=';
 
-    foreach (const QByteArray &line, lines) {
+    for (const QByteArray &line: lines) {
         if (constOkValue==line) {
             break;
         }
