@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -41,9 +41,13 @@
 
 const QString Song::constCddaProtocol=QLatin1String("/[cantata-cdda]/");
 const QString Song::constMopidyLocal=QLatin1String("local:track:");
+const QString Song::constForkedDaapdLocal=QLatin1String("file:");
 
 static QString unknownStr;
 static QString variousArtistsStr;
+static QString singleTracksStr;
+static bool useOrigYear = false;
+
 const QString & Song::unknown()
 {
     return unknownStr;
@@ -54,10 +58,16 @@ const QString & Song::variousArtists()
     return variousArtistsStr;
 }
 
+const QString & Song::singleTracks()
+{
+    return singleTracksStr;
+}
+
 void Song::initTranslations()
 {
     unknownStr=QObject::tr("Unknown");
     variousArtistsStr=QObject::tr("Various Artists");
+    singleTracksStr=QObject::tr("Single Tracks");
 }
 
 // When displaying albums, we use the 1st track's year as the year of the album.
@@ -67,13 +77,13 @@ static QHash<QString, quint16> albumYears;
 
 void Song::storeAlbumYear(const Song &s)
 {
-    albumYears.insert(s.albumKey(), s.year);
+    albumYears.insert(s.albumKey(), s.displayYear());
 }
 
 int Song::albumYear(const Song &s)
 {
     QHash<QString, quint16>::ConstIterator it=albumYears.find(s.albumKey());
-    return it==albumYears.end() ? s.year : it.value();
+    return it==albumYears.end() ? s.displayYear() : it.value();
 }
 
 static int songType(const Song &s)
@@ -124,7 +134,13 @@ QString Song::decodePath(const QString &file, bool cdda)
     if (cdda) {
         return QString(file).replace("/", "_").replace(":", "_");
     }
-    return file.startsWith(constMopidyLocal) ? QUrl::fromPercentEncoding(file.mid(constMopidyLocal.length()).toLatin1()) : file;
+    if (file.startsWith(constMopidyLocal)) {
+        return QUrl::fromPercentEncoding(file.mid(constMopidyLocal.length()).toLatin1());
+    }
+    if (file.startsWith(constForkedDaapdLocal)) {
+        return file.mid(constForkedDaapdLocal.length());
+    }
+    return file;
 }
 
 QString Song::encodePath(const QString &file)
@@ -240,11 +256,20 @@ int Song::compareTo(const Song &o) const
         if (track!=o.track) {
             return track<o.track ? -1 : 1;
         }
-        if (year!=o.year) {
-            return year<o.year ? -1 : 1;
-        }
-        if (origYear!=o.origYear) {
-            return origYear<o.origYear ? -1 : 1;
+        if (useOrigYear) {
+            if (origYear!=o.origYear) {
+                return origYear<o.origYear ? -1 : 1;
+            }
+            if (year!=o.year) {
+                return year<o.year ? -1 : 1;
+            }
+        } else {
+            if (year!=o.year) {
+                return year<o.year ? -1 : 1;
+            }
+            if (origYear!=o.origYear) {
+                return origYear<o.origYear ? -1 : 1;
+            }
         }
         compare=title.localeAwareCompare(o.title);
         if (0!=compare) {
@@ -270,6 +295,12 @@ bool Song::isEmpty() const
     return (artist.isEmpty() && album.isEmpty() && title.isEmpty() && name().isEmpty()) || file.isEmpty();
 }
 
+bool Song::sameMetadata(const Song &o) const
+{
+    return disc == o.disc && track == o.track && time == o.time && year == o.year && origYear == o.origYear &&
+           artist == o.artist && albumartist == o.albumartist && album == o.album && title == o.title;
+}
+
 void Song::guessTags()
 {
     if (isEmpty() && !isStream()) {
@@ -299,7 +330,7 @@ void Song::guessTags()
             static const QSet<QChar> constSeparators=QSet<QChar>() << QLatin1Char(' ') << QLatin1Char('-') << QLatin1Char('_') << QLatin1Char('.');
             int separator=0;
 
-            foreach (const QChar &sep, constSeparators) {
+            for (const QChar &sep: constSeparators) {
                 separator=title.indexOf(sep);
                 if (1==separator || 2==separator) {
                     break;
@@ -382,7 +413,7 @@ void Song::setIgnorePrefixes(const QSet<QString> &prefixes)
 
 static QString ignorePrefix(const QString &str)
 {
-    foreach (const QString &p, prefixesToIngore) {
+    for (const QString &p: prefixesToIngore) {
         if (str.startsWith(p+QLatin1Char(' '))) {
             return str.mid(p.length()+1)+QLatin1String(", ")+p;
         }
@@ -401,6 +432,16 @@ QString Song::sortString(const QString &str)
     return sort==str ? QString() : sort;
 }
 
+bool Song::useOriginalYear()
+{
+    return useOrigYear;
+}
+
+void Song::setUseOriginalYear(bool u)
+{
+    useOrigYear = u;
+}
+
 quint16 Song::setKey(int location)
 {
     if (isStandardStream()) {
@@ -460,6 +501,11 @@ void Song::addGenre(const QString &g)
     }
 }
 
+quint16 Song::displayYear() const
+{
+    return useOrigYear && origYear>0 ? origYear : year;
+}
+
 QString Song::entryName() const
 {
     if (title.isEmpty()) {
@@ -516,7 +562,7 @@ QString Song::trackAndTitleStr(bool showArtistIfDifferent) const
 //               QLatin1Char(' ')+(addArtist ? artistSong() : title);
 //    }
     return //(disc>0 ? (QString::number(disc)+QLatin1Char('.')) : QString())+
-           (track>0 ? (track>9 ? QString::number(track)+QLatin1Char(' ') : (QLatin1Char('0')+QString::number(track)+QLatin1Char(' '))) : QString())+
+           (track>0 && SingleTracks!=type ? (track>9 ? QString::number(track)+QLatin1Char(' ') : (QLatin1Char('0')+QString::number(track)+QLatin1Char(' '))) : QString())+
            (showArtistIfDifferent && diffArtist() ? artistSong() : title) +
            (origYear>0 && origYear != year ? QLatin1String(" (")+QString::number(origYear)+QLatin1Char(')') : QString());
 }
@@ -555,7 +601,7 @@ QString Song::toolTip() const
         addField(QObject::tr("Year"), QString::number(year), toolTip);
     }
     if (origYear>0) {
-        addField(QObject::tr("Orignal Year"), QString::number(origYear), toolTip);
+        addField(QObject::tr("Original Year"), QString::number(origYear), toolTip);
     }
     if (time>0) {
         addField(QObject::tr("Length"), Utils::formatTime(time, true), toolTip);
@@ -710,8 +756,8 @@ QString Song::basicArtist() const
     QStringList toStrip=QStringList() << QLatin1String("ft. ") << QLatin1String("feat. ") << QLatin1String("featuring ") << QLatin1String("f. ");
     QStringList prefixes=QStringList() << QLatin1String(" ") << QLatin1String(" (") << QLatin1String(" [");
 
-    foreach (const QString s, toStrip) {
-        foreach (const QString p, prefixes) {
+    for (const QString &s: toStrip) {
+        for (const QString &p: prefixes) {
             int strip = artist.toLower().indexOf(p+s);
             if (-1!=strip) {
                 return artist.mid(0, strip);
@@ -721,22 +767,72 @@ QString Song::basicArtist() const
     return artist;
 }
 
-QString Song::describe(bool withMarkup) const
+QString Song::filePath(const QString &base) const
+{
+    if (isCantataStream()) {
+        return QUrl(file).path();
+    }
+    if (isLocalFile()) {
+        return file;
+    }
+    QString fileName=decodePath(file, isCdda());
+    if (!base.isEmpty()) {
+        bool haveAbsPath=fileName.startsWith("/"); // Utils::constDirSep
+        if (!haveAbsPath) {
+            return QString(base+fileName);
+        }
+    }
+    return fileName;
+}
+
+QString Song::describe() const
 {
     QString albumText=album.isEmpty() ? name() : displayAlbum(album, Song::albumYear(*this));
 
     if (title.isEmpty()) {
-        return withMarkup ? albumText : (QLatin1String("<b>")+albumText+QLatin1String("</b>"));
+        return QLatin1String("<b>")+albumText+QLatin1String("</b>");
     }
-    QString descr=artist.isEmpty()
+    return artist.isEmpty()
                     ? QObject::tr("<b>%1</b> on <b>%2</b>", "Song on Album").arg(title).arg(albumText)
                     : QObject::tr("<b>%1</b> by <b>%2</b> on <b>%3</b>", "Song by Artist on Album").arg(title).arg(artist).arg(albumText);
+}
 
-    if (!withMarkup) {
-        descr=descr.replace("<b>", "");
-        descr=descr.replace("</b>", "");
+QString Song::mainText() const
+{
+    if (isEmpty()) {
+        return QString();
+    }
+
+    QString n = name();
+    if (isStream() && !isCantataStream() && !isCdda() && !isDlnaStream()) {
+        return n.isEmpty() ? Song::unknown() : n;
+    } else if (title.isEmpty() && artist.isEmpty() && (!n.isEmpty() || !file.isEmpty())) {
+        return n.isEmpty() ? file : n;
+    } else {
+        return title+(origYear>0 && !Song::useOriginalYear() && origYear!=year ? QLatin1String(" (")+QString::number(origYear)+QLatin1Char(')') : QString());
+    }
+}
+
+QString Song::subText() const
+{
+    if (isEmpty()) {
+        return QString();
+    }
+
+    if (isStream() && !isCantataStream() && !isCdda() && !isDlnaStream()) {
+        if (artist.isEmpty() && title.isEmpty() && !name().isEmpty()) {
+            return QObject::tr("(Stream)");
+        } else {
+            return artist.isEmpty() ? title : artistSong();
+        }
+    } else if (album.isEmpty() && artist.isEmpty()) {
+        return mainText().isEmpty() ? QString() : Song::unknown();
+    } else if (album.isEmpty()) {
+        return artist;
+    } else {
+        // Artist here is always artist, and not album artist or composer
+        return artist + QString(" – ")+displayAlbum(false);
     }
-    return descr;
 }
 
 bool Song::useComposer() const
