@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 
@@ -82,7 +82,8 @@ struct Song
         Stream          = 3,
         CantataStream   = 4,
         Cdda            = 5,
-        OnlineSvrTrack  = 6
+        OnlineSvrTrack  = 6,
+        LocalFile       = 7
     };
 
     enum Blank {
@@ -105,8 +106,8 @@ struct Song
     quint16 time;
     quint16 track;
     quint16 origYear;
-    quint16 year : 12;
-    mutable Type type : 3;
+    quint16 year;
+    mutable Type type : 7;
     mutable bool guessed : 1;
     qint32 id;
     qint32 size;
@@ -118,9 +119,11 @@ struct Song
 
     static const QString & unknown();
     static const QString & variousArtists();
+    static const QString & singleTracks();
     static void initTranslations();
     static const QString constCddaProtocol;
     static const QString constMopidyLocal;
+    static const QString constForkedDaapdLocal;
     static void storeAlbumYear(const Song &s);
     static int albumYear(const Song &s);
     static void sortViaType(QList<Song> &songs);
@@ -132,6 +135,8 @@ struct Song
     static QSet<QString> ignorePrefixes();
     static void setIgnorePrefixes(const QSet<QString> &prefixes);
     static QString sortString(const QString &str);
+    static bool useOriginalYear();
+    static void setUseOriginalYear(bool u);
 
     Song();
     Song(const Song &o) { *this=o; }
@@ -143,12 +148,14 @@ struct Song
     virtual ~Song() { }
     bool isEmpty() const;
     bool isDifferent(const Song &s) const { return file!=s.file || year!=s.year || artist!=s.artist || album!=s.album || title!=s.title || name()!=s.name(); }
+    bool sameMetadata(const Song &o) const;
     void guessTags();
     void revertGuessedTags();
     void fillEmptyFields();
     quint16 setKey(int location);
     virtual void clear();
     void addGenre(const QString &g);
+    quint16 displayYear() const;
     QString entryName() const;
     QString artistOrComposer() const;
     QString albumName() const;
@@ -204,19 +211,24 @@ struct Song
     bool setAlbumArtist();
     static QString capitalize(const QString &s);
     bool capitalise();
+    bool isLocalFile() const { return LocalFile==type; }
     bool isStream() const { return Stream==type || CantataStream==type; }
     bool isStandardStream() const { return Stream==type && !isDlnaStream(); }
     bool isDlnaStream() const { return Stream==type && !albumArtist().isEmpty() && !album.isEmpty() && track>0; }
     bool isNonMPD() const { return isStream() || OnlineSvrTrack==type || Cdda==type || (!file.isEmpty() && file.startsWith(Utils::constDirSep)); }
+    bool hasProtocolOrIsAbsolute() const { return !file.isEmpty() && (file.startsWith(Utils::constDirSep) || (!file.startsWith(constForkedDaapdLocal) && file.contains(":/")));}
     bool isCantataStream() const { return CantataStream==type; }
     bool isCdda() const { return Cdda==type; }
     QString albumKey() const;
     bool isCueFile() const { return Playlist==type && file.endsWith(QLatin1String(".cue"), Qt::CaseInsensitive); }
     bool isFromCue() const { return CueFile::isCue(file); }
     QString basicArtist() const;
-    QString filePath() const { return decodePath(file, isCdda()); }
-    QString displayAlbum(bool useComp=true) const { return displayAlbum(useComp ? albumName() : album, year); }
-    QString describe(bool withMarkup=false) const;
+    QString filePath(const QString &base=QString()) const;
+    QString displayAlbum(bool useComp=true) const { return displayAlbum(useComp ? albumName() : album, displayYear()); }
+    QString describe() const;
+    // Main text + sub text for now-playing and notifications
+    QString mainText() const;
+    QString subText() const;
     bool useComposer() const;
     void populateSorts();
 //    QString basicDescription() const;
