@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -76,7 +76,7 @@ Song MagnatuneXmlParser::parseSong(QXmlStreamReader &xml)
                 s.year=value.toInt();
             } else  if (QLatin1String("magnatunegenres")==name) {
                 QStringList genres=value.split(',', QString::SkipEmptyParts);
-//                foreach (const QString &g, genres) {
+//                for (const QString &g: genres) {
 //                    s.addGenre(g);
 //                }
                 s.genres[0]=genres.first();
@@ -264,3 +264,5 @@ void MagnatuneService::configure(QWidget *p)
         cfg.set("username", username);
         cfg.set("password", password);    }
 }
+
+#include "moc_magnatuneservice.cpp"
