@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -57,6 +57,7 @@ OnlineDbWidget::OnlineDbWidget(OnlineDbService *s, QWidget *p)
     connect(view, SIGNAL(updateToPlayQueue(QModelIndex,bool)), this, SLOT(updateToPlayQueue(QModelIndex,bool)));
     view->setOpenAfterSearch(SqlLibraryModel::T_Album!=srv->topLevel());
     connect(StdActions::self()->addRandomAlbumToPlayQueueAction, SIGNAL(triggered()), SLOT(addRandomAlbum()));
+    connect(srv, SIGNAL(modelReset()), this, SLOT(modelReset()));
 }
 
 OnlineDbWidget::~OnlineDbWidget()
@@ -135,7 +136,7 @@ void OnlineDbWidget::addRandomAlbum()
     QStringList genres;
     QStringList artists;
     QModelIndexList selected=view->selectedIndexes(false); // Dont need sorted selection here...
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         SqlLibraryModel::Item *item=static_cast<SqlLibraryModel::Item *>(idx.internalPointer());
         switch (item->getType()) {
         case SqlLibraryModel::T_Genre:
@@ -163,13 +164,19 @@ void OnlineDbWidget::addRandomAlbum()
     QList<Song> songs=srv->getAlbumTracks(album.artist, album.id);
     if (!songs.isEmpty()) {
         QStringList files;
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             files.append(s.file);
         }
         emit add(files, /*replace ? MPDConnection::ReplaceAndplay : */MPDConnection::Append, 0, false);
     }
 }
 
+void OnlineDbWidget::modelReset()
+{
+    int count = srv->trackCount();
+    view->setMinSearchDebounce(count <= 12500 ? 1000u : count <= 18000 ? 1500u : 2000u);
+}
+
 void OnlineDbWidget::doSearch()
 {
     srv->search(view->searchText());
@@ -188,7 +195,7 @@ void OnlineDbWidget::controlActions()
     QModelIndexList selected=view->selectedIndexes(false); // Dont need sorted selection here...
     bool allowRandomAlbum=isVisible() && !selected.isEmpty();
     if (allowRandomAlbum) {
-        foreach (const QModelIndex &idx, selected) {
+        for (const QModelIndex &idx: selected) {
             if (SqlLibraryModel::T_Track==static_cast<SqlLibraryModel::Item *>(idx.internalPointer())->getType() ||
                 SqlLibraryModel::T_Album==static_cast<SqlLibraryModel::Item *>(idx.internalPointer())->getType()) {
                 allowRandomAlbum=false;
@@ -211,3 +218,5 @@ void OnlineDbWidget::updateToPlayQueue(const QModelIndex &idx, bool replace)
         emit add(files, replace ? MPDConnection::ReplaceAndplay : MPDConnection::Append, 0, false);
     }
 }
+
+#include "moc_onlinedbwidget.cpp"
