@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -94,9 +94,9 @@ public:
     {
     }
 
-    void parse(const QVariant &data)
+    void parse(const QVariant &data) override
     {
-        foreach (const QVariant &resultVariant, data.toMap()[QLatin1String("results")].toList()) {
+        for (const QVariant &resultVariant: data.toMap()[QLatin1String("results")].toList()) {
             QVariantMap result(resultVariant.toMap());
             if (result[QLatin1String("kind")].toString() != QLatin1String("podcast")) {
                 continue;
@@ -107,7 +107,7 @@ public:
                        result[QLatin1String("artworkUrl100")].toUrl(),
                        QString(),
                        result[QLatin1String("collectionViewUrl")].toString(),
-                       0);
+                       nullptr);
         }
     }
 };
@@ -124,17 +124,17 @@ public:
     {
     }
 
-    void parse(const QVariant &data)
+    void parse(const QVariant &data) override
     {
         QVariantList list=data.toList();
-        foreach (const QVariant &var, list) {
+        for (const QVariant &var: list) {
             QVariantMap map=var.toMap();
             addPodcast(map[QLatin1String("title")].toString(),
                        map[QLatin1String("url")].toUrl(),
                        map[QLatin1String("logo_url")].toUrl(),
                        map[QLatin1String("description")].toString(),
                        map[QLatin1String("website")].toString(),
-                       0);
+                       nullptr);
         }
     }
 };
@@ -142,8 +142,8 @@ public:
 PodcastPage::PodcastPage(QWidget *p, const QString &n)
     : QWidget(p)
     , pageName(n)
-    , job(0)
-    , imageJob(0)
+    , job(nullptr)
+    , imageJob(nullptr)
 {
     tree = new QTreeWidget(this);
     tree->setItemDelegate(new BasicItemDelegate(tree));
@@ -189,7 +189,7 @@ void PodcastPage::cancel()
     spinner->stop();
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
 }
 
@@ -198,7 +198,7 @@ void PodcastPage::cancelImage()
     imageSpinner->stop();
     if (imageJob) {
         imageJob->cancelAndDelete();
-        imageJob=0;
+        imageJob=nullptr;
     }
 }
 
@@ -292,7 +292,7 @@ void PodcastPage::imageJobFinished()
         imageCache.insert(imageJob->property(constOrigUrlProperty).toUrl(), new QImage(img), img.byteCount());
         updateText();
     }
-    imageJob=0;
+    imageJob=nullptr;
 }
 
 void PodcastPage::jobFinished()
@@ -308,8 +308,8 @@ void PodcastPage::jobFinished()
     if (spinner) {
         spinner->stop();
     }
-    parseResonse(j->ok() ? j->actualJob() : 0);
-    job=0;
+    parseResonse(j->ok() ? j->actualJob() : nullptr);
+    job=nullptr;
 }
 
 void PodcastPage::openLink(const QUrl &url)
@@ -453,11 +453,11 @@ void OpmlBrowsePage::parseResonse(QIODevice *dev)
     if (1==parsed.categories.count() && parsed.podcasts.isEmpty()) {
         parsed=parsed.categories.at(0);
     }
-    foreach (const OpmlParser::Category &cat, parsed.categories) {
-        addCategory(cat, 0);
+    for (const OpmlParser::Category &cat: parsed.categories) {
+        addCategory(cat, nullptr);
     }
-    foreach (const OpmlParser::Podcast &pod, parsed.podcasts) {
-        addPodcast(pod, 0);
+    for (const OpmlParser::Podcast &pod: parsed.podcasts) {
+        addPodcast(pod, nullptr);
     }
     if (!isLoadingFromCache && tree->topLevelItemCount()>0) {
         QString cacheFile=generateCacheFileName(url, true);
@@ -481,11 +481,11 @@ void OpmlBrowsePage::addCategory(const OpmlParser::Category &cat, QTreeWidgetIte
 
     catItem->setData(0, IsPodcastRole, false);
     catItem->setIcon(0, Icons::self()->folderIcon);
-    foreach (const OpmlParser::Podcast &pod, cat.podcasts) {
+    for (const OpmlParser::Podcast &pod: cat.podcasts) {
         addPodcast(pod, catItem);
     }
-    foreach (const OpmlParser::Category &cat, cat.categories) {
-        addCategory(cat, catItem);
+    for (const OpmlParser::Category &subCat: cat.categories) {
+        addCategory(subCat, catItem);
     }
 }
 
@@ -561,7 +561,7 @@ void PodcastUrlPage::parseResonse(QIODevice *dev)
         emit error(tr("Cantata only supports audio podcasts! The URL entered contains only video podcasts."));
         return;
     }
-    addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), 0);
+    addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), nullptr);
 }
 
 int PodcastSearchDialog::instanceCount()
@@ -610,7 +610,7 @@ PodcastSearchDialog::PodcastSearchDialog(PodcastService *s, QWidget *parent)
     pages << loadDirectories(Utils::dataDir(), false, loaded);
     pages << loadDirectories(CANTATA_SYS_CONFIG_DIR, true, loaded);
 
-    foreach (PodcastPage *p, pages) {
+    for (PodcastPage *p: pages) {
         connect(p, SIGNAL(rssSelected(QUrl)), SLOT(rssSelected(QUrl)));
         connect(p, SIGNAL(error(QString)), SLOT(showError(QString)));
     }
@@ -660,7 +660,7 @@ void PodcastSearchDialog::msgWidgetVisible(bool v)
 void PodcastSearchDialog::pageChanged()
 {
     PageWidgetItem *pwi=pageWidget->currentPage();
-    PodcastPage *page=pwi ? qobject_cast<PodcastPage *>(pwi->widget()) : 0;
+    PodcastPage *page=pwi ? qobject_cast<PodcastPage *>(pwi->widget()) : nullptr;
     rssSelected(page ? page->currentRss() : QUrl());
 }
 
@@ -721,3 +721,5 @@ void PodcastSearchDialog::slotButtonClicked(int button)
         break;
     }
 }
+
+#include "moc_podcastsearchdialog.cpp"
