@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -126,12 +126,12 @@ const QString constFilename=QLatin1String("FILENAME:");
 
 DynamicPlaylists::DynamicPlaylists()
     : RulesPlaylists("dice", "dynamic")
-    , localTimer(0)
+    , localTimer(nullptr)
     , usingRemote(false)
-    , remoteTimer(0)
+    , remoteTimer(nullptr)
     , remotePollingEnabled(false)
     , statusTime(0)
-    , currentJob(0)
+    , currentJob(nullptr)
     , currentCommand(Unknown)
 {
     connect(this, SIGNAL(clear()), MPDConnection::self(), SLOT(clear()));
@@ -378,6 +378,13 @@ bool DynamicPlaylists::controlApp(bool isStart)
         connect(localTimer, SIGNAL(timeout()), SLOT(checkHelper()));
     }
     bool rv=process.waitForFinished(1000);
+    if (isStart && rv) {
+        int exitCode = process.exitCode();
+        DBUG << exitCode;
+        if (0!=exitCode) {
+            emit error(tr("Could not start dynamic helper. Please check all <a href=\"https://github.com/CDrummond/cantata/wiki/Dynamic-Palylists-Helper\">dependencies</a> are installed."));
+        }
+    }
     localTimer->start(1000);
     return rv;
 }
@@ -393,9 +400,9 @@ void DynamicPlaylists::parseRemote(const QStringList &response)
     Entry e;
     Rule r;
 
-    foreach (const QString &part, response) {
+    for (const QString &part: response) {
         QStringList lines=part.split('\n', QString::SkipEmptyParts);
-        foreach (const QString &s, lines) {
+        for (const QString &s: lines) {
             QString str=s.trimmed();
             if (str.isEmpty() || str.startsWith('#')) {
                 continue;
@@ -429,7 +436,7 @@ void DynamicPlaylists::parseRemote(const QStringList &response)
                     e.maxDuration=vals.at(1).toUInt();
                 }
             } else {
-                foreach (const QString &k, keys) {
+                for (const QString &k: keys) {
                     if (str.startsWith(k+constKeyValSep)) {
                         r.insert(k, str.mid(k.length()+1));
                     }
@@ -709,3 +716,5 @@ void DynamicPlaylists::remoteDynamicSupported(bool s)
 //        #endif
     }
 }
+
+#include "moc_dynamicplaylists.cpp"
