@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -70,6 +70,11 @@ SmartPlaylistsPage::SmartPlaylistsPage(QWidget *p)
     view->addAction(editAction);
     view->addAction(removeAction);
     view->alwaysShowHeader();
+    view->setInfoText(tr("A 'smart' playlist contains a set of rules to select tracks from your music library to play. "
+                         "The playlist also controls the order in which tracks are added. "
+                         "Unlike 'dynamic' playlists, the play queue is not dynamically updated.")
+                      +QLatin1String("\n\n\n")+
+                      tr("Use the + icon (below) to create a new 'smart' playlist."));
 }
 
 SmartPlaylistsPage::~SmartPlaylistsPage()
@@ -124,12 +129,12 @@ void SmartPlaylistsPage::remove()
     }
 
     QStringList names;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         names.append(idx.data(Qt::DisplayRole).toString());
     }
 
-    foreach (const QString &name, names) {
-        DynamicPlaylists::self()->del(name);
+    for (const QString &name: names) {
+        SmartPlaylists::self()->del(name);
     }
 }
 
@@ -142,7 +147,7 @@ void SmartPlaylistsPage::headerClicked(int level)
 
 void SmartPlaylistsPage::enableWidgets(bool enable)
 {
-    foreach (QWidget *c, controls) {
+    for (QWidget *c: controls) {
         c->setEnabled(enable);
     }
 
@@ -179,10 +184,15 @@ void SmartPlaylistsPage::searchResponse(const QString &id, const QList<Song> &so
 
 void SmartPlaylistsPage::filterCommand()
 {
-    if (command.minDuration>0 || command.maxDuration>0) {
+    bool filterDuration = command.minDuration>0 || command.maxDuration>0;
+    bool filterAge = command.maxAge>0;
+
+    if (filterDuration || filterAge) {
+        uint maxAge=time(nullptr)-(command.maxAge*24*60*60);
         QSet<Song> toRemove;
         for (const auto &s: command.songs) {
-            if (command.minDuration>s.time || (command.maxDuration>0 && s.time>command.maxDuration)) {
+            if ((filterAge && s.lastModified<maxAge) ||
+                (filterDuration && ((command.minDuration>s.time || (command.maxDuration>0 && s.time>command.maxDuration))) ) ) {
                 toRemove.insert(s);
             } else {
                 command.toCheck.append(s.file);
@@ -337,13 +347,13 @@ void SmartPlaylistsPage::addSongsToPlayQueue()
         files.append(songs.takeFirst().file);
     }
     if (!files.isEmpty()) {
-        emit add(files, command.action, command.priorty, command.decreasePriority);
+        emit add(files, command.action, command.priority, command.decreasePriority);
         view->clearSelection();
     }
     command.clear();
 }
 
-void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action, quint8 priorty, bool decreasePriority)
+void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action, quint8 priority, bool decreasePriority)
 {
     if (!name.isEmpty()) {
         return;
@@ -363,7 +373,7 @@ void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action,
         return;
     }
 
-    command = Command(pl, action, priorty, decreasePriority, command.id+1);
+    command = Command(pl, action, priority, decreasePriority, command.id+1);
 
     QList<RulesPlaylists::Rule>::ConstIterator it = pl.rules.constBegin();
     QList<RulesPlaylists::Rule>::ConstIterator end = pl.rules.constEnd();
@@ -402,7 +412,7 @@ void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action,
                     if (mpdGenres.isEmpty()) {
                         mpdGenres = MpdLibraryModel::self()->getGenres();
                     }
-                    foreach (const QString &g, mpdGenres) {
+                    for (const QString &g: mpdGenres) {
                         if (g.startsWith(find)) {
                             genres.append(g);
                         }
@@ -431,17 +441,17 @@ void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action,
                 if (dates.isEmpty()) {
                     rules.append(match + baseRule);
                 } else {
-                    foreach(int d, dates) {
+                    for(int d: dates) {
                         rules.append(match + baseRule + " Date \"" + QByteArray::number(d) + "\"");
                     }
                 }
             } else {
-                foreach (const QString &genre, genres) {
+                for (const QString &genre: genres) {
                     QByteArray rule = match + baseRule + " Genre  " + MPDConnection::encodeName(genre);
                     if (dates.isEmpty()) {
                         rules.append(rule);
                     } else {
-                        foreach(int d, dates) {
+                        for (int d: dates) {
                             rules.append(rule + " Date \"" + QByteArray::number(d) + "\"");
                         }
                     }
@@ -470,3 +480,5 @@ void SmartPlaylistsPage::addSelectionToPlaylist(const QString &name, int action,
     }
     emit search(command.includeRules.takeFirst(), "I:"+QString::number(command.id));
 }
+
+#include "moc_smartplaylistspage.cpp"
