@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -152,7 +152,7 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
         return;
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (!CueFile::isCue(s.file)) {
            origSongs.append(s);
         }
@@ -190,7 +190,10 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
     state=State_Idle;
     enableButton(User1, origSongs.count());
     view->clear();
-    foreach (const Song &s, origSongs) {
+    for (const Song &s: origSongs) {
+        if (s.isLocalFile()) {
+            base = QString();
+        }
         new QTreeWidgetItem(view, QStringList() << s.albumArtist() << s.album << s.title);
     }
     Dialog::show();
@@ -303,8 +306,8 @@ void RgDialog::stopScanning()
 void RgDialog::createScanner(const QList<int> &indexes)
 {
     QMap<int, QString> fileMap;
-    foreach (int i, indexes) {
-        fileMap[i]=base+origSongs.at(i).filePath();
+    for (int i: indexes) {
+        fileMap[i]=origSongs.at(i).filePath(base);
     }
 
     AlbumScanner *s=new AlbumScanner(fileMap);
@@ -317,7 +320,7 @@ void RgDialog::createScanner(const QList<int> &indexes)
 void RgDialog::clearScanners()
 {
     QList<AlbumScanner *> scannerList=scanners.keys();
-    foreach (AlbumScanner *sc, scannerList) {
+    for (AlbumScanner *sc: scannerList) {
         sc->stop();
     }
     scanners.clear();
@@ -379,8 +382,8 @@ bool RgDialog::saveTags()
     QMap<int, Tags::ReplayGain>::ConstIterator end=tagsToSave.constEnd();
 
     for (; it!=end; ++it) {
-        QString filePath=origSongs.at(it.key()).filePath();
-        switch (Tags::updateReplaygain(base+filePath, it.value())) {
+        QString filePath=origSongs.at(it.key()).filePath(base);
+        switch (Tags::updateReplaygain(filePath, it.value())) {
         case Tags::Update_Failed:
             failed.append(filePath);
             break;
@@ -624,7 +627,7 @@ void RgDialog::removeItems()
     if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Remove the selected tracks from the list?"),
                                                    tr("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
         QList<QTreeWidgetItem *> selection=view->selectedItems();
-        foreach (QTreeWidgetItem *item, selection) {
+        for (QTreeWidgetItem *item: selection) {
             int index=view->indexOfTopLevelItem(item);
             view->setItemHidden(item, true);
             removedItems.insert(index);
@@ -646,3 +649,5 @@ void RgDialog::closeEvent(QCloseEvent *event)
         Dialog::closeEvent(event);
     }
 }
+
+#include "moc_rgdialog.cpp"
