@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * This file contains code from QMPDClient:
  * Copyright (C) 2005-2008 Håvard Tautra Knutsen <havtknut@tihlde.org>
@@ -28,7 +28,7 @@
 #include "scrobbler.h"
 #include "pausabletimer.h"
 #include "config.h"
-#include "gui/covers.h"
+#include "gui/apikeys.h"
 #include "network/networkaccessmanager.h"
 #include "mpd-interface/mpdconnection.h"
 #include "support/globalstatic.h"
@@ -113,10 +113,10 @@ static QString md5(const QString &s)
 static void sign(QMap<QString, QString> &params)
 {
     QString s;
-    params[QLatin1String("api_key")] = Covers::constLastFmApiKey;
+    params[QLatin1String("api_key")] = ApiKeys::self()->get(ApiKeys::LastFm);
     QStringList keys=params.keys();
     keys.sort();
-    foreach (const QString &k, keys) {
+    for (const QString &k: keys) {
         s += k+params[k];
     }
     s += constSecretKey;
@@ -162,7 +162,7 @@ Scrobbler::Track::Track(const Song &s)
 }
 
 Scrobbler::Scrobbler()
-    : QObject(0)
+    : QObject(nullptr)
     , scrobblingEnabled(false)
     , loveIsEnabled(false)
     , scrobbler("Last.fm")
@@ -176,8 +176,8 @@ Scrobbler::Scrobbler()
     , scrobbleViaMpd(false)
     , failedCount(0)
     , lastState(MPDState_Inactive)
-    , authJob(0)
-    , scrobbleJob(0)
+    , authJob(nullptr)
+    , scrobbleJob(nullptr)
 {
     hardFailTimer = new QTimer(this);
     hardFailTimer->setInterval(60*1000);
@@ -443,7 +443,7 @@ void Scrobbler::scrobbleNowPlaying()
     sign(params);
     DBUG << currentSong.title << currentSong.artist << currentSong.albumartist << currentSong.album << currentSong.track << currentSong.length;
     nowPlayingSent=true;
-    lastNowPlaying=time(NULL);
+    lastNowPlaying=time(nullptr);
     if (fakeScrobbling) {
         DBUG << "MSG" << params;
     } else {
@@ -533,7 +533,7 @@ void Scrobbler::scrobbleFinished()
     if (job==scrobbleJob) {
         QByteArray data=job->readAll();
         DBUG << job->errorString() << data << songQueue.size() << lastScrobbledSongs.size();
-        scrobbleJob=0;
+        scrobbleJob=nullptr;
 
         int errorCode=NoError;
         QXmlStreamReader reader(data);
@@ -646,7 +646,7 @@ void Scrobbler::authResp()
     if (job!=authJob) {
         return;
     }
-    authJob=0;
+    authJob=nullptr;
     sessionKey.clear();
 
     QByteArray data=job->readAll();
@@ -746,7 +746,7 @@ void Scrobbler::saveCache()
         writer.writeAttribute("version", "1");
 
         const QQueue<Track> &queue=lastScrobbledSongs.isEmpty() ? songQueue : lastScrobbledSongs;
-        foreach (const Track &t, queue) {
+        for (const Track &t: queue) {
             writer.writeEmptyElement("track");
             writer.writeAttribute(QLatin1String("artist"), t.artist);
             writer.writeAttribute(QLatin1String("album"), t.album);
@@ -778,7 +778,7 @@ void Scrobbler::mpdStateUpdated(bool songChanged)
             nowPlayingTimer->pause();
             break;
         case MPDState_Playing: {
-            time_t now=time(NULL);
+            time_t now=time(nullptr);
             currentSong.timestamp = now-MPDStatus::self()->timeElapsed();
             DBUG << "Timestamp:" << currentSong.timestamp << "scrobbledCurrent:" << scrobbledCurrent << "nowPlayingSent:" << nowPlayingSent
                  << "now:" << now << "lastNowPlaying:" << lastNowPlaying << "isRepeat:" << isRepeat;
@@ -841,14 +841,14 @@ void Scrobbler::cancelJobs()
         authJob->close();
         authJob->abort();
         authJob->deleteLater();
-        authJob=0;
+        authJob=nullptr;
     }
     if (scrobbleJob) {
         disconnect(scrobbleJob, SIGNAL(finished()), this, SLOT(scrobbleFinished()));
         authJob->close();
         authJob->abort();
         authJob->deleteLater();
-        scrobbleJob=0;
+        scrobbleJob=nullptr;
     }
 }
 
@@ -864,7 +864,7 @@ void Scrobbler::loadScrobblers()
 {
     if (scrobblers.isEmpty()) {
         QStringList dirs=QStringList() << Utils::dataDir() << CANTATA_SYS_CONFIG_DIR;
-        foreach (const QString &dir, dirs) {
+        for (const QString &dir: dirs) {
             if (dir.isEmpty()) {
                 continue;
             }
@@ -899,3 +899,5 @@ QString Scrobbler::scrobblerUrl()
     }
     return scrobblers[scrobbler];
 }
+
+#include "moc_scrobbler.cpp"
