@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,7 +31,7 @@ class ScrobblingLove : public ToolButton
     Q_OBJECT
 public:
     ScrobblingLove(QWidget *p);
-    virtual ~ScrobblingLove() { }
+    ~ScrobblingLove() override { }
 
 private Q_SLOTS:
     void sendLove();
