@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,7 +31,7 @@ class ScrobblingStatus : public QWidget
     Q_OBJECT
 public:
     ScrobblingStatus(QWidget *p);
-    virtual ~ScrobblingStatus() { }
+    ~ScrobblingStatus() override { }
 
     bool isChecked() const { return btn->isChecked(); }
 public Q_SLOTS:
