@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -47,7 +47,7 @@ BuddyLabel::BuddyLabel(QWidget *p, QWidget *b)
 bool BuddyLabel::event(QEvent *e)
 {
     if (QEvent::Shortcut==e->type()) {
-        mouseReleaseEvent(0);
+        mouseReleaseEvent(nullptr);
         e->accept();
         return true;
     } else {
