@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -40,8 +40,8 @@ Dialog::Dialog(QWidget *parent, const QString &name, const QSize &defSize)
     : QDialog(parent)
     , defButton(0)
     , buttonTypes(0)
-    , mw(0)
-    , buttonBox(0)
+    , mw(nullptr)
+    , buttonBox(nullptr)
     , shown(false)
 {
     if (!name.isEmpty()) {
@@ -142,7 +142,7 @@ QSet<QString> standardNames()
                                             << close().text << help().text << overwrite().text << reset().text << cont().text << del().text
                                             << stop().text << remove().text << back().text << forward().text;
 
-        foreach (QString s, strings) {
+        for (QString s: strings) {
             names.insert(s.remove("&"));
         }
     }
@@ -390,7 +390,7 @@ void Dialog::create()
 QAbstractButton *Dialog::getButton(ButtonCode button)
 {
     QDialogButtonBox::StandardButton mapped=mapType(button);
-    QAbstractButton *b=QDialogButtonBox::NoButton==mapped ? 0 : buttonBox->button(mapped);
+    QAbstractButton *b=QDialogButtonBox::NoButton==mapped ? nullptr : buttonBox->button(mapped);
     if (!b && userButtons.contains(button)) {
         b=userButtons[button];
     }
@@ -437,3 +437,5 @@ void Dialog::closeEvent(QCloseEvent *e)
     QDialog::closeEvent(e);
 }
 #endif
+
+#include "moc_dialog.cpp"
