@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  */
 
 /**************************************************************************
@@ -44,6 +44,7 @@
 #include <QStyleOption>
 #include <QHBoxLayout>
 #include <QMouseEvent>
+#include <QWheelEvent>
 #include <QPainter>
 #include <QSplitter>
 #include <QStackedWidget>
@@ -100,17 +101,6 @@ void FancyTabProxyStyle::drawPrimitive(PrimitiveElement element, const QStyleOpt
     }
 }
 
-int FancyTabProxyStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget, QStyleHintReturn *returnData) const
-{
-    if (SH_TabBar_Alignment==hint && widget && qobject_cast<const QTabBar *>(widget)) {
-        QTabBar::Shape shape=static_cast<const QTabBar *>(widget)->shape();
-        if (QTabBar::RoundedNorth==shape || QTabBar::RoundedSouth==shape) {
-            return (int)Qt::AlignCenter;
-        }
-    }
-    return QProxyStyle::styleHint(hint, option, widget, returnData);
-}
-
 void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const
 {
     const QStyleOptionTab *tabOpt = qstyleoption_cast<const QStyleOptionTab*>(option);
@@ -123,6 +113,7 @@ void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption
     const QRect rect = tabOpt->rect;
     const bool selected = tabOpt->state&State_Selected;
     const bool underMouse = tabOpt->state&State_MouseOver;
+    bool active = tabOpt->state&State_Active;
     const bool verticalTabs = tabOpt->shape == QTabBar::RoundedWest;
     const QString text = tabOpt->text;
 
@@ -154,29 +145,32 @@ void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption
         QColor col = option->palette.highlight().color();
         #endif
 
-        if (!selected) {
+        if (selected) {
+            #ifndef Q_OS_MAC
+            if (!active && option->palette.highlight().color()==QApplication::palette().color(QPalette::Active, QPalette::Highlight)) {
+                active = true;
+            }
+            #endif
+        } else {
             #if defined Q_OS_WIN
             col.setAlphaF(0.1);
             #else
             col.setAlphaF(0.2);
             #endif
         }
-        p->fillRect(rect, col);
+        p->fillRect(draw_rect, col);
     }
 
     int textFlags = Qt::AlignTop | Qt::AlignVCenter;
     #ifdef Q_OS_MAC
-    p->setPen(selected && option->state&State_Active
-              ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
+    p->setPen(selected && active ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
     #elif defined Q_OS_WIN
     p->setPen(QApplication::palette().foreground().color());
     #else
-    p->setPen(selected && option->state&State_Active
-              ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
+    p->setPen(selected && active ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
     #endif
 
-    drawIcon(tabOpt->icon, iconRect, p, tabOpt->iconSize,
-             selected && option->state&State_Active);
+    drawIcon(tabOpt->icon, iconRect, p, tabOpt->iconSize, selected && active);
 
     QString txt=text;
     txt.replace("&", "");
@@ -242,7 +236,7 @@ FancyTabBar::FancyTabBar(QWidget *parent, bool text, int iSize, Pos pos)
     setMouseTracking(true); // Needed for hover events
     triggerTimer.setSingleShot(true);
 
-    QBoxLayout* layout=0;
+    QBoxLayout* layout=nullptr;
 
     if (Side!=pos) {
         setMinimumHeight(tabSizeHint().height());
@@ -290,12 +284,12 @@ QSize FancyTabBar::tabSizeHint() const
     if (withText) {
         QFontMetrics fm(font());
         int maxTw=0;
-        foreach (FancyTab *tab, tabs) {
+        for (FancyTab *tab: tabs) {
             maxTw=qMax(maxTw, tab->sizeHint().width());
         }
         return QSize(qMax(icnSize + spacing, maxTw), icnSize + spacing + fm.height() + padding);
     } else {
-        return QSize(icnSize + spacing, icnSize + spacing + padding);
+        return QSize(icnSize + spacing + padding, icnSize + spacing + padding);
     }
 }
 
@@ -381,6 +375,24 @@ void FancyTabBar::mousePressEvent(QMouseEvent *e)
     }
 }
 
+void FancyTabBar::wheelEvent(QWheelEvent *ev)
+{
+    int numDegrees = ev->delta() / 8;
+    int numSteps = numDegrees / -15;
+    int prevIndex = currentIdx;
+
+    if (numSteps>0) {
+        currentIdx=qMin(currentIdx+numSteps, tabs.size()-1);
+    } else if (numSteps<0) {
+        currentIdx=qMax(currentIdx+numSteps, 0);
+    }
+    if (currentIdx!=prevIndex) {
+        update();
+        triggerTimer.start(0);
+    }
+    ev->accept();
+}
+
 void FancyTabBar::addTab(const QIcon &icon, const QString &label, const QString &tt)
 {
     FancyTab *tab = new FancyTab(this);
@@ -473,12 +485,12 @@ void FancyTabBar::setCurrentIndex(int index)
 FancyTabWidget::FancyTabWidget(QWidget *parent)
     : QWidget(parent)
     , styleSetting(0)
-    , tabBar(NULL)
+    , tabBar(nullptr)
     , stack_(new QStackedWidget(this))
     , sideWidget(new QWidget)
     , sideLayout(new QVBoxLayout)
     , topLayout(new QVBoxLayout)
-    , menu(0)
+    , menu(nullptr)
     , proxyStyle(new FancyTabProxyStyle)
 {
     sideLayout->setSpacing(0);
@@ -530,7 +542,7 @@ int FancyTabWidget::count() const
 int FancyTabWidget::visibleCount() const
 {
     int c=0;
-    foreach (const Item &i, items) {
+    for (const Item &i: items) {
         if (i.enabled) {
             c++;
         }
@@ -576,7 +588,7 @@ void FancyTabWidget::setStyle(int s)
     }
     // Remove previous tab bar
     delete tabBar;
-    tabBar = NULL;
+    tabBar = nullptr;
 
     //   use_background_ = false;
     // Create new tab bar
@@ -784,3 +796,5 @@ QStringList FancyTabWidget::hiddenPages() const
     }
     return pages;
 }
+
+#include "moc_fancytabwidget.cpp"
