@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,7 +29,7 @@
 class LineEdit : public QLineEdit
 {
 public:
-    LineEdit(QWidget *parent = 0) : QLineEdit(parent) { setClearButtonEnabled(true); }
+    LineEdit(QWidget *parent = nullptr) : QLineEdit(parent) { setClearButtonEnabled(true); }
     void setReadOnly(bool e);
 };
 
