@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -49,7 +49,7 @@ static void splitMessage(const QString &orig, QString &msg, QString &sub)
     static QStringList constSeps=QStringList() << QLatin1String("\n\n") << QLatin1String("<br/><br/>");
     msg=orig;
 
-    foreach (const QString &sep, constSeps) {
+    for (const QString &sep: constSeps) {
         QStringList parts=orig.split(sep);
         if (parts.count()>1) {
             msg=parts.takeAt(0);
@@ -119,7 +119,7 @@ class YesNoListDialog : public Dialog
 {
 public:
     YesNoListDialog(QWidget *p) : Dialog(p) { }
-    void slotButtonClicked(int b) {
+    void slotButtonClicked(int b) override {
         switch(b) {
         case Dialog::Ok:
         case Dialog::Yes:
