@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -33,20 +33,13 @@ class UrlLabel : public QLabel
 
 public:
     UrlLabel(QWidget *p);
-    virtual ~UrlLabel() { }
+    ~UrlLabel() override { }
 
     void setText(const QString &t);
     void setProperty(const char *name, const QVariant &value);
 
 Q_SIGNALS:
     void leftClickedUrl();
-
-protected:
-    void mousePressEvent(QMouseEvent *ev);
-    void mouseReleaseEvent(QMouseEvent *);
-
-private:
-    bool pressed;
 };
 
 #endif // URLLABEL_H
