@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -56,7 +56,7 @@ int TrackOrganiser::instanceCount()
 
 TrackOrganiser::TrackOrganiser(QWidget *parent)
     : SongDialog(parent, "TrackOrganiser",  QSize(800, 500))
-    , schemeDlg(0)
+    , schemeDlg(nullptr)
     , autoSkip(false)
     , paused(false)
     , updated(false)
@@ -71,7 +71,7 @@ TrackOrganiser::TrackOrganiser(QWidget *parent)
     setMainWidget(mainWidet);
     configFilename->setIcon(Icons::self()->configureIcon);
     setButtonGuiItem(Ok, GuiItem(tr("Rename")));
-    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(update()));
+    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(updateMaybe()));
     progress->setVisible(false);
     files->setItemDelegate(new BasicItemDelegate(files));
     files->setAlternatingRowColors(false);
@@ -94,7 +94,7 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
     // If we are called from the TagEditor dialog, then forceUpdate will be true. This is so that we dont do 2
     // MPD updates (one from TagEditor, and one from here!)
     alwaysUpdate=forceUpdate;
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (!CueFile::isCue(s.file)) {
            origSongs.append(s);
         }
@@ -203,10 +203,23 @@ void TrackOrganiser::updateView()
     files->clear();
     bool different=false;
     readOptions();
-    foreach (const Song &s, origSongs) {
-        QString modified=opts.createFilename(s);
+
+    QString musicFolder;
+    #ifdef ENABLE_DEVICES_SUPPORT
+    if (!deviceUdi.isEmpty()) {
+        Device *dev=getDevice();
+        if (!dev) {
+            return;
+        }
+        musicFolder=dev->path();
+    } else
+    #endif
+        musicFolder=MPDConnection::self()->getDetails().dir;
+
+    for (const Song &s: origSongs) {
+        QString modified=musicFolder + opts.createFilename(s);
         //different=different||(modified!=s.file);
-        QString orig=s.filePath();
+        QString orig=s.filePath(musicFolder);
         bool diff=modified!=orig;
         different|=diff;
         QTreeWidgetItem *item=new QTreeWidgetItem(files, QStringList() << orig << modified);
@@ -258,10 +271,9 @@ void TrackOrganiser::renameFile()
     #endif
         musicFolder=MPDConnection::self()->getDetails().dir;
 
-    QString orig=s.filePath();
-    if (modified!=orig) {
-        QString source=musicFolder+orig;
-        QString dest=musicFolder+modified;
+    QString source=s.filePath(musicFolder);
+    QString dest=musicFolder+modified;
+    if (source!=dest) {
         bool skip=false;
         if (!QFile::exists(source)) {
             if (autoSkip) {
@@ -359,7 +371,7 @@ void TrackOrganiser::renameFile()
             QDir sArtistDir(sDir); sArtistDir.cdUp();
             QDir dDir(Utils::getDir(dest));
             #ifdef ENABLE_DEVICES_SUPPORT
-            Device *dev=deviceUdi.isEmpty() ? 0 : getDevice();
+            Device *dev=deviceUdi.isEmpty() ? nullptr : getDevice();
             if (sDir.absolutePath()!=dDir.absolutePath()) {
                 Device::moveDir(sDir.absolutePath(), dDir.absolutePath(), musicFolder, dev ? dev->coverFile()
                                                                                            : QString(Covers::albumFileName(s)+QLatin1String(".jpg")));
@@ -375,12 +387,14 @@ void TrackOrganiser::renameFile()
             if (sArtistDir.exists() && dArtistDir.exists() && sArtistDir.absolutePath()!=sDir.absolutePath() && sArtistDir.absolutePath()!=dArtistDir.absolutePath()) {
                 QStringList artistImages;
                 QFileInfoList entries=sArtistDir.entryInfoList(QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot);
-                QString artistImage=Covers::artistFileName(s);
-                QSet<QString> acceptable=QSet<QString>() << artistImage+QLatin1String(".jpg") << artistImage+QLatin1String(".png")
+                QSet<QString> acceptable=QSet<QString>() << Covers::constArtistImage+QLatin1String(".jpg")
+                                                         << Covers::constArtistImage+QLatin1String(".png")
+                                                         << Covers::constComposerImage+QLatin1String(".jpg")
+                                                         << Covers::constComposerImage+QLatin1String(".png")
                                                          << ContextWidget::constBackdropFileName+QLatin1String(".jpg")
                                                          << ContextWidget::constBackdropFileName+QLatin1String(".png");
 
-                foreach (const QFileInfo &entry, entries) {
+                for (const QFileInfo &entry: entries) {
                     if (entry.isDir() || !acceptable.contains(entry.fileName())) {
                         artistImages.clear();
                         break;
@@ -390,7 +404,7 @@ void TrackOrganiser::renameFile()
                 }
                 if (!artistImages.isEmpty()) {
                     bool delDir=true;
-                    foreach (const QString &f, artistImages) {
+                    for (const QString &f: artistImages) {
                         if (!QFile::rename(sArtistDir.absolutePath()+Utils::constDirSep+f, dArtistDir.absolutePath()+Utils::constDirSep+f)) {
                             delDir=false;
                             break;
@@ -405,7 +419,7 @@ void TrackOrganiser::renameFile()
                     }
                 }
             }
-            item->setText(0, modified);
+            item->setText(0, dest);
             item->setFont(0, font());
             item->setFont(1, font());
             Song to=s;
@@ -413,6 +427,8 @@ void TrackOrganiser::renameFile()
             if (s.file.startsWith(Song::constMopidyLocal)) {
                 origPath=to.file;
                 to.file=Song::encodePath(to.file);
+            } else if (MPDConnection::self()->isForkedDaapd()) {
+                to.file=Song::constForkedDaapdLocal + dest;
             } else {
                 to.file=modified;
             }
@@ -458,7 +474,7 @@ void TrackOrganiser::removeItems()
                                                    tr("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
 
         QList<QTreeWidgetItem *> selection=files->selectedItems();
-        foreach (QTreeWidgetItem *item, selection) {
+        for (QTreeWidgetItem *item: selection) {
             int idx=files->indexOfTopLevelItem(item);
             if (idx>-1 && idx<origSongs.count()) {
                 origSongs.removeAt(idx);
@@ -532,18 +548,20 @@ Device * TrackOrganiser::getDevice(QWidget *p)
     if (!dev) {
         MessageBox::error(p ? p : this, tr("Device has been removed!"));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isConnected()) {
         MessageBox::error(p ? p : this, tr("Device is not connected."));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isIdle()) {
         MessageBox::error(p ? p : this, tr("Device is busy?"));
         reject();
-        return 0;
+        return nullptr;
     }
     return dev;
 }
 #endif
+
+#include "moc_trackorganiser.cpp"
