@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -81,7 +81,7 @@ ActionLabel::~ActionLabel()
     if (0==--theUsageCount) {
         for (int i=0; i<constNumIcons; ++i) {
             delete theIcons[i];
-            theIcons[i]=NULL;
+            theIcons[i]=nullptr;
         }
     }
 }
@@ -108,3 +108,5 @@ void ActionLabel::rotateIcon()
 
     setPixmap(*theIcons[count]);
 }
+
+#include "moc_actionlabel.cpp"
