@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -188,7 +188,7 @@ static QString streamText(const Song &song, const QString &trackTitle, bool useN
 GroupedViewDelegate::GroupedViewDelegate(GroupedView *p)
     : ActionItemDelegate(p)
     , view(p)
-    , ratingPainter(0)
+    , ratingPainter(nullptr)
 {
 }
 
@@ -478,7 +478,6 @@ GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
     setSelectionBehavior(SelectRows);
     setForceSingleColumn(true);
     connect(this, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
-    setStyleSheet("QTreeView::branch { border: 0px; }");
     GroupedViewDelegate *delegate=new GroupedViewDelegate(this);
     setItemDelegate(delegate);
     if (isPlayQueue) {
@@ -652,7 +651,7 @@ QModelIndexList GroupedView::selectedIndexes(bool sorted) const
     QSet<QModelIndex> indexSet;
     QModelIndexList sel;
 
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         if (!indexSet.contains(idx)) {
             indexSet.insert(idx);
             sel.append(idx);
@@ -816,17 +815,17 @@ void GroupedView::itemClicked(const QModelIndex &idx)
             if (list.count()) {
                 #if 0 // Commendted out as (as noted below) unselection is not working, and we always add to 'unsel' above (because of playlists)
                 if (unsel.isEmpty()) { // TODO: This is not working!!!   CHECK selModel if re-add!!!
-                    foreach(const QModelIndex &i, list) {
+                    for (const QModelIndex &i: list) {
                         selModel->select(i, QItemSelectionModel::Deselect|QItemSelectionModel::Rows);
                     }
                 } else {
-                    foreach(const QModelIndex &i, unsel) {
+                    for (const QModelIndex &i: unsel) {
                         selModel->select(i, QItemSelectionModel::Select|QItemSelectionModel::Rows);
                     }
                 }
                 #else
                 if (!unsel.isEmpty() && selModel) {
-                    foreach(const QModelIndex &i, unsel) {
+                    for (const QModelIndex &i: unsel) {
                         selModel->select(i, QItemSelectionModel::Select|QItemSelectionModel::Rows);
                     }
                 }
@@ -871,3 +870,10 @@ void GroupedView::collapse(const QModelIndex &idx, bool singleOnly)
         }
     }
 }
+
+void GroupedView::drawBranches(QPainter *, const QRect &, const QModelIndex &) const
+{
+    // Don't want any branch lines drawn!
+}
+
+#include "moc_groupedview.cpp"
