@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,9 +24,11 @@
 #include "itemview.h"
 #include "groupedview.h"
 #include "tableview.h"
+#include "categorizedview.h"
 #include "messageoverlay.h"
 #include "models/roles.h"
 #include "gui/covers.h"
+#include "gui/stdactions.h"
 #include "models/proxymodel.h"
 #include "actionitemdelegate.h"
 #include "basicitemdelegate.h"
@@ -47,7 +49,12 @@
 #include <QTimer>
 #include <QKeyEvent>
 #include <QProxyStyle>
+#include <QScrollBar>
+#include <QDebug>
 
+#define COVERS_DBUG if (Covers::verboseDebugEnabled()) qWarning() << metaObject()->className() << QThread::currentThread()->objectName() << __FUNCTION__
+
+#define RESPONSIVE_LAYOUT
 static int detailedViewDecorationSize=22;
 static int simpleViewDecorationSize=16;
 static int listCoverSize=22;
@@ -106,7 +113,7 @@ bool KeyEventHandler::eventFilter(QObject *obj, QEvent *event)
 }
 
 ViewEventHandler::ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v)
-    : KeyEventHandler(v, 0)
+    : KeyEventHandler(v, nullptr)
     , delegate(d)
 {
 }
@@ -136,24 +143,82 @@ static inline double subTextAlpha(bool selected)
     return selected ? 0.7 : 0.5;
 }
 
+static int zoomedSize(QListView *view, int size) {
+    return view
+            ? qobject_cast<ListView *>(view)
+              ? static_cast<ListView *>(view)->zoom()*size
+              : qobject_cast<CategorizedView *>(view)
+                ? static_cast<CategorizedView *>(view)->zoom()*size
+                : size
+            : size;
+}
+
+static QSize zoomedSize(QListView *view, QSize size) {
+    return view
+            ? qobject_cast<ListView *>(view)
+              ? static_cast<ListView *>(view)->zoom()*size
+              : qobject_cast<CategorizedView *>(view)
+                ? static_cast<CategorizedView *>(view)->zoom()*size
+                : size
+            : size;
+}
+
 class ListDelegate : public ActionItemDelegate
 {
 public:
-    ListDelegate(ListView *v, QAbstractItemView *p)
+    ListDelegate(QListView *v, QAbstractItemView *p)
         : ActionItemDelegate(p)
         , view(v)
+        #ifdef RESPONSIVE_LAYOUT
+        , viewGap(Utils::scaleForDpi(8))
+        #endif
+        , isCategorizedView(v && qobject_cast<CategorizedView *>(v))
     {
     }
 
-    virtual ~ListDelegate()
+    ~ListDelegate() override
     {
     }
 
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
+    #ifdef RESPONSIVE_LAYOUT
+    // Calculate width for each item in IconMode. The idea is to have the icons evenly spaced out.
+    int calcItemWidth() const
+    {
+        int viewWidth = view->viewport()->width();
+
+        // KVantum returns a -ve number for spacing if using overlay scrollbars. I /think/ this
+        // is what messes the layout code. The subtracion below seems to work-around this - but
+        // give a larger right-hand margin!
+        int sbarSpacing = view->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing);
+        if (sbarSpacing<0) {
+            viewWidth-=3*viewGap;
+        } else {
+            QScrollBar *sb=view->verticalScrollBar();
+            int sbarWidth=sb && sb->isVisible() ? 0 : view->style()->pixelMetric(QStyle::PM_ScrollBarExtent);
+            viewWidth-=sbarSpacing+sbarWidth;
+        }
+
+        int standardWidth = zoomedSize(view, gridCoverSize)+viewGap;
+        int iconWidth = standardWidth;
+        int viewCount = view->model() ? view->model()->rowCount(view->rootIndex()) : -1;
+        int numItems = viewWidth/iconWidth;
+        if (numItems>1 && viewCount>1 && (viewCount+1)>numItems) {
+            iconWidth=qMax(iconWidth-1, (int)(viewWidth/numItems)-3);
+        }
+        return iconWidth;
+    }
+    #endif
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         Q_UNUSED(option)
         if (view && QListView::IconMode==view->viewMode()) {
-            return QSize(gridCoverSize+8, gridCoverSize+(QApplication::fontMetrics().height()*2.5));
+            double textSpace = !isCategorizedView || view->model()->data(QModelIndex(), Cantata::Role_CatergizedHasSubText).toBool() ? 2.5 : 1.75;
+            #ifdef RESPONSIVE_LAYOUT
+            if (!isCategorizedView) {
+                return QSize(calcItemWidth(), zoomedSize(view, gridCoverSize)+(QApplication::fontMetrics().height()*textSpace));
+            }
+            #endif
+            return QSize(zoomedSize(view, gridCoverSize)+8, zoomedSize(view, gridCoverSize)+(QApplication::fontMetrics().height()*textSpace));
         } else {
             int imageSize = index.data(Cantata::Role_ListImage).toBool() ? listCoverSize : 0;
             // TODO: Any point to checking one-line here? All models return sub-text...
@@ -170,7 +235,7 @@ public:
         }
     }
 
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         if (!index.isValid()) {
             return;
@@ -181,10 +246,12 @@ public:
         bool active=option.state&QStyle::State_Active;
         bool drawBgnd=true;
         bool iconMode = view && QListView::IconMode==view->viewMode();
+        bool iconSubText = iconMode && (!isCategorizedView || view->model()->data(QModelIndex(), Cantata::Role_CatergizedHasSubText).toBool());
+
         QStyleOptionViewItem opt(option);
         opt.showDecorationSelected=true;
 
-        if (!underMouse) {
+        if (!isCategorizedView && !underMouse) {
             if (mouseOver && !selected) {
                 drawBgnd=false;
             }
@@ -219,26 +286,28 @@ public:
 
         QPixmap pix;
         if (showCapacity) {
-            const_cast<QAbstractItemModel *>(index.model())->setData(index, iconMode ? gridCoverSize : listCoverSize, Cantata::Role_Image);
+            const_cast<QAbstractItemModel *>(index.model())->setData(index, iconMode ? zoomedSize(view, gridCoverSize) : listCoverSize, Cantata::Role_Image);
             pix=index.data(Cantata::Role_Image).value<QPixmap>();
         }
         if (pix.isNull() && (iconMode || index.data(Cantata::Role_ListImage).toBool())) {
             Song cSong=index.data(iconMode ? Cantata::Role_GridCoverSong : Cantata::Role_CoverSong).value<Song>();
+            COVERS_DBUG << "Cover song" << cSong.albumArtist() << cSong.album << cSong.file << index.data().toString() << iconMode;
             if (!cSong.isEmpty()) {
-                QPixmap *cp=Covers::self()->get(cSong, iconMode ? gridCoverSize : listCoverSize, getCoverInUiThread(index));
+                QPixmap *cp=Covers::self()->get(cSong, iconMode ? zoomedSize(view, gridCoverSize) : listCoverSize, getCoverInUiThread(index));
                 if (cp) {
                     pix=*cp;
                 }
             }
         }
         if (pix.isNull()) {
-            int size=iconMode ? gridCoverSize : detailedViewDecorationSize;
+            COVERS_DBUG << "No cover image, use decoration";
+            int size=iconMode ? zoomedSize(view, gridCoverSize) : detailedViewDecorationSize;
             pix=index.data(Qt::DecorationRole).value<QIcon>().pixmap(size, size, selected &&
                                                                      textColor==qApp->palette().color(QPalette::HighlightedText)
                                                                         ? QIcon::Selected : QIcon::Normal);
         }
 
-        bool oneLine = childText.isEmpty();
+        bool oneLine = (iconMode && !iconSubText) || childText.isEmpty();
         ActionPos actionPos = iconMode ? AP_VTop : AP_HMiddle;
         bool rtl = QApplication::isRightToLeft();
 
@@ -247,7 +316,7 @@ public:
         }
 
         painter->save();
-        painter->setClipRect(r);
+        //painter->setClipRect(r);
 
         QFont textFont(index.data(Qt::FontRole).value<QFont>());
         QFontMetrics textMetrics(textFont);
@@ -368,11 +437,17 @@ public:
             opt.direction=QApplication::layoutDirection();
             opt.fontMetrics=textMetrics;
 
-            QApplication::style()->drawControl(QStyle::CE_ProgressBar, &opt, painter, 0L);
+            QApplication::style()->drawControl(QStyle::CE_ProgressBar, &opt, painter, nullptr);
         }
 
         if (drawBgnd && mouseOver) {
-            drawIcons(painter, AP_VTop==actionPos ? option.rect : r, mouseOver, rtl, actionPos, index);
+            QRect rect(AP_VTop==actionPos ? option.rect : r);
+            #ifdef RESPONSIVE_LAYOUT
+            if (!isCategorizedView && AP_VTop==actionPos) {
+                rect.adjust(0, 0, actionPosAdjust(), 0);
+            }
+            #endif
+            drawIcons(painter, rect, mouseOver, rtl, actionPos, index);
         }
         if (!iconMode) {
             BasicItemDelegate::drawLine(painter, option.rect, textColor);
@@ -382,26 +457,35 @@ public:
 
     virtual bool getCoverInUiThread(const QModelIndex &) const { return false; }
     virtual QWidget * itemView() const { return view; }
+    #ifdef RESPONSIVE_LAYOUT
+    int actionPosAdjust() const {
+        return !isCategorizedView && view && QListView::IconMode==view->viewMode() ? -(((calcItemWidth()-(zoomedSize(view, gridCoverSize)+viewGap))/2.0)+0.5) : 0;
+    }
+    #endif
 
 protected:
-    ListView *view;
+    QListView *view;
+    #ifdef RESPONSIVE_LAYOUT
+    int viewGap;
+    #endif
+    bool isCategorizedView;
 };
 
 class TreeDelegate : public ListDelegate
 {
 public:
     TreeDelegate(QAbstractItemView *p)
-        : ListDelegate(0, p)
+        : ListDelegate(nullptr, p)
         , simpleStyle(true)
         , treeView(p)
     {
     }
 
-    virtual ~TreeDelegate()
+    ~TreeDelegate() override
     {
     }
 
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         if (noIcons) {
             return QStyledItemDelegate::sizeHint(option, index);
@@ -420,7 +504,7 @@ public:
         return sz;
     }
 
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         if (!index.isValid()) {
             return;
@@ -525,13 +609,13 @@ public:
     void setSimple(bool s) { simpleStyle=s; }
     void setNoIcons(bool n) { noIcons=n; }
 
-    virtual bool getCoverInUiThread(const QModelIndex &idx) const
+    bool getCoverInUiThread(const QModelIndex &idx) const override
     {
         // Want album covers in artists view to load quickly...
         return idx.isValid() && idx.data(Cantata::Role_LoadCoverInUIThread).toBool();
     }
 
-    virtual QWidget * itemView() const { return treeView; }
+    QWidget * itemView() const override { return treeView; }
 
     bool simpleStyle;
     bool noIcons;
@@ -568,32 +652,40 @@ QString ItemView::modeStr(Mode m)
     case Mode_IconTop:      return QLatin1String("icontop");
     case Mode_GroupedTree:  return QLatin1String("grouped");
     case Mode_Table:        return QLatin1String("table");
+    case Mode_Categorized:  return QLatin1String("categorized");
     }
 }
 
 static const char *constPageProp="page";
 static const char *constAlwaysShowProp="always";
-static Action *backAction=0;
+static Action *backAction=nullptr;
 
+static const QLatin1String constZoomKey("gridZoom");
 const QLatin1String ItemView::constSearchActiveKey("searchActive");
 const QLatin1String ItemView::constViewModeKey("viewMode");
 const QLatin1String ItemView::constStartClosedKey("startClosed");
 const QLatin1String ItemView::constSearchCategoryKey("searchCategory");
 
+static const double constMinZoom = 1.0;
+static const double constMaxZoom = 4.0;
+static const double constZoomStep = 0.25;
+
 ItemView::ItemView(QWidget *p)
     : QWidget(p)
-    , searchTimer(0)
-    , itemModel(0)
+    , searchTimer(nullptr)
+    , itemModel(nullptr)
     , currentLevel(0)
     , mode(Mode_SimpleTree)
-    , groupedView(0)
-    , tableView(0)
-    , spinner(0)
-    , msgOverlay(0)
+    , groupedView(nullptr)
+    , tableView(nullptr)
+    , categorizedView(nullptr)
+    , spinner(nullptr)
+    , msgOverlay(nullptr)
     , performedSearch(false)
     , searchResetLevel(0)
     , openFirstLevelAfterSearch(false)
     , initialised(false)
+    , minSearchDebounce(250)
 {
     setupUi(this);
     if (!backAction) {
@@ -620,7 +712,7 @@ ItemView::ItemView(QWidget *p)
     ViewEventHandler *treeViewEventHandler=new ViewEventHandler(td, treeView);
     listView->installFilter(listViewEventHandler);
     treeView->installFilter(treeViewEventHandler);
-    connect(searchWidget, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
+    connect(searchWidget, SIGNAL(returnPressed()), this, SLOT(doSearch()));
     connect(searchWidget, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
     connect(searchWidget, SIGNAL(active(bool)), this, SLOT(searchActive(bool)));
     connect(treeView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
@@ -641,6 +733,11 @@ ItemView::ItemView(QWidget *p)
     treeView->setAttribute(Qt::WA_MacShowFocusRect, 0);
     listView->setAttribute(Qt::WA_MacShowFocusRect, 0);
     #endif
+
+    QWidget::addAction(StdActions::self()->zoomInAction);
+    QWidget::addAction(StdActions::self()->zoomOutAction);
+    connect(StdActions::self()->zoomInAction, SIGNAL(triggered()), SLOT(zoomIn()));
+    connect(StdActions::self()->zoomOutAction, SIGNAL(triggered()), SLOT(zoomOut()));
 }
 
 ItemView::~ItemView()
@@ -663,12 +760,17 @@ void ItemView::load(Configuration &config)
     setMode(toMode(config.get(constViewModeKey, modeStr(mode))));
     setStartClosed(config.get(constStartClosedKey, isStartClosed()));
     setSearchCategory(config.get(constSearchCategoryKey, searchCategory()));
+    int zoom=config.get(constZoomKey, 100);
+    if (zoom>100) {
+        listView->setZoom(zoom/100.0);
+    }
 }
 
 void ItemView::save(Configuration &config)
 {
     config.set(constSearchActiveKey, searchWidget->isActive());
     config.set(constViewModeKey, modeStr(mode));
+    config.set(constZoomKey, (int)(listView->zoom()*100));
     if (groupedView) {
         config.set(constStartClosedKey, isStartClosed());
     }
@@ -712,7 +814,7 @@ void ItemView::allowTableView(TableView *v)
         tableView->setParent(stackedWidget);
         stackedWidget->addWidget(tableView);
         tableView->setProperty(constPageProp, stackedWidget->count()-1);
-        ViewEventHandler *viewHandler=new ViewEventHandler(0, tableView);
+        ViewEventHandler *viewHandler=new ViewEventHandler(nullptr, tableView);
         tableView->installFilter(viewHandler);
         connect(tableView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
         connect(tableView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
@@ -725,6 +827,28 @@ void ItemView::allowTableView(TableView *v)
     }
 }
 
+void ItemView::allowCategorized()
+{
+    if (!categorizedView) {
+        categorizedView=new CategorizedView(stackedWidget);
+        categorizedView->setParent(stackedWidget);
+        stackedWidget->addWidget(categorizedView);
+        categorizedView->setProperty(constPageProp, stackedWidget->count()-1);
+        //KCategorizedView seems to handle mouse-over events better
+        //ViewEventHandler *viewHandler=new ViewEventHandler(nullptr, categorizedView);
+        //categorizedView->installFilter(viewHandler);
+        connect(categorizedView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
+        connect(categorizedView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(activateItem(const QModelIndex &)));
+        connect(categorizedView, SIGNAL(itemDoubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
+        connect(categorizedView, SIGNAL(itemClicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
+        categorizedView->setProperty(ProxyStyle::constModifyFrameProp, ProxyStyle::VF_Side|ProxyStyle::VF_Top);
+        #ifdef Q_OS_MAC
+        categorizedView->setAttribute(Qt::WA_MacShowFocusRect, 0);
+        #endif
+        categorizedView->setItemDelegate(new ListDelegate(categorizedView, categorizedView));
+    }
+}
+
 void ItemView::addAction(QAction *act)
 {
     treeView->addAction(act);
@@ -735,6 +859,9 @@ void ItemView::addAction(QAction *act)
     if (tableView) {
         tableView->addAction(act);
     }
+    if (categorizedView) {
+        categorizedView->addAction(act);
+    }
 }
 
 void ItemView::addSeparator()
@@ -747,7 +874,7 @@ void ItemView::addSeparator()
 void ItemView::setMode(Mode m)
 {
     initialised=true;
-    if (m<0 || m>=Mode_Count || (Mode_GroupedTree==m && !groupedView) || (Mode_Table==m && !tableView)) {
+    if (m<0 || m>=Mode_Count || (Mode_GroupedTree==m && !groupedView) || (Mode_Table==m && !tableView) || (Mode_Categorized==m && !categorizedView)) {
         m=Mode_SimpleTree;
     }
 
@@ -766,16 +893,20 @@ void ItemView::setMode(Mode m)
         setBackgroundImage(QIcon());
     }
 
+    bool wasListView = usingListView();
     mode=m;
     int stackIndex=0;
     if (usingTreeView()) {
-        listView->setModel(0);
+        listView->setModel(nullptr);
         if (groupedView) {
-            groupedView->setModel(0);
+            groupedView->setModel(nullptr);
         }
         if (tableView) {
             tableView->saveHeader();
-            tableView->setModel(0);
+            tableView->setModel(nullptr);
+        }
+        if (categorizedView) {
+            categorizedView->setModel(nullptr);
         }
         treeView->setModel(itemModel);
         treeView->setHidden(false);
@@ -786,11 +917,14 @@ void ItemView::setMode(Mode m)
         }
         treeView->reset();
     } else if (Mode_GroupedTree==mode) {
-        treeView->setModel(0);
-        listView->setModel(0);
+        treeView->setModel(nullptr);
+        listView->setModel(nullptr);
         if (tableView) {
             tableView->saveHeader();
-            tableView->setModel(0);
+            tableView->setModel(nullptr);
+        }
+        if (categorizedView) {
+            categorizedView->setModel(nullptr);
         }
         groupedView->setHidden(false);
         treeView->setHidden(true);
@@ -801,10 +935,13 @@ void ItemView::setMode(Mode m)
         stackIndex=groupedView->property(constPageProp).toInt();
     } else if (Mode_Table==mode) {
         int w=view()->width();
-        treeView->setModel(0);
-        listView->setModel(0);
+        treeView->setModel(nullptr);
+        listView->setModel(nullptr);
         if (groupedView) {
-            groupedView->setModel(0);
+            groupedView->setModel(nullptr);
+        }
+        if (categorizedView) {
+            categorizedView->setModel(nullptr);
         }
         tableView->setHidden(false);
         treeView->setHidden(true);
@@ -815,15 +952,33 @@ void ItemView::setMode(Mode m)
         }
         tableView->resize(w, tableView->height());
         stackIndex=tableView->property(constPageProp).toInt();
+    } else if (Mode_Categorized==mode) {
+        //int w=view()->width();
+        treeView->setModel(nullptr);
+        listView->setModel(nullptr);
+        if (groupedView) {
+            groupedView->setModel(nullptr);
+        }
+        if (tableView) {
+            tableView->setModel(nullptr);
+        }
+        categorizedView->setHidden(false);
+        treeView->setHidden(true);
+        categorizedView->setModel(itemModel);
+        stackIndex=categorizedView->property(constPageProp).toInt();
+        categorizedView->setGridSize(zoomedSize(listView, iconGridSize));
     } else {
         stackIndex=1;
-        treeView->setModel(0);
+        treeView->setModel(nullptr);
         if (groupedView) {
-            groupedView->setModel(0);
+            groupedView->setModel(nullptr);
         }
         if (tableView) {
             tableView->saveHeader();
-            tableView->setModel(0);
+            tableView->setModel(nullptr);
+        }
+        if (categorizedView) {
+            categorizedView->setModel(nullptr);
         }
         listView->setModel(itemModel);
         goToTop();
@@ -851,6 +1006,9 @@ void ItemView::setMode(Mode m)
         setBackgroundImage(oldBgndIcon);
     }
     controlViewFrame();
+    if (wasListView && !usingListView()) {
+        goToTop();
+    }
 }
 
 QModelIndexList ItemView::selectedIndexes(bool sorted) const
@@ -861,6 +1019,8 @@ QModelIndexList ItemView::selectedIndexes(bool sorted) const
         return groupedView->selectedIndexes(sorted);
     } else if (Mode_Table==mode) {
         return tableView->selectedIndexes(sorted);
+    } else if (Mode_Categorized==mode) {
+        return categorizedView->selectedIndexes(sorted);
     }
     return listView->selectedIndexes(sorted);
 }
@@ -869,10 +1029,18 @@ void ItemView::goToTop()
 {
     setLevel(0);
     prevTopIndex.clear();
-    if (dynamic_cast<ProxyModel *>(itemModel)) {
-        static_cast<ProxyModel *>(itemModel)->setRootIndex(QModelIndex());
+    if (usingListView()) {
+        if (dynamic_cast<ProxyModel *>(itemModel)) {
+            static_cast<ProxyModel *>(itemModel)->setRootIndex(QModelIndex());
+        }
+        listView->setRootIndex(QModelIndex());
+    } else if (Mode_Categorized==mode) {
+        categorizedView->setRootIndex(QModelIndex());
+        categorizedView->setPlain(false);
+        // Setting grid size causes categorizedview to re-lyout items. If we don't do this
+        // then items are all messed up!
+        categorizedView->setGridSizeOwn(categorizedView->gridSize());
     }
-    listView->setRootIndex(QModelIndex());
     setTitle();
     emit rootIndexSet(QModelIndex());
 }
@@ -891,6 +1059,24 @@ void ItemView::setEnabled(bool en)
     if (listView) {
         listView->setEnabled(en);
     }
+    if (categorizedView) {
+        categorizedView->setEnabled(en);
+    }
+}
+
+void ItemView::setInfoText(const QString &info)
+{
+    listView->setInfoText(info);
+    treeView->setInfoText(info);
+    if (groupedView) {
+        groupedView->setInfoText(info);
+    }
+    if (tableView) {
+        tableView->setInfoText(info);
+    }
+    if (categorizedView) {
+        categorizedView->setInfoText(info);
+    }
 }
 
 void ItemView::setLevel(int l, bool haveChildren)
@@ -900,7 +1086,7 @@ void ItemView::setLevel(int l, bool haveChildren)
     if (Mode_IconTop==mode) {
         if (0==currentLevel || haveChildren) {
             if (QListView::IconMode!=listView->viewMode()) {
-                listView->setGridSize(iconGridSize);
+                listView->setGridSize(zoomedSize(listView, iconGridSize));
                 listView->setViewMode(QListView::IconMode);
                 listView->setResizeMode(QListView::Adjust);
 //                listView->setAlternatingRowColors(false);
@@ -938,6 +1124,8 @@ QAbstractItemView * ItemView::view() const
         return groupedView;
     } else if(Mode_Table==mode) {
         return tableView;
+    } else if(Mode_Categorized==mode) {
+        return categorizedView;
     } else {
         return listView;
     }
@@ -1004,6 +1192,9 @@ void ItemView::setAcceptDrops(bool v)
     if (tableView) {
         tableView->setAcceptDrops(v);
     }
+    if (categorizedView) {
+        categorizedView->setAcceptDrops(v);
+    }
 }
 
 void ItemView::setDragDropOverwriteMode(bool v)
@@ -1016,6 +1207,9 @@ void ItemView::setDragDropOverwriteMode(bool v)
     if (tableView) {
         tableView->setDragDropOverwriteMode(v);
     }
+    if (categorizedView) {
+        categorizedView->setDragDropOverwriteMode(v);
+    }
 }
 
 void ItemView::setDragDropMode(QAbstractItemView::DragDropMode v)
@@ -1028,14 +1222,8 @@ void ItemView::setDragDropMode(QAbstractItemView::DragDropMode v)
     if (tableView) {
         tableView->setDragDropMode(v);
     }
-}
-
-void ItemView::setGridSize(const QSize &sz)
-{
-    iconGridSize=sz;
-    if (Mode_IconTop==mode && 0==currentLevel) {
-        listView->setGridSize(listGridSize);
-        static_cast<ActionItemDelegate *>(listView->itemDelegate())->setLargeIcons(iconGridSize.width()>(ActionItemDelegate::constLargeActionIconSize*6));
+    if (categorizedView) {
+        categorizedView->setDragDropMode(v);
     }
 }
 
@@ -1070,6 +1258,13 @@ void ItemView::setDeleteAction(QAction *act)
             static_cast<KeyEventHandler *>(tableView->filter())->setDeleteAction(act);
         }
     }
+    if (categorizedView) {
+        if (!categorizedView->filter() || !qobject_cast<KeyEventHandler *>(categorizedView->filter())) {
+            categorizedView->installEventFilter(new KeyEventHandler(categorizedView, act));
+        } else {
+            static_cast<KeyEventHandler *>(categorizedView->filter())->setDeleteAction(act);
+        }
+    }
 }
 
 void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
@@ -1084,6 +1279,8 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
         if (scrollTo) {
             v->scrollTo(idx, QAbstractItemView::PositionAtTop);
         }
+    } else if (Mode_Categorized==mode) {
+        // TODO
     } else {
         if (idx.parent().isValid()) {
             QList<QModelIndex> indexes;
@@ -1098,7 +1295,7 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
             if (dynamic_cast<ProxyModel *>(itemModel)) {
                 static_cast<ProxyModel *>(itemModel)->setRootIndex(QModelIndex());
             }
-            foreach (const QModelIndex &i, indexes) {
+            for (const QModelIndex &i: indexes) {
                 activateItem(i, false);
             }
             if (p.isValid()) {
@@ -1108,6 +1305,8 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
                 listView->scrollTo(idx, QAbstractItemView::PositionAtTop);
             }
             setTitle();
+        } else if (idx.isValid() && scrollTo) {
+            listView->scrollTo(idx, QAbstractItemView::PositionAtTop);
         }
     }
 
@@ -1139,6 +1338,11 @@ bool ItemView::isSearchActive() const
     return searchWidget->isActive();
 }
 
+void ItemView::setSearchToolTip(const QString &str)
+{
+    searchWidget->setToolTip(str);
+}
+
 void ItemView::closeSearch()
 {
     if (searchWidget->isActive()) {
@@ -1212,6 +1416,8 @@ void ItemView::setBackgroundImage(const QIcon &icon)
         groupedView->setBackgroundImage(bgndIcon);
     } else if (Mode_Table==mode && tableView) {
         tableView->setBackgroundImage(bgndIcon);
+    } else if (Mode_Categorized==mode && categorizedView) {
+        categorizedView->setBackgroundImage(bgndIcon);
     } else if (Mode_List==mode || Mode_IconTop==mode) {
         listView->setBackgroundImage(bgndIcon);
     }
@@ -1311,26 +1517,43 @@ void ItemView::backActivated()
 
     emit headerClicked(currentLevel);
 
-    if (!usingListView() || 0==currentLevel) {
+    if (!(usingListView() || Mode_Categorized==mode) || 0==currentLevel) {
         return;
     }
     setLevel(currentLevel-1);
-    if (dynamic_cast<ProxyModel *>(itemModel)) {
-        static_cast<ProxyModel *>(itemModel)->setRootIndex(listView->rootIndex().parent());
+
+    if (usingListView()) {
+        if (dynamic_cast<ProxyModel *>(itemModel)) {
+            static_cast<ProxyModel *>(itemModel)->setRootIndex(listView->rootIndex().parent());
+        }
+        listView->setRootIndex(listView->rootIndex().parent());
+        emit rootIndexSet(listView->rootIndex().parent());
+    } else {
+        categorizedView->setRootIndex(categorizedView->rootIndex().parent());
+        categorizedView->setPlain(false);
+        // Setting grid size causes categorizedview to re-lyout items. If we don't do this
+        // then items are all messed up!
+        categorizedView->setGridSizeOwn(categorizedView->gridSize());
+        emit rootIndexSet(categorizedView->rootIndex().parent());
     }
-    listView->setRootIndex(listView->rootIndex().parent());
-    emit rootIndexSet(listView->rootIndex().parent());
     setTitle();
 
-    if (prevTopIndex.isEmpty()) return;
+    if (prevTopIndex.isEmpty()) {
+        return;
+    }
+
     QModelIndex prevTop = prevTopIndex.takeLast();
-    if (qobject_cast<QSortFilterProxyModel *>(listView->model())) {
-        QModelIndex idx=static_cast<QSortFilterProxyModel *>(listView->model())->mapFromSource(prevTop);
-        if (idx.isValid()) {
-            listView->scrollTo(idx, QAbstractItemView::PositionAtTop);
+    if (usingListView()) {
+        if (qobject_cast<QSortFilterProxyModel *>(listView->model())) {
+            QModelIndex idx=static_cast<QSortFilterProxyModel *>(listView->model())->mapFromSource(prevTop);
+            if (idx.isValid()) {
+                listView->scrollTo(idx, QAbstractItemView::PositionAtTop);
+            }
+        } else {
+            listView->scrollTo(prevTop, QAbstractItemView::PositionAtTop);
         }
     } else {
-        listView->scrollTo(prevTop, QAbstractItemView::PositionAtTop);
+        categorizedView->scrollTo(prevTop, QAbstractItemView::PositionAtTop);
     }
 }
 
@@ -1343,9 +1566,15 @@ void ItemView::setExpanded(const QModelIndex &idx, bool exp)
 
 QAction * ItemView::getAction(const QModelIndex &index)
 {
+    QModelIndex idx = Mode_Categorized==mode ? categorizedView->mapFromSource(index) : index;
     QAbstractItemDelegate *abs=view()->itemDelegate();
-    ActionItemDelegate *d=abs ? qobject_cast<ActionItemDelegate *>(abs) : 0;
-    return d ? d->getAction(index) : 0;
+    ActionItemDelegate *d=abs ? qobject_cast<ActionItemDelegate *>(abs) : nullptr;
+    #ifdef RESPONSIVE_LAYOUT
+    ListDelegate *l=abs ? dynamic_cast<ListDelegate *>(abs) : nullptr;
+    return d ? d->getAction(idx, l ? l->actionPosAdjust() : 0) : nullptr;
+    #else
+    return d ? d->getAction(idx) : nullptr;
+    #endif
 }
 
 void ItemView::itemClicked(const QModelIndex &index)
@@ -1384,7 +1613,37 @@ void ItemView::activateItem(const QModelIndex &index, bool emitRootSet)
         if (!index.parent().isValid()) {
             tableView->setExpanded(index, !tableView->TreeView::isExpanded(index));
         }
-    } else if (index.isValid() && (index.child(0, 0).isValid() || itemModel->canFetchMore(index)) && index!=listView->rootIndex()) {
+    } else if (Mode_Categorized==mode) {
+        if (index == categorizedView->rootIndex()) {
+            return;
+        }
+        if (itemModel->canFetchMore(index)) {
+            itemModel->fetchMore(index);
+        }
+        QModelIndex fistChild=index.child(0, 0);
+        if (!fistChild.isValid()) {
+            return;
+        }
+
+        QModelIndex curTop=categorizedView->indexAt(QPoint(8, 24), true);
+        //if (qobject_cast<QSortFilterProxyModel *>(categorizedView->model())) {
+        //    curTop=static_cast<QSortFilterProxyModel *>(categorizedView->model())->mapToSource(curTop);
+        //}
+        prevTopIndex.append(curTop);
+        bool haveChildren=itemModel->canFetchMore(fistChild);
+        setLevel(currentLevel+1, haveChildren || fistChild.child(0, 0).isValid());
+        categorizedView->setPlain(!haveChildren);
+        categorizedView->setRootIndex(index);
+        setTitle();
+
+//        if (dynamic_cast<ProxyModel *>(itemModel)) {
+//            static_cast<ProxyModel *>(itemModel)->setRootIndex(index);
+//        }
+        if (emitRootSet) {
+            emit rootIndexSet(index);
+        }
+        categorizedView->scrollToTop();
+    } else if (usingListView() && (index.isValid() && (index.child(0, 0).isValid() || itemModel->canFetchMore(index)) && index!=listView->rootIndex())) {
         if (itemModel->canFetchMore(index)) {
             itemModel->fetchMore(index);
         }
@@ -1434,6 +1693,8 @@ void ItemView::addTitleButtonClicked()
 {
     if ((Mode_List==mode || Mode_IconTop==mode) && view()->rootIndex().isValid()) {
         emit updateToPlayQueue(view()->rootIndex(), false);
+    } else if (Mode_Categorized==mode && categorizedView->rootIndex().isValid()) {
+        emit updateToPlayQueue(categorizedView->rootIndex(), false);
     }
 }
 
@@ -1441,18 +1702,52 @@ void ItemView::replaceTitleButtonClicked()
 {
     if ((Mode_List==mode || Mode_IconTop==mode) && view()->rootIndex().isValid()) {
         emit updateToPlayQueue(view()->rootIndex(), true);
+    } else if (Mode_Categorized==mode && categorizedView->rootIndex().isValid()) {
+        emit updateToPlayQueue(categorizedView->rootIndex(), true);
     }
 }
 
 void ItemView::coverLoaded(const Song &song, int size)
 {
     Q_UNUSED(song)
-    if (Mode_BasicTree==mode || Mode_GroupedTree==mode || !isVisible() || (Mode_IconTop==mode && size!=gridCoverSize) || (Mode_IconTop!=mode && size!=listCoverSize)) {
+
+    if (Mode_BasicTree==mode || Mode_GroupedTree==mode || !isVisible() || (Mode_IconTop==mode && size!=zoomedSize(listView, gridCoverSize)) ||
+        (Mode_IconTop!=mode && Mode_Categorized!=mode && size!=listCoverSize) || (Mode_Categorized==mode && size!=zoomedSize(categorizedView, gridCoverSize))) {
         return;
     }
     view()->viewport()->update();
 }
 
+void ItemView::zoomIn()
+{
+    if (listView->isVisible() && Mode_IconTop==mode) {
+        if (listView->zoom()+constZoomStep<=constMaxZoom) {
+            listView->setZoom(listView->zoom()+constZoomStep);
+            listView->setGridSize(zoomedSize(listView, iconGridSize));
+        }
+    } else if (categorizedView && categorizedView->isVisible()) {
+        if (categorizedView->zoom()+constZoomStep<=constMaxZoom) {
+            categorizedView->setZoom(categorizedView->zoom()+constZoomStep);
+            categorizedView->setGridSize(zoomedSize(categorizedView, iconGridSize));
+        }
+    }
+}
+
+void ItemView::zoomOut()
+{
+    if (listView->isVisible() && Mode_IconTop==mode) {
+        if (listView->zoom()-constZoomStep>=constMinZoom) {
+            listView->setZoom(listView->zoom()-constZoomStep);
+            listView->setGridSize(zoomedSize(listView, iconGridSize));
+        }
+    } else if (categorizedView && categorizedView->isVisible()) {
+        if (categorizedView->zoom()-constZoomStep>=constMinZoom) {
+            categorizedView->setZoom(categorizedView->zoom()-constZoomStep);
+            categorizedView->setGridSize(zoomedSize(categorizedView, iconGridSize));
+        }
+    }
+}
+
 void ItemView::delaySearchItems()
 {
     if (searchWidget->text().isEmpty()) {
@@ -1470,12 +1765,15 @@ void ItemView::delaySearchItems()
             connect(searchTimer, SIGNAL(timeout()), this, SLOT(doSearch()));
         }
         int len=searchWidget->text().trimmed().length();
-        searchTimer->start(len<2 ? 1000 : len<4 ? 750 : 500);
+        searchTimer->start(qMin(qMax(minSearchDebounce, len<2 ? 1000u : len<4 ? 750u : 500u), 5000u));
     }
 }
 
 void ItemView::doSearch()
 {
+    if (searchTimer) {
+        searchTimer->stop();
+    }
     performedSearch=true;
     emit searchItems();
 }
@@ -1499,10 +1797,14 @@ void ItemView::setTitle()
     if (!model) {
         return;
     }
+    QString sub = model->data(index, Cantata::Role_TitleSubText).toString();
+    if (sub.isEmpty()) {
+        sub = model->data(index, Cantata::Role_SubText).toString();
+    }
     title->update(model->data(index, Mode_IconTop==mode ? Cantata::Role_GridCoverSong : Cantata::Role_CoverSong).value<Song>(),
                   model->data(index, Qt::DecorationRole).value<QIcon>(),
                   model->data(index, Cantata::Role_TitleText).toString(),
-                  model->data(index, Cantata::Role_SubText).toString(),
+                  sub,
                   model->data(index, Cantata::Role_TitleActions).toBool());
 }
 
@@ -1513,3 +1815,5 @@ void ItemView::controlViewFrame()
                                 ? ProxyStyle::VF_Side
                                 : (searchWidget->isActive() ? ProxyStyle::VF_Side : (ProxyStyle::VF_Side|ProxyStyle::VF_Top)));
 }
+
+#include "moc_itemview.cpp"
