@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -45,7 +45,7 @@ void MenuButton::controlState()
     if (!menu()) {
         return;
     }
-    foreach (QAction *a, menu()->actions()) {
+    for (QAction *a: menu()->actions()) {
         if (a->isEnabled() && a->isVisible() && !a->isSeparator()) {
             setEnabled(true);
             return;
@@ -111,3 +111,5 @@ bool MenuButton::eventFilter(QObject *o, QEvent *e)
 
     return ToolButton::eventFilter(o, e);
 }
+
+#include "moc_menubutton.cpp"
