@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,6 +26,8 @@
 #include "volumeslider.h"
 #include "mpd-interface/song.h"
 #include "gui/settings.h"
+#include "support/actioncollection.h"
+#include "support/action.h"
 #include "mpd-interface/mpdconnection.h"
 #include "models/playqueuemodel.h"
 #include "support/utils.h"
@@ -42,8 +44,10 @@
 #include <QToolTip>
 #include <QSpacerItem>
 #include <QToolButton>
+#include <QClipboard>
 
 static const int constPollMpd = 2; // Poll every 2 seconds when playing
+static const char * constUserSettingProp = "user-setting";
 
 class PosSliderProxyStyle : public QProxyStyle
 {
@@ -54,7 +58,7 @@ public:
         setBaseStyle(qApp->style());
     }
 
-    int styleHint(StyleHint stylehint, const QStyleOption *opt, const QWidget *widget, QStyleHintReturn *returnData) const
+    int styleHint(StyleHint stylehint, const QStyleOption *opt, const QWidget *widget, QStyleHintReturn *returnData) const override
     {
         if (QStyle::SH_Slider_AbsoluteSetButtons==stylehint) {
             return Qt::LeftButton|QProxyStyle::styleHint(stylehint, opt, widget, returnData);
@@ -107,7 +111,7 @@ public:
         Settings::self()->saveShowTimeRemaining(showRemaining);
     }
 
-    bool event(QEvent *e)
+    bool event(QEvent *e) override
     {
         switch (e->type()) {
         case QEvent::MouseButtonPress:
@@ -250,7 +254,7 @@ void PosSlider::setRange(int min, int max)
 
 NowPlayingWidget::NowPlayingWidget(QWidget *p)
     : QWidget(p)
-    , timer(0)
+    , timer(nullptr)
     , lastVal(0)
     , pollCount(0)
 {
@@ -259,16 +263,18 @@ NowPlayingWidget::NowPlayingWidget(QWidget *p)
     slider=new PosSlider(this);
     time=new TimeLabel(this, slider);
     ratingWidget=new RatingWidget(this);
+    infoLabel=new QLabel(this);
     QFont f=track->font();
     QFont small=Utils::smallFont(f);
     f.setBold(true);
     track->setFont(f);
     artist->setFont(small);
     time->setFont(small);
+    infoLabel->setFont(small);
     slider->setOrientation(Qt::Horizontal);
     QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, this);
-    QBoxLayout *topLayout=new QBoxLayout(QBoxLayout::LeftToRight, 0);
-    QBoxLayout *botLayout=new QBoxLayout(QBoxLayout::LeftToRight, 0);
+    QBoxLayout *topLayout=new QBoxLayout(QBoxLayout::LeftToRight, nullptr);
+    QBoxLayout *botLayout=new QBoxLayout(QBoxLayout::LeftToRight, nullptr);
     int space=Utils::layoutSpacing(this);
     int pad=qMax(space, Utils::scaleForDpi(8));
     #ifdef Q_OS_MAC
@@ -283,6 +289,7 @@ NowPlayingWidget::NowPlayingWidget(QWidget *p)
     botLayout->setSpacing(space/2);
     topLayout->addWidget(track);
     topLayout->addWidget(ratingWidget);
+    topLayout->addWidget(infoLabel);
     layout->addLayout(topLayout);
     botLayout->addWidget(artist);
     botLayout->addWidget(time);
@@ -300,39 +307,27 @@ NowPlayingWidget::NowPlayingWidget(QWidget *p)
     connect(ratingWidget, SIGNAL(valueChanged(int)), SLOT(setRating(int)));
     connect(this, SIGNAL(setRating(QString,quint8)), MPDConnection::self(), SLOT(setRating(QString,quint8)));
     connect(PlayQueueModel::self(), SIGNAL(currentSongRating(QString,quint8)), this, SLOT(rating(QString,quint8)));
+    connect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(updateInfo()));
+
+    Action *copy=ActionCollection::get()->createAction("copy-current-info", tr("Copy To Clipboard"));
+    copy->setSettingsText(tr("Now Playing")+QLatin1String(" / ")+Utils::strippedText(copy->text()));
+    artist->addAction(copy);
+    artist->setContextMenuPolicy(Qt::NoContextMenu);
+    track->addAction(copy);
+    track->setContextMenuPolicy(Qt::NoContextMenu);
+    connect(copy, SIGNAL(triggered()), SLOT(copyInfo()));
 }
 
 void NowPlayingWidget::update(const Song &song)
 {
-    QString name=song.name();
     currentSongFile=song.file;
     ratingWidget->setEnabled(!song.isEmpty() && Song::Standard==song.type);
     ratingWidget->setValue(0);
-    if (song.isEmpty()) {
-        track->setText(" ");
-        artist->setText(" ");
-    } else if (song.isStream() && !song.isCantataStream() && !song.isCdda() && !song.isDlnaStream()) {
-        track->setText(name.isEmpty() ? Song::unknown() : name);
-        if (song.artist.isEmpty() && song.title.isEmpty() && !name.isEmpty()) {
-            artist->setText(tr("(Stream)"));
-        } else {
-            artist->setText(song.artist.isEmpty() ? song.title : song.artistSong());
-        }
-    } else {
-        if (song.title.isEmpty() && song.artist.isEmpty() && (!name.isEmpty() || !song.file.isEmpty())) {
-            track->setText(name.isEmpty() ? song.file : name);
-        } else {
-            track->setText(song.title+(song.origYear>0 && song.origYear!=song.year ? QLatin1String(" (")+QString::number(song.origYear)+QLatin1Char(')') : QString()));
-        }
-        if (song.album.isEmpty() && song.artist.isEmpty()) {
-            artist->setText(track->fullText().isEmpty() ? QString() : Song::unknown());
-        } else if (song.album.isEmpty()) {
-            artist->setText(song.artist);
-        } else {
-            // Artist here is always artist, and not album artist or composer
-            artist->setText(song.artist+QString(" – ")+song.displayAlbum(false));
-        }
-    }
+    updateInfo();
+    track->setText(song.mainText());
+    artist->setText(song.subText());
+    track->setContextMenuPolicy(track->fullText().isEmpty() ? Qt::NoContextMenu : Qt::ActionsContextMenu);
+    artist->setContextMenuPolicy(artist->fullText().isEmpty() ? Qt::NoContextMenu : Qt::ActionsContextMenu);
 }
 
 void NowPlayingWidget::startTimer()
@@ -387,7 +382,9 @@ int NowPlayingWidget::value() const
 
 void NowPlayingWidget::readConfig()
 {
-    ratingWidget->setVisible(Settings::self()->showRatingWidget());
+    ratingWidget->setProperty(constUserSettingProp, Settings::self()->showRatingWidget());
+    infoLabel->setProperty(constUserSettingProp, Settings::self()->showTechnicalInfo());
+    controlWidgets();
 }
 
 void NowPlayingWidget::saveConfig()
@@ -411,7 +408,7 @@ void NowPlayingWidget::updateTimes()
 
 void NowPlayingWidget::updatePos()
 {
-    int elapsed=(startTime.elapsed()/1000.0)+0.5;
+    quint16 elapsed=(startTime.elapsed()/1000.0)+0.5;
     slider->setValue(lastVal+elapsed);
     MPDStatus::self()->setGuessedElapsed(lastVal+elapsed);
     if (++pollCount>=constPollMpd) {
@@ -440,6 +437,41 @@ void NowPlayingWidget::setRating(int v)
     emit setRating(currentSongFile, v);
 }
 
+void NowPlayingWidget::updateInfo()
+{
+    if (currentSongFile.isEmpty()) {
+        infoLabel->setText(QString());
+        return;
+    }
+
+    QString info;
+    info = MPDStatus::self()->bitrate()>0
+            ? MPDStatus::self()->samplerate()>0
+                ? info.sprintf("%d kbps, %.1f kHz", MPDStatus::self()->bitrate(), MPDStatus::self()->samplerate()/1000.0)
+                : info.sprintf("%d kbps", MPDStatus::self()->bitrate())
+            : MPDStatus::self()->samplerate()>0
+                  ? info.sprintf("%.1f kHz", MPDStatus::self()->samplerate()/1000.0)
+                  : QString();
+
+    int pos=currentSongFile.lastIndexOf('.');
+    if (pos>1) {
+        QString ext = currentSongFile.mid(pos+1).toUpper();
+
+        if (ext.length()<5) {
+            if (!info.isEmpty()) {
+                info+=", ";
+            }
+            info+=ext;
+        }
+    }
+    infoLabel->setText(QLatin1Char(' ') + info);
+}
+
+void NowPlayingWidget::copyInfo()
+{
+    QApplication::clipboard()->setText(track->text()+QLatin1Char(' ')+artist->text().replace(QString(" – "), QLatin1String(" ")));
+}
+
 void NowPlayingWidget::initColors()
 {
     ensurePolished();
@@ -449,5 +481,36 @@ void NowPlayingWidget::initColors()
     artist->setPalette(btn.palette());
     time->setPalette(btn.palette());
     slider->updateStyleSheet(track->palette().windowText().color());
-    ratingWidget->setColor(Utils::clampColor(track->palette().text().color()));
+    ratingWidget->setColor(Utils::clampColor(track->palette().buttonText().color()));
+    infoLabel->setPalette(btn.palette());
 }
+
+void NowPlayingWidget::resizeEvent(QResizeEvent *ev)
+{
+    QWidget::resizeEvent(ev);
+    controlWidgets();
+}
+
+void NowPlayingWidget::controlWidgets()
+{
+    bool rwEnabled=ratingWidget->property(constUserSettingProp).toBool();
+    bool infoEnabled=infoLabel->property(constUserSettingProp).toBool();
+
+    if ((!rwEnabled && !infoEnabled) || width()<Utils::scaleForDpi(280)) {
+        ratingWidget->setVisible(false);
+        infoLabel->setVisible(false);
+    } else if (rwEnabled && infoEnabled) {
+        if (width()>Utils::scaleForDpi(390)) {
+            ratingWidget->setVisible(true);
+            infoLabel->setVisible(true);
+        } else if (width()>Utils::scaleForDpi(280)) {
+            ratingWidget->setVisible(true);
+            infoLabel->setVisible(false);
+        }
+    } else {
+        infoLabel->setVisible(infoEnabled);
+        ratingWidget->setVisible(rwEnabled);
+    }
+}
+
+#include "moc_nowplayingwidget.cpp"
