@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -42,13 +42,14 @@ static QString viewTypeString(ItemView::Mode mode)
     case ItemView::Mode_List:         return QObject::tr("List");
     case ItemView::Mode_IconTop:      return QObject::tr("Grid");
     case ItemView::Mode_Table:        return QObject::tr("Table");
+    case ItemView::Mode_Categorized:  return QObject::tr("Categorized");
     }
 }
 
 SinglePageWidget::SinglePageWidget(QWidget *p)
     : QWidget(p)
     , btnFlags(0)
-    , refreshAction(0)
+    , refreshAction(nullptr)
 {
     QGridLayout *layout=new QGridLayout(this);
     view=new ItemView(this);
@@ -111,27 +112,27 @@ void SinglePageWidget::init(int flags, const QList<QWidget *> &leftXtra, const Q
 
     if (!left.isEmpty()) {
         QHBoxLayout *ll=new QHBoxLayout();
-        foreach (QWidget *b, left) {
+        for (QWidget *b: left) {
             ll->addWidget(b);
         }
         static_cast<QGridLayout *>(layout())->addItem(ll, 2, 0, 1, 1);
     }
     if (!right.isEmpty()) {
         QHBoxLayout *rl=new QHBoxLayout();
-        foreach (QWidget *b, right) {
+        for (QWidget *b: right) {
             rl->addWidget(b);
         }
         static_cast<QGridLayout *>(layout())->addItem(rl, 2, 4, 1, 1);
     }
 }
 
-void SinglePageWidget::addSelectionToPlaylist(const QString &name, int action, quint8 priorty, bool decreasePriority)
+void SinglePageWidget::addSelectionToPlaylist(const QString &name, int action, quint8 priority, bool decreasePriority)
 {
     // Always get tracks and playlists - this way error message can be shown. #902
     QStringList files=selectedFiles(true);
     if (!files.isEmpty()) {
         if (name.isEmpty()) {
-            emit add(files, action, priorty, decreasePriority);
+            emit add(files, action, priority, decreasePriority);
         } else {
             emit addSongsToPlaylist(name, files);
         }
@@ -158,7 +159,7 @@ QList<QAction *> SinglePageWidget::createActions(const QList<SinglePageWidget::M
 {
     QList<QAction *> actions;
     QActionGroup *group=new QActionGroup(parent);
-    foreach (const MenuItem &v, values) {
+    for (const MenuItem &v: values) {
         QAction *act=new QAction(v.first, parent);
         connect(act, SIGNAL(toggled(bool)), parent, slot);
         act->setActionGroup(group);
@@ -187,7 +188,7 @@ Action * SinglePageWidget::createMenuGroup(const QString &name, const QList<Sing
 QList<QAction *> SinglePageWidget::createViewActions(QList<ItemView::Mode> modes)
 {
     QList<QPair<QString, int> > vals;
-    foreach (ItemView::Mode m, modes) {
+    for (ItemView::Mode m: modes) {
         vals.append(MenuItem(viewTypeString(m), m));
     }
     return createActions(vals, view->viewMode(), this, SLOT(viewModeSelected()));
@@ -227,3 +228,5 @@ void SinglePageWidget::controlActions()
     }
     StdActions::self()->addRandomAlbumToPlayQueueAction->setVisible(false);
 }
+
+#include "moc_singlepagewidget.cpp"
