@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,8 +29,8 @@
 class SizeWidget : public QWidget
 {
 public:
-    explicit SizeWidget(QWidget *parent = 0);
-    void paintEvent(QPaintEvent *e);
+    explicit SizeWidget(QWidget *parent = nullptr);
+    void paintEvent(QPaintEvent *e) override;
 };
 
 #endif
