@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -35,7 +35,7 @@ public:
 
     SongDialog(QWidget *parent, const QString &name=QString(), const QSize &defSize=QSize())
         : Dialog(parent, name, defSize) { }
-    virtual ~SongDialog() { }
+    ~SongDialog() override { }
 
 protected:
     bool songsOk(const QList<Song> &songs, const QString &base, bool isMpd);
