@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,8 +29,8 @@
 class SpacerWidget : public QWidget
 {
 public:
-    SpacerWidget(QWidget *parent = 0);
-    virtual ~SpacerWidget() { }
+    SpacerWidget(QWidget *parent = nullptr);
+    ~SpacerWidget() override { }
 };
 
 #endif // SPACER_WIDGET_H
