@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -40,14 +40,14 @@ class StackedPageWidget : public QStackedWidget, public Page
 
 public:
     StackedPageWidget(QWidget *p);
-    virtual ~StackedPageWidget();
+    ~StackedPageWidget() override;
 
     void setView(int v);
-    void focusSearch();
+    void focusSearch() override;
     QStringList selectedFiles(bool allowPlaylists) const;
-    QList<Song> selectedSongs(bool allowPlaylists) const;
-    void addSelectionToPlaylist(const QString &name, int action, quint8 priorty, bool decreasePriority);
-    void removeItems();
+    QList<Song> selectedSongs(bool allowPlaylists) const override;
+    void addSelectionToPlaylist(const QString &name, int action, quint8 priority, bool decreasePriority) override;
+    void removeItems() override;
 
 Q_SIGNALS:
     void close();
