@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,8 +25,10 @@
 #include "support/squeezedtextlabel.h"
 #include "support/utils.h"
 #include "support/icon.h"
+#include "support/proxystyle.h"
 #include "gui/stdactions.h"
 #include "toolbutton.h"
+#include "groupedview.h"
 #ifdef Q_OS_MAC
 #include "support/osxstyle.h"
 #endif
@@ -34,22 +36,21 @@
 #include <QAction>
 #include <QImage>
 #include <QPixmap>
-#include <QGridLayout>
+#include <QHBoxLayout>
+#include <QVBoxLayout>
 #include <QEvent>
 #include <QMouseEvent>
 #include <QApplication>
 #include <QPainter>
 
-static int twHeight=-1;
-
 TitleWidget::TitleWidget(QWidget *p)
-    : QWidget(p)
+    : QListView(p)
     , pressed(false)
-    , underMouse(false)
-    , controls(0)
+    , controls(nullptr)
 {
-    QGridLayout *layout=new QGridLayout(this);
-    QVBoxLayout *textLayout=new QVBoxLayout(0);
+    setProperty(ProxyStyle::constModifyFrameProp, ProxyStyle::VF_Side|ProxyStyle::VF_Top);
+    QHBoxLayout *layout=new QHBoxLayout(this);
+    QVBoxLayout *textLayout=new QVBoxLayout(nullptr);
     image=new QLabel(this);
     mainText=new SqueezedTextLabel(this);
     subText=new SqueezedTextLabel(this);
@@ -68,46 +69,38 @@ TitleWidget::TitleWidget(QWidget *p)
     pal.setColor(QPalette::WindowText, col);
     subText->setPalette(pal);
     chevron->setFont(f);
-    int spacing=Utils::layoutSpacing(this);
     mainText->ensurePolished();
     subText->ensurePolished();
-    int size=mainText->sizeHint().height()+subText->sizeHint().height()+spacing;
-    if (size<72) {
-        size=Icon::stdSize(size);
-    }
-    int pad=Utils::scaleForDpi(6);
-    size=qMax(qMax(size, QFontMetrics(mainText->font()).height()+QFontMetrics(subText->font()).height()+spacing), Utils::scaleForDpi(40))+pad;
-    image->setFixedSize(size, size);
     setToolTip(tr("Click to go back"));
-    spacing=qMin(4, spacing-1);
-    layout->addItem(new QSpacerItem(spacing, spacing), 0, 0, 2, 1);
-    layout->addWidget(chevron, 0, 1, 2, 1);
-    layout->addWidget(image, 0, 2, 2, 1);
+    int spacing = Utils::layoutSpacing(this);
+    layout->addItem(new QSpacerItem(spacing, 1, QSizePolicy::Fixed, QSizePolicy::Fixed));
+    layout->addWidget(chevron);
+    layout->addItem(new QSpacerItem(spacing, 1, QSizePolicy::Fixed, QSizePolicy::Fixed));
+    layout->addWidget(image);
+    layout->addItem(new QSpacerItem(spacing, 1, QSizePolicy::Fixed, QSizePolicy::Fixed));
     textLayout->addWidget(mainText);
     textLayout->addWidget(subText);
-    layout->addItem(textLayout, 0, 3, 2, 1);
-    mainText->installEventFilter(this);
-    subText->installEventFilter(this);
-    image->installEventFilter(this);
-    installEventFilter(this);
-    setAttribute(Qt::WA_Hover);
+    layout->addItem(textLayout);
+    mainText->setAttribute(Qt::WA_TransparentForMouseEvents);
+    subText->setAttribute(Qt::WA_TransparentForMouseEvents);
+    image->setAttribute(Qt::WA_TransparentForMouseEvents);
+    viewport()->installEventFilter(this);
+    viewport()->setAttribute(Qt::WA_Hover);
     connect(Covers::self(), SIGNAL(cover(Song,QImage,QString)), this, SLOT(coverRetrieved(Song,QImage,QString)));
     connect(Covers::self(), SIGNAL(coverUpdated(Song,QImage,QString)), this, SLOT(coverRetrieved(Song,QImage,QString)));
     connect(Covers::self(), SIGNAL(artistImage(Song,QImage,QString)), this, SLOT(coverRetrieved(Song,QImage,QString)));
     layout->setMargin(0);
-    layout->setSpacing(spacing);
+    layout->setSpacing(2);
     textLayout->setMargin(0);
-    textLayout->setSpacing(spacing);
+    textLayout->setSpacing(2);
     mainText->setAlignment(Qt::AlignBottom);
     subText->setAlignment(Qt::AlignTop);
     image->setAlignment(Qt::AlignCenter);
     chevron->setAlignment(Qt::AlignCenter);
     chevron->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
-    if (-1==twHeight) {
-        ToolButton tb;
-        twHeight=qMax((tb.iconSize().height()*2), size);
-    }
-    setFixedHeight(twHeight);
+    image->setFixedSize(GroupedView::coverSize(), GroupedView::coverSize());
+    setFixedHeight(image->height()+((frameWidth()+2)*2));
+    setFocusPolicy(Qt::NoFocus);
 }
 
 void TitleWidget::update(const Song &sng, const QIcon &icon, const QString &text, const QString &sub, bool showControls)
@@ -130,10 +123,7 @@ void TitleWidget::update(const Song &sng, const QIcon &icon, const QString &text
             ToolButton *replace=new ToolButton(this);
             add->QAbstractButton::setIcon(StdActions::self()->appendToPlayQueueAction->icon());
             replace->QAbstractButton::setIcon(StdActions::self()->replacePlayQueueAction->icon());
-            int size=qMax(add->iconSize().height()+6, (height()/2)-1);
-            if (size>(height()-1)) {
-                size--;
-            }
+            int size=qMin(add->iconSize().height()+6, height()/2);
             add->setFixedSize(QSize(size, size));
             replace->setFixedSize(add->size());
             add->setToolTip(tr("Add All To Play Queue"));
@@ -142,7 +132,7 @@ void TitleWidget::update(const Song &sng, const QIcon &icon, const QString &text
             l->addWidget(add);
             connect(add, SIGNAL(clicked()), this, SIGNAL(addToPlayQueue()));
             connect(replace, SIGNAL(clicked()), this, SIGNAL(replacePlayQueue()));
-            static_cast<QGridLayout *>(layout())->addWidget(controls, 0, 4, 2, 1);
+            static_cast<QHBoxLayout *>(layout())->addWidget(controls);
         }
         controls->setVisible(true);
     }
@@ -172,60 +162,54 @@ void TitleWidget::update(const Song &sng, const QIcon &icon, const QString &text
 
 bool TitleWidget::eventFilter(QObject *o, QEvent *event)
 {
-    switch(event->type()) {
-    case QEvent::HoverEnter:
-        if (isEnabled() && o==this) {
-            /*
+    if (isEnabled()) {
+        switch(event->type()) {
+        case QEvent::HoverEnter: {
+            QPalette pal = qApp->palette();
             #ifdef Q_OS_MAC
-            setStyleSheet(QString("QLabel{color:%1;}").arg(OSXStyle::self()->viewPalette().highlight().color().name()));
+            QColor col(OSXStyle::self()->viewPalette().color(QPalette::Highlight));
             #else
-            setStyleSheet(QLatin1String("QLabel{color:palette(highlight);}"));
+            QColor col(pal.color(QPalette::Highlight));
             #endif
-            */
-            underMouse = true;
+            col.setAlphaF(0.2);
+            pal.setColor(QPalette::Base, col);
+            setPalette(pal);
+            break;
         }
-        break;
-    case QEvent::HoverLeave:
-        if (isEnabled() && o==this) {
-            //setStyleSheet(QString());
-            underMouse = false;
+        case QEvent::HoverLeave: {
+            QPalette pal = qApp->palette();
+            QColor col(pal.color(QPalette::Base));
+            pal.setColor(QPalette::Base, col);
+            setPalette(pal);
+            break;
         }
-        break;
-    case QEvent::MouseButtonPress:
-        if (Qt::LeftButton==static_cast<QMouseEvent *>(event)->button() && Qt::NoModifier==static_cast<QMouseEvent *>(event)->modifiers()) {
-            pressed=true;
+        case QEvent::MouseButtonPress:
+            if (Qt::LeftButton==static_cast<QMouseEvent *>(event)->button() && Qt::NoModifier==static_cast<QMouseEvent *>(event)->modifiers()) {
+                QPalette pal = qApp->palette();
+                #ifdef Q_OS_MAC
+                QColor col(OSXStyle::self()->viewPalette().color(QPalette::Highlight));
+                #else
+                QColor col(pal.color(QPalette::Highlight));
+                #endif
+                col.setAlphaF(0.5);
+                pal.setColor(QPalette::Base, col);
+                setPalette(pal);
+                pressed=true;
+            }
+            break;
+        case QEvent::MouseButtonRelease:
+            if (pressed && Qt::LeftButton==static_cast<QMouseEvent *>(event)->button() && !QApplication::overrideCursor()) {
+                actions().first()->trigger();
+            }
+            pressed=false;
+            break;
+        default:
+            break;
         }
-        break;
-    case QEvent::MouseButtonRelease:
-        if (pressed && Qt::LeftButton==static_cast<QMouseEvent *>(event)->button() && !QApplication::overrideCursor()) {
-            actions().first()->trigger();
-        }
-        pressed=false;
-        break;
-    default:
-        break;
     }
     return QWidget::eventFilter(o, event);
 }
 
-void TitleWidget::paintEvent(QPaintEvent *e)
-{
-    if (pressed || underMouse) {
-        QPainter p(this);
-        #ifdef Q_OS_MAC
-        QColor col = OSXStyle::self()->viewPalette().highlight().color();
-        #else
-        QColor col = palette().highlight().color();
-        #endif
-
-        QPainterPath path = Utils::buildPath(rect().adjusted(1, 1, -1, -1), 2.0);
-        p.setRenderHint(QPainter::Antialiasing);
-        col.setAlphaF(pressed ? 0.5 : 0.2);
-        p.fillPath(path, col);
-    }
-    QWidget::paintEvent(e);
-}
-
 void TitleWidget::coverRetrieved(const Song &s, const QImage &img, const QString &file)
 {
     Q_UNUSED(file);
@@ -251,7 +235,9 @@ void TitleWidget::coverRetrieved(const Song &s, const QImage &img, const QString
 void TitleWidget::setImage(const QImage &img)
 {
     double dpr=DEVICE_PIXEL_RATIO();
-    QPixmap pix=QPixmap::fromImage(img.scaled((image->width()-6)*dpr, (image->height()-6)*dpr, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+    QPixmap pix=QPixmap::fromImage(img.scaled(image->width()*dpr, image->height()*dpr, Qt::KeepAspectRatio, Qt::SmoothTransformation));
     pix.setDevicePixelRatio(dpr);
     image->setPixmap(pix);
 }
+
+#include "moc_titlewidget.cpp"
