@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -32,11 +32,11 @@ class Icon;
 class WizardPage : public QWizardPage
 {
 public:
-    WizardPage(QWidget *parent = 0) : QWizardPage(parent) { }
-    virtual ~WizardPage() { }
+    WizardPage(QWidget *parent = nullptr) : QWizardPage(parent) { }
+    ~WizardPage() override { }
 
     void setBackground(const Icon &i);
-    void paintEvent(QPaintEvent *e);
+    void paintEvent(QPaintEvent *e) override;
 
 private:
     QPixmap pix;
